{ *************************************************************************** }
{                                                                             }
{ Kylix and Delphi Cross-Platform Visual Component Library                    }
{                                                                             }
{ Copyright (c) 1997, 2001 Borland Software Corporation                       }
{                                                                             }
{ This file may be distributed and/or modified under the terms of the GNU     }
{ General Public License (GPL) version 2 as published by the Free Software    }
{ Foundation and appearing at http://www.borland.com/kylix/gpl.html.          }
{                                                                             }
{ Licensees holding a valid Borland No-Nonsense License for this Software may }
{ use this file in accordance with such license, which appears in the file    }
{ license.txt that came with this Software.                                   }
{                                                                             }
{ *************************************************************************** }


unit DBLocalS;

{$R-,T-,H+,X+}

interface

{$IFDEF MSWINDOWS}
uses Windows, SysUtils, Variants, Classes, Db, DBCommon, Midas, SqlTimSt, DBClient, DBLocal, Provider, SqlExpr;
{$ENDIF}
{$IFDEF LINUX}
uses Libc, SysUtils, Variants, Classes, DB, DBCommon, Midas, SqlTimSt, DBClient, DBLocal, Provider, SqlExpr;
{$ENDIF}

type

{ TSQLClientDataSet }

  TSQLClientDataSet = class(TCustomCachedDataSet)
  private
    FLocalParams: TParams;
    FDataSet: TSQLDataSet;
    FLocalCommandText: string;
    FSQLConnection: TSQLConnection;
    function GetCommandType: TSQLCommandType;
    function GetConnection: TSQLConnection;
    function GetConnectionName: string;
    function GetMasterFields: string;
    function GetObjectView: Boolean;
    procedure SetCommandType(const Value: TSQLCommandType);
    procedure SetConnection(const Value: TSQLConnection);
    procedure SetConnectionName(const Value: string);
    procedure SetInternalCommandText(Value: string);
    procedure SetLocalParams;
    procedure SetMasterFields(const Value: string);
    procedure SetObjectView(const Value: Boolean);
  protected
    procedure Loaded; override;
    function GetCommandText: string; override;
    procedure SetActive(Value: Boolean); override;
    procedure SetCommandText(Value: string); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure CloneCursor(Source: TCustomClientDataSet; Reset: Boolean;
      KeepSettings: Boolean = False); override;
    function GetQuoteChar: string;
  published
    property CommandText: string read GetCommandText write SetCommandText;
    property CommandType: TSQLCommandType read GetCommandType write SetCommandType default ctQuery;
    property DBConnection: TSQLConnection read GetConnection write SetConnection;
    property ConnectionName: string read GetConnectionName write SetConnectionName;
    property MasterFields read GetMasterFields write SetMasterFields;
    property ObjectView read GetObjectView write SetObjectView default False;
  end;

implementation

uses MidConst, SqlConst;

const
  ProvDataSetName = 'DataSet1';


{ TSQLClientDataSet }

constructor TSQLClientDataSet.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDataSet := TSQLDataSet.Create(nil);
  FDataSet.Name := Self.Name + ProvDataSetName;
  Provider.DataSet := FDataSet;
  SqlDBType := typeDBX;
  FSQLConnection := nil;
  FLocalParams := TParams.Create;
end;

destructor TSQLClientDataSet.Destroy;
begin
  inherited Destroy;
  if Assigned(FDataSet) then
    FreeAndNil(FDataSet);
  if Assigned(FSQLConnection) then
    FreeAndNil(FSQLConnection);
  if Assigned(FLocalParams) then
    FreeAndNil(FLocalParams);
end;

function TSQLClientDataSet.GetCommandType: TSQLCommandType;
begin
  Result := FDataSet.CommandType;
end;

procedure TSQLClientDataSet.SetCommandType(const Value: TSQLCommandType);
begin
  if Value <> CommandType then
  begin
    FDataSet.CommandType := Value;
    if csLoading in ComponentState then
    begin
      if FLocalCommandText <> FDataSet.CommandText then
        FDataSet.CommandText := FLocalCommandText;
    end else if csDesigning in ComponentState then
      CommandText := '';
  end;
end;

function TSQLClientDataSet.GetCommandText: string;
begin
  Result := FLocalCommandText;
end;

procedure TSQLClientDataSet.SetInternalCommandText(Value: string);
begin
  if Assigned(Provider.DataSet) then
  begin
    if Assigned(TSQLDataSet(Provider.DataSet).SqlConnection) and (Value <> TSQLDataSet(Provider.DataSet).CommandText) then
    begin
      TSQLDataSet(Provider.DataSet).CommandType := TSQLCommandType(CommandType);
      TSQLDataSet(Provider.DataSet).CommandText := Value;
    end;
  end else
    DataBaseError(SNoDataProvider);
end;

procedure TSQLClientDataSet.SetCommandText(Value: string);
begin
  if Value <> CommandText then
  begin
    CheckInactive;
    FLocalCommandText := Value;
    inherited SetCommandText(Value);
    SetInternalCommandText(Value);
  end;
end;

function TSQLClientDataSet.GetConnection: TSQLConnection;
begin
  if Assigned(FSQLConnection) then Result := FSQLConnection
  else Result := FDataSet.SQLConnection as TSQLConnection;
end;

procedure TSQLClientDataSet.SetConnection(const Value: TSQLConnection);
begin
  if FDataSet.SQLConnection <> Value then
  begin
    CheckInactive;
    if Assigned(Value) then
      SetConnectionName('');
    FDataSet.SQLConnection := Value;
  end;
end;

function TSQLClientDataSet.GetConnectionName: string;
begin
  if Assigned(FSQLConnection) then
    Result := FSQLConnection.ConnectionName
  else
    Result := '';
end;

procedure TSQLClientDataSet.SetConnectionName(const Value: string);
begin
  if Value <> '' then
  begin
    CheckInactive;
    SetConnection(nil);
    if Assigned(FSQLConnection) then
    begin
      FSQLConnection.Close;
      if Value = FSQLConnection.ConnectionName then 
        exit
      else
        FreeAndNil(FSQLConnection);
    end;
    FSQLConnection := TSQLConnection.Create(nil);
    FSQLConnection.LoadParamsOnConnect := True;
    FSQLConnection.ConnectionName := Value;
    FDataSet.SQLConnection := FSQLConnection;
    if FLocalCommandText <> '' then
      SetInternalCommandText(FLocalCommandText);
  end else
  begin
    if Assigned(FSQLConnection) then
    begin
      FDataSet.SQLConnection := nil;
      FreeAndNil(FSQLConnection);
    end;
  end;
end;

function TSQLClientDataSet.GetQuoteChar: string;
begin
  Result := FDataSet.GetQuoteChar;
end;

function TSQLClientDataSet.GetMasterFields: string;
begin
  Result := inherited MasterFields;
end;

procedure TSQLClientDataSet.SetMasterFields(const Value: string);
begin
  inherited MasterFields := Value;
  if Value <> '' then
    IndexFieldNames := Value;
end;

procedure TSQLClientDataSet.SetLocalParams;

  procedure SetParamInfo;
  var
    I: Integer;
    List: TStrings;
  begin
    List := TStringList.Create;
    try
      List.CommaText := MasterFields;
      for I := 0 to List.Count -1 do
      begin
        if Assigned(MasterSource) and Assigned(MasterSource.DataSet) then
          FLocalParams[I].AssignField(MasterSource.DataSet.FieldByName(List[I]))
      end;
    finally
      List.Free;
    end;
  end;

  function GenerateParamSQLForDetail(CommandText, MasterFields: string): string;
  const
    SWhere = ' where ';     { do not localize }
    SAnd = ' and ';         { do not localize }
  var
    I: Integer;
    List: TStrings;
  begin
    List := TStringList.Create;
    try
      List.CommaText := MasterFields;
      for I := 0 to List.Count -1 do
      begin
        if I > 0 then Result := Result + SAnd;
        Result := Result + format('%s = :%s', [List.Strings[I], List.Strings[I]]); 
      end;
      if pos(SWhere, CommandText) > 0 then
        Result := SAnd + Result
      else
        Result := SWhere + Result
    finally
      List.Free;
    end;
  end;

var
  SaveCommand: string;
begin
  if (MasterFields <> '') and Assigned(MasterSource) and Assigned(MasterSource.DataSet) then
  begin
    FLocalParams.Clear;
    SaveCommand := CommandText + GenerateParamSQLForDetail(CommandText, MasterFields);
    FLocalParams.ParseSQL(SaveCommand, True);
    SetParamInfo;
  end;
end;

procedure TSQLClientDataSet.SetActive(Value: Boolean);
begin
  if Value then
  begin
    FDataSet.ObjectView := ObjectView;
    if MasterFields <> '' then
    begin
      SetLocalParams;
      Params := FLocalParams;
    end else
      FDataSet.Params := Params;
  end;
  inherited SetActive(Value);
end;

procedure TSQLClientDataSet.SetObjectView(const Value: Boolean);
begin
  inherited ObjectView := Value;
  FDataSet.ObjectView := Value;
end;

function TSQLClientDataSet.GetObjectView: Boolean;
begin
  Result := inherited ObjectView; 
end;

procedure TSQLClientDataSet.Loaded; 
begin
  inherited Loaded;
  SetInternalCommandText(FLocalCommandText);
  FDataSet.ObjectView := ObjectView;
end;

procedure TSQLClientDataSet.CloneCursor(Source: TCustomClientDataSet; Reset: Boolean;
   KeepSettings: Boolean = False); 
begin
  if not (Source is TSQLClientDataSet) then
    DatabaseError(SInvalidClone);
  Provider.DataSet := TSQLClientDataSet(Source).Provider.DataSet;
  DBConnection := TSQLClientDataSet(Source).DBConnection;
  CommandText := TSQLClientDataSet(Source).CommandText;
  inherited CloneCursor(Source, Reset, KeepSettings);
end;

end.
