
/****************************************************************************

dbExpress - Cross-Platform, database independent data access interface

Copyright (c) 2000, 2001 Borland Software Corporation

This file may be distributed and/or modified under the terms of the GNU
General Public License (GPL) version 2 as published by the Free Software

Software Foundation and appearing at
http://www.borland.com/kylix/gpl.html.

Licensees holding a valid Borland No-Nonsense License for this Software
may use this file in accordance with such license, which appears in the file
license.txt that came with this Software.

****************************************************************************/

#include <dbxpress.h>
#include <sqlx.h>

#define D400 146097L
#define D100 36524L
#define D4 1461
#define D1 365
#define   MILSECSPERMIN      60000L
#define   MILSECSPERHOUR   3600000L
#define   MILSECSPERDAY   86400000L

//const static  far int   iDayTable[][13] =
int   iDayTable[][13] =
               { 
                  { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 },
                  { 0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 } 
               };

/*
** Local Prototypes.
*/

INT16 LocalDayOfYear( INT16 iMon, INT16 iDay, INT16 iYear );
void LocalMonthDay( UINT16 iN, 
                    UINT16 *piMon, 
                    UINT16 *piDay, 
                    UINT16 iYr );
BOOL LocalIsLeap( INT16 iYr );
BOOL LsValidDate( UINT16 iMon, UINT16 iDay, UINT16 iYear);

BOOL LsValidBCDate( UINT16 iMon, UINT16 iDay, INT16 iYear)
{
    return iYear <= 0
             && iMon > 0
             && iMon < 13
             && iDay > 0
             && iDay <= iDayTable[1][iMon];
}

INT16 LocalDayOfBCYear( INT16 iMon, INT16 iDay, INT16 iYear )
{
   INT16   iSum = 0, i;
   for (i = 12; i > iMon; )
     iSum += iDayTable[1][i--];
   iSum += iDayTable[1][iMon] - iDay;

   return iSum + 1;
}

// in the years before 1/1/1 every year is a leap year, this is by
// our own internal definition, our logical dates need only
// to be comparable, no other operations ( such as date difference )
// are supported.
// Also, the year 0 is valid : the day before 1/1/1 is 12/31/0 
DBIDATE EncodeBCDate (
   UINT16     iMon,
   UINT16     iDay,
   INT16      iYear
)
{
   if ( ! LsValidBCDate ( iMon, iDay, iYear ) )
      return (DBIDATE) 0;
   return  - ((366L * -iYear) + LocalDayOfBCYear(iMon, iDay, iYear));
}

void  LocalBCMonthDay( 
   UINT16 iDays, 
   UINT16 *piMon, 
   UINT16 *piDay 
)
{
   INT16  i = 0, iDaysInMo = 0;

   for ( i = 12; iDays >= (iDaysInMo = iDayTable[1][i]); i--)
      iDays -= iDaysInMo;

   *piMon = i;
   *piDay = iDayTable[1][i] - iDays;  
}								   	  

VOID DecodeBCDate( 
   DBIDATE Date, 
   pUINT16 piMon, 
   pUINT16 piDay, 
   pINT16  piYear
)
{
   *piYear = (INT16)(++Date / 366L);    // decrement since encode added one
   LocalBCMonthDay( (UINT16) ( -Date % 366L), piMon, piDay);
   return;
}

/*
*****************************************************************************
** Exported Functions.
*****************************************************************************
*/


DBIDATE  LsDateEncode ( UINT16     iMon, 
                    UINT16     iDay,
                    INT16     iYear )
{
   INT16 iLastYear = iYear - 1;               

   if( iYear <= 0 )
      return EncodeBCDate( iMon, iDay, iYear );

   if ( ! LsValidDate ( iMon, iDay, iYear ) )
      return (DBIDATE) 0;

   return LocalDayOfYear( iMon, iDay, iYear ) + 
              365L*iLastYear + iLastYear/4 - iLastYear/100 + iLastYear/400;
}



void LsDateDecode( DBIDATE  Date, 
                   pUINT16 iMon, 
                   pUINT16 iDay, 
                   pINT16 iYear )
{
   INT32       mod400, mod100;
   UINT16      mod4, mod1;
   INT16       iYr;


   if ( ! Date )
   {
      *iMon = *iDay = *iYear = 0;
      return;
   }
   if( Date < 0 )
   {
      DecodeBCDate( Date, iMon, iDay, iYear);
      return;
   }

   mod400   = Date % D400;
   mod100   = mod400 % D100;
   mod4     = (UINT16) mod100 % D4;
   mod1     = (UINT16) mod4 % D1;

   iYr      = (UINT16) 1 + (Date/D400)*400 + (mod400/D100)*100 + 
                          (mod100/D4)*4 + (mod4/D1);

   if ( !mod1 ) 
   {                   /* test end of year condition */
     iYr     -= 1;
     mod1     = ( (!mod4) && (mod100 || !mod400) ) ? 366 : 365;
   }

   LocalMonthDay( mod1, iMon, iDay, iYr );

      
   *iYear = iYr;

}

/*
DBIDATE LsGetDate (VOID)
{
    OSDate      osDate;

    OsGetDate(&osDate);

    return LsDateEncode(osDate.month, osDate.day, osDate.year);
}
*/


/*
** Return 
**    1 if date is valid 
**    0 if Invalid
*/

BOOL LsValidDate( UINT16 iMon, UINT16 iDay, UINT16 iYear)
{
    return iYear > 0
             && iMon > 0
             && iMon < 13
             && iDay > 0
             && iDay <= iDayTable[LocalIsLeap(iYear)][iMon];
}

INT32  LsTimeEncode  (UINT16     iHour,              
                      UINT16     iMin,              
                      UINT16     iMilSec,
                      pTIME      pTime)          
{
   if( iHour > 23  || iMin > 59 || iMilSec > 59999L )
      return DBXERR_INVALIDTIME;

   *pTime = (iHour * 60 + iMin) * MILSECSPERMIN  + iMilSec;
   return( DBXERR_NONE );
}

void  LsTimeDecode  ( TIME time,
                      pUINT16  iHour,
                      pUINT16  iMin,
                      pUINT16  iMilSec )
{

   if( time >= MILSECSPERDAY )
   {
      *iHour   = 0;
      *iMin    = 0;
      *iMilSec = 0;
   }
   else
   {
      *iHour = (UINT16) ( time / MILSECSPERHOUR );
      *iMin  = (UINT16) ((time - ( (TIME)*iHour * MILSECSPERHOUR) ) / MILSECSPERMIN);
      *iMilSec = (UINT16 ) ( time  - ((TIME)*iHour * (TIME) MILSECSPERHOUR) 
                            - ((TIME)*iMin * (TIME) MILSECSPERMIN));
   }
}

/*
DBIResult DBIFN  EXPORT LsGetTime (
    pTIME       pTime
)
{
    OSTime      osTime;

    OsGetTime(&osTime);

    return LsTimeEncode(osTime.hour, osTime.min, osTime.milsec, pTime);
}

DBIResult  DBIFN EXPORT LsTimeStampEncode  ( 
   DBIDATE   Date,
   TIME   Time,
   pTIMESTAMP ptimeStamp
)
{
   if( Time > (TIME) MILSECSPERDAY )
      return DBXERR_INVALIDTIMESTAMP;

   *ptimeStamp =  
      (TIMESTAMP) Date * (TIMESTAMP) MILSECSPERDAY + (TIMESTAMP) Time ;

#ifdef OLDSTUFF
   // Note: both formulas are identical (except perhaps for round-off) -- bs --
   //       what is the reason for treating them so differently?       -- bs --
   //
   if( Date <  0 )
      *ptimeStamp =  
         (TIMESTAMP)(Date+1) * (TIMESTAMP) MILSECSPERDAY  - 
         ( (TIMESTAMP) MILSECSPERDAY - (TIMESTAMP) Time );
   else
      *ptimeStamp =  
         (TIMESTAMP) Date * (TIMESTAMP) MILSECSPERDAY + (TIMESTAMP) Time ;
#endif //OLDSTUFF -- bs -- 05/20/94

   return DBXERR_NONE ;
}
void DBIFN EXPORT LsTimeStampDecode ( 
   TIMESTAMP TimeStamp,
   pDBIDATE     Date,
   pTIME     Time 
)
{
   // Note: date truncates towards zero.  if there is a remainder,
   //       its value will be one too close to zero. -- bs --
   //
   *Date = (DBIDATE) (TimeStamp / MILSECSPERDAY);
   *Time = (TIME) (TimeStamp - ( *Date * MILSECSPERDAY ));
   if( *Time < (TIME)0 )
   {
      (*Date)--;
      (*Time) += MILSECSPERDAY;
   }

   // MAS 1/3/95:  Time is zero-based, that is, it runs from 
   // 0 through (MILSECSPERDAY - 1).  Due to roundoff error above,
   // we may get a time of MILSECSPERDAY, which is not valid.
   // This is corrected here.

   if (*Time >= MILSECSPERDAY)
   {
      *Time -= MILSECSPERDAY;
      (*Date)++;
   }

#ifdef OLDSTUFF
// if( TimeStamp < (TIMESTAMP) 0 )
// {
//    *Date = (DBIDATE) (TimeStamp / MILSECSPERDAY);
//    *Time = (TIME) ((TIMESTAMP) MILSECSPERDAY + (TimeStamp - 
//              ((TIMESTAMP) (*Date + 1) * (TIMESTAMP) MILSECSPERDAY)));
// }
// else
// {
//    *Date = (DBIDATE) (TimeStamp / MILSECSPERDAY);
//    *Time = (TIME) (TimeStamp - ( *Date * MILSECSPERDAY ));
// }
#endif //OLDSTUFF -- bs -- 05/20/94
}


DBIResult DBIFN EXPORT LsGetTimeStampEncode (
    pTIMESTAMP  ptimeStamp
)
{
    DBIDATE     lsDate = LsGetDate();
    TIME        lsTime;
    DBIResult   rc = LsGetTime(&lsTime);

    if (rc == DBXERR_NONE)
        rc = LsTimeStampEncode(lsDate, lsTime, ptimeStamp);

    return rc;                          // return status
}
*/


/*
*****************************************************************************
** Local Functions.
*****************************************************************************
*/


INT16 LocalDayOfYear( INT16 iMon, INT16 iDay, INT16 iYear )
{
   INT16   iSum, i;
   INT16   iLeapFlag = LocalIsLeap( iYear );

   for (i = 1, iSum = iDay; i < iMon;)
     iSum += iDayTable[iLeapFlag][i++];

   return iSum;
}


void LocalMonthDay( UINT16 iN, 
                               UINT16 *piMon, 
                               UINT16 *piDay, 
                               UINT16 iYr )
{
   INT16   i, iDaysInMo, iLeapFlag;

   iLeapFlag = LocalIsLeap(iYr);

   for (i = 1; iN > (iDaysInMo = iDayTable[iLeapFlag][i++]);)
      iN -= iDaysInMo;

   *piMon = --i;
   *piDay = iN;
}


/*
** Return 
**    1 if year is a leap year
**    0 otherwise
*/

BOOL LocalIsLeap( INT16 iYr )
{
    return (!(iYr % 4)) && (iYr % 100 || (!(iYr % 400)));
}

/*
DBIDATE   DBIFN LsFdateEncode( INT16 ff_fdate)
{
   typedef union { 
      INT16    idate;
      struct {
         unsigned int day   : 5;
         unsigned int mon   : 4;
         unsigned int yr    : 7;
      }   bdate;
   } FDATE; 

   FDATE fdate;
   fdate.idate = ff_fdate;
   return( LsDateEncode( 
            fdate.bdate.mon,
            fdate.bdate.day,
            fdate.bdate.yr + 1980));
}
TIME   DBIFN LsFtimeEncode( INT16 ff_ftime)
{
   typedef union {
      INT16  itime;
      struct _ftime { 
         unsigned int sec   : 5;
         unsigned int min   : 6;
         unsigned int hr    : 5;
      } btime; 
   }   FTIME;
   TIME time = 0;
   FTIME ftime;
   ftime.itime = ff_ftime;
   (VOID) LsTimeEncode(ftime.btime.hr, ftime.btime.min, 
         ftime.btime.sec * 2000, &time);
   return time;
}
*/
