
/****************************************************************************

dbExpress - Cross-Platform, database independent data access interface

Copyright (c) 2000, 2001 Borland Software Corporation

This file may be distributed and/or modified under the terms of the GNU
General Public License (GPL) version 2 as published by the Free Software

Software Foundation and appearing at
http://www.borland.com/kylix/gpl.html.

Licensees holding a valid Borland No-Nonsense License for this Software
may use this file in accordance with such license, which appears in the file
license.txt that came with this Software.

****************************************************************************/

#include "mysqlobj.h"

////////////////////////////////////////////////////////////////////////////////

MYSQLCommand::MYSQLCommand()
{

  //Initialize all private members

  MySQLResult = NULL;
  MySQLFields = NULL;
  MySQLRow    = NULL;
  pDataLength = NULL;

  pConn    = NULL;
  pszSQL   = NULL;
  pszError = NULL;
  iError   = 0;
  uColumns = 0;
  uParams  = 0;
  pCols    = NULL;
  pParamList   = NULL;
  uBoundParams = 0;
  ulBlobSize   = 0;
  iRefCount    = 0;
  uRecordBufferSize = 0;

}

////////////////////////////////////////////////////////////////////////////////

MYSQLCommand::~MYSQLCommand()
{
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::exec (
   ppSQLCursor ppCur
)
{
   SQLResult retcode = SQL_SUCCESS;
   MYSQL* pMySQL     = NULL;
   pCHAR  pParamSQL  = NULL;
   UINT32 lLen       = 0;

   retcode = this->getConnectionHandle( &pMySQL );

   if ( !retcode )
   {
      if ( this->uParams != 0 )
      {
         retcode = getParamQuery(&pParamSQL, &lLen); 
         //printf("SQL Query : %d, %s\n",lLen, pParamSQL);

         if ( !retcode ) 
         {
            sqlTrace(pParamSQL, 0);
            sqlTrace(szST_real_query, iST_real_query);
            retcode =  (*pmysql_real_query) ( pMySQL , pParamSQL, lLen);
         }

         if ( pParamSQL )
            delete[] pParamSQL;

      }
      else 
      {
         sqlTrace(pszSQL, 0);
         sqlTrace(szST_real_query, iST_real_query);
         retcode =  (*pmysql_real_query) ( pMySQL ,this->pszSQL, strlen(this->pszSQL) );

      }

   }

   if (!retcode)
   {
      sqlTrace(szST_field_count, iST_field_count);
      this->uColumns = mysql_field_count(pMySQL);

      if (this->uColumns)
      {
          *ppCur  = new MYSQLCursor(this);
          (*ppCur)->AddRef();
          retcode = this->describeColumns();

      }
   }

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::getParamQuery (
   ppCHAR  ppParamSQL,
   pUINT32 plLen
)
{
   SQLResult retcode  = SQL_SUCCESS;
   pCHAR pszNewSQL    = NULL;
   UINT32 ulParamLen  = 0;

   this->getParamsLength(&ulParamLen);
  
   pszNewSQL = new CHAR[strlen(pszSQL)+ulParamLen];
   assert(pszNewSQL);

   if ( pszNewSQL )
   {
      pCHAR pBegin = pszNewSQL;
      pCHAR pN     = pszNewSQL;

      memset(pszNewSQL,'\0', strlen(pszSQL)+ulParamLen );

      if (this->uParams)
      {
         pCHAR pStart = NULL, p = NULL;

         pStart = p = (pCHAR)pszSQL;

         for (UINT16 index = 0; index < this->uParams; index++)
         {
            p = strchr(p, '?');
            memcpy(pN, pStart, p-pStart);
            pN += p-pStart;

            //Concatenate the parameter data

            if ( (pParamList+index)->iInd == SQLX_NULL_DATA)
            {
               strcat(pN,"NULL");
               pN += 4;

            }
            else
            {
               *pN++ = '\'';

               sqlTrace(szST_escape_string, iST_escape_string);
               pN += (*pmysql_escape_string) ( pN, (pCHAR) (pParamList+index)->pData, (pParamList+index)->ulLength );

               *pN++ = '\'';
            }


            //Increment to the next char after '?'
            ++p;
            pStart = p;

         }//for

         //Concatenate the query after the last '?' if any.
         if (p)
         { 
            strncat(pN, p, strlen(p));
            pN += strlen(p);
         }

      }

      //*plLen      = strlen(pszSQL) + ulParamLen;
      *plLen      = pN - pBegin ;
      *ppParamSQL = pszNewSQL;

   }
   else
      retcode = DBXERR_NOMEMORY;

   return retcode;

}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::getParamsLength (
   pUINT32 pulLength
)
{
   SQLResult retcode = SQL_SUCCESS;
   
   *pulLength = 0;

   for ( UINT16 uParamNumber = 0; uParamNumber < this->uParams; uParamNumber++ )
   {
      pParameter pParam = &this->pParamList[uParamNumber];

      if ( pParam->iInd == SQLX_NULL_DATA )
      {
         *pulLength +=  5;  // space for "NULL" 
      }
      else
         *pulLength += 2 * pParam->ulLength;  // Extra space for escape character
 
   }//for
   
   *pulLength = *pulLength + 1; // for NULL terminator
   
   return retcode;
   
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::getRowsAffected (
   pINT32 plRows
)
{

   SQLResult retcode = SQL_SUCCESS;
   MYSQL* pMySQL     = NULL;
   *plRows           = 0;

   this->getConnectionHandle( &pMySQL );

   sqlTrace(szST_affected_rows, iST_affected_rows);
   *plRows = mysql_affected_rows ( pMySQL );
 
   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::setOption (
   eSQLCommandOption eCommOption,
   INT32             lValue )
{
   SQLResult retcode = SQL_SUCCESS;

   switch (eCommOption)
   {
      case eCommBlobSize:
         this->ulBlobSize = lValue;
         break;

      default:
         retcode = DBXERR_NOTSUPPORTED;
         break;
   }

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::getOption (
   eSQLCommandOption eCommOption,
   pINT32           plValue,
   INT16            iMaxLen,
   pINT16           piLen )
{
   SQLResult retcode = SQL_SUCCESS;

   switch (eCommOption)
   {

      default:
         retcode = DBXERR_NOTSUPPORTED;
         break;
   }

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::executeImmediate (pCHAR pszSQL, ppSQLCursor ppCur)
{
   SQLResult retcode = SQL_SUCCESS;
   MYSQL* pMySQL     = NULL;

   if (ppCur)
      *ppCur = NULL;

   this->getConnectionHandle( &pMySQL );

   sqlTrace(pszSQL, 0);
   sqlTrace(szST_real_query, iST_real_query);
   retcode =  (*pmysql_real_query) ( pMySQL , pszSQL, strlen(pszSQL) );

   if (!retcode)
   {
      sqlTrace(szST_field_count, iST_field_count);
      this->uColumns = mysql_field_count(pMySQL);

      if (this->uColumns)
      {
          *ppCur  = new MYSQLCursor(this);
          (*ppCur)->AddRef();
          retcode = this->describeColumns();
      }

   }

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::numResultColumns (
   pUINT16    puColumns )
{
   SQLResult retcode = SQL_SUCCESS;

   *puColumns = this->uColumns;

   return retcode;
}


////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::fetch ()
{
   SQLResult retcode = SQL_SUCCESS;

   sqlTrace(szST_fetch_row, iST_fetch_row);
   this->MySQLRow = (*pmysql_fetch_row) ( this->MySQLResult );

   if ( this->MySQLRow != NULL )
   {
      sqlTrace(szST_fetch_lengths, iST_fetch_lengths);
      this->pDataLength = (*pmysql_fetch_lengths) ( this->MySQLResult );
   }
   else
      retcode = DBXERR_EOF;

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::extendedFetch (
   pUINT16     puRows )
{
   return DBXERR_NOTSUPPORTED;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::close ()
{
   SQLResult retcode  = SQL_SUCCESS;
   UINT32 index       = 0;
    
   //Free Columns
   if ( this->pCols )
   {
      for ( index = 0; index < this->uColumns; index++ )
      {
         ::free((pCols+index)->pszColName );

         if ( (pCols+index)->pszTblName )
            ::free((pCols+index)->pszTblName );
      }

      ::free(this->pCols);
      this->pCols = NULL;
   }


   //Free Parameter buffer
   if ( this->pParamList )
   {
      for ( index = 0; index < this->uParams; index++ )
      {
         if ( (pParamList+index)->pData && (pParamList+index)->iInd != -1 )
         {
            delete [] (pParamList+index)->pData;
            (pParamList+index)->pData = NULL;
         }
      }

   }


   //Free result
   if ( this->MySQLResult )
   {
      sqlTrace(szST_free_result, iST_free_result);
      (*pmysql_free_result) ( this->MySQLResult );
   }

   //Reset members
   this->uRecordBufferSize = 0;
   this->uColumns          = 0;
   this->uBoundParams      = 0;

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

UINT32 MYSQLCommand::Release ()
{
   UINT32 ulCount = 0;

   iRefCount--;

   ulCount = iRefCount;
   if ( iRefCount == 0 )
   {
      //Free SQL
      if ( this->pszSQL )
         delete [] this->pszSQL;

      //Free Parameters
      if ( this->pParamList )
         delete [](this->pParamList);

      //delete this object
      delete this;
   }

   return ulCount;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::getErrorMessageLen (
   pUINT16 puErrorLen )
{
   SQLResult retcode = SQL_SUCCESS;
   MYSQL* pMySQL     = NULL;
  
   this->getConnectionHandle( &pMySQL );

   sqlTrace(szST_error, iST_error);  
   this->pszError = mysql_error( pMySQL );

   *puErrorLen    = strlen( this->pszError);

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::getErrorMessage (
   pBYTE pszErrorMessage )
{
  SQLResult retcode = SQL_SUCCESS;

  strcpy((pCHAR)pszErrorMessage,this->pszError);

  return retcode;
}

////////////////////////////////////////////////////////////////////////////////

VOID MYSQLCommand::setConnection(pMYSQLConnection pConn)
{
   this->pConn = pConn; 
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::getColumn ( UINT16 uColNumber, 
   ppColumn ppCol
)
{
   SQLResult retcode  = SQL_SUCCESS;

   if  ( uColNumber > 0 && uColNumber <= this->uColumns )
   {
      uColNumber --;
      *ppCol = (pCols + uColNumber);
   }
   else
      retcode = DBXERR_OUTOFRANGE;

   return retcode;

}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::getRow ( MYSQL_ROW* pRow )
{
   SQLResult retcode  = SQL_SUCCESS;
    
   if ( this->MySQLRow )
   {
      *pRow = this->MySQLRow;
   }
   else
      retcode = DBXERR_INVALIDHNDL;
  
   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::getRowDataLength ( ppUINT32 pDataLen )
{
   SQLResult retcode  = SQL_SUCCESS;
    
   if ( this->pDataLength )
   {
      *pDataLen = this->pDataLength;
   }
   else
      retcode = DBXERR_INVALIDHNDL;
  
   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::getConnectionHandle ( MYSQL** ppConnection )
{
   SQLResult retcode  = SQL_SUCCESS;
    
   if ( this->pConn )
   {
      retcode = this->pConn->getConnectionHandle( ppConnection );
   }
   else
      retcode = DBXERR_INVALIDHNDL;
  
   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::setLogicalTypeInfo ( pColumn pCol )
{
   SQLResult retcode = SQL_SUCCESS;

   if ( pCol )
   {
      pCol->iLogPrecision = pCol->iPrecision;
      pCol->iLogScale     = pCol->iScale;
      pCol->uLogLength    = pCol->uLength;

      switch ( pCol->uPhyType )
      {
         case FIELD_TYPE_SHORT:
            pCol->uLogType = fldINT16;
            pCol->iLogPrecision = sizeof(INT16); 
            pCol->uLogLength    = sizeof(INT16);
            pCol->uLogSubType = 0;
            break;

         case FIELD_TYPE_TINY: // 1 Byte
         case FIELD_TYPE_YEAR:
            pCol->uLogType = fldINT16;
            pCol->iLogPrecision = sizeof(INT16); 
            pCol->uLogLength    = sizeof(INT16);
            pCol->uLogSubType = 0;
            break;

         case FIELD_TYPE_LONG:
            pCol->uLogType = fldINT32;
            pCol->iLogPrecision = sizeof(INT32); 
            pCol->uLogLength    = sizeof(INT32);
            pCol->uLogSubType = 0;
            break;

         case FIELD_TYPE_INT24: // 3 Bytes
            pCol->uLogType = fldINT32;
            pCol->iLogPrecision = sizeof(INT32); 
            pCol->uLogLength    = sizeof(INT32);
            pCol->uLogSubType = 0;
            break;

         case FIELD_TYPE_FLOAT:
            pCol->uLogType = fldFLOAT;
            pCol->iLogPrecision = 4; 
            pCol->uLogLength    = sizeof(DFLOAT);
            pCol->uLogSubType = 0;
            break;

         case FIELD_TYPE_DOUBLE:
            pCol->uLogType = fldFLOAT;
            pCol->iLogPrecision = sizeof(DFLOAT); 
            pCol->uLogLength    = sizeof(DFLOAT);
            pCol->uLogSubType = 0;
            break;

         case FIELD_TYPE_SET:
         case FIELD_TYPE_ENUM:
         case FIELD_TYPE_STRING:
         case FIELD_TYPE_VAR_STRING:
         
            if ( pCol->uFlags & BINARY_FLAG )
            {
               pCol->uLogType = fldVARBYTES;
               pCol->uLogSubType = 0;
               pCol->uLogLength  = pCol->uLength + sizeof(UINT32);    //size prefixed
               pCol->uLength     = pCol->uLogLength ;
            }
            else
            {
               pCol->uLogType = fldZSTRING;

               if ( pCol->uPhyType == FIELD_TYPE_STRING )
                  pCol->uLogSubType = fldstFIXED;
               else
                  pCol->uLogSubType = 0;

               pCol->uLogLength  = pCol->uLength + 1;    //Null terminator
               pCol->uLength     = pCol->uLogLength ;
            }
            break;


         case FIELD_TYPE_TIMESTAMP:
         case FIELD_TYPE_DATETIME:

            pCol->uLogType = fldDATETIME;
            pCol->uLogSubType = 0;
            pCol->uLogLength  = sizeof(SQLLTIMESTAMP);
            break;

         case FIELD_TYPE_TIME:

            pCol->uLogType = fldTIME;
            pCol->uLogSubType = 0;
            pCol->uLogLength  = sizeof(INT32);
            break;

         case FIELD_TYPE_DATE:

            pCol->uLogType = fldDATE;
            pCol->uLogSubType = 0;
            pCol->uLogLength  = sizeof(INT32);
            break;

         case FIELD_TYPE_TINY_BLOB:
         case FIELD_TYPE_MEDIUM_BLOB:
         case FIELD_TYPE_LONG_BLOB:
         case FIELD_TYPE_BLOB:

            pCol->uLogType = fldBLOB;
            pCol->iLogPrecision = 1;

            if ( pCol->uFlags & BINARY_FLAG )
               pCol->uLogSubType = fldstBINARY;
            else
               pCol->uLogSubType = fldstMEMO;

            break;


         case FIELD_TYPE_LONGLONG:

            pCol->uLogType = fldBCD;
            pCol->uLogLength  = sizeof(FMTBcd);
            pCol->uLogSubType = 0;
            break;

         case FIELD_TYPE_DECIMAL:

            pCol->uLogType = fldBCD;
            pCol->uLogLength  = sizeof(FMTBcd);
            pCol->uLogSubType = 0;
            break;

         default:
            retcode = DBXERR_INVALIDFLDTYPE;           
            break;
      }

   }

   return retcode;

}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::describeColumns (
)
{
   SQLResult retcode    = SQL_SUCCESS;
   UINT16 index         = 0;
   MYSQL* pMySQL        = NULL;

   //Describe columns
   if ( this->uColumns != 0 )
   {
      pCols  = (pColumn) calloc (sizeof(Column), this->uColumns);
      
      if ( !pCols )
         retcode = DBXERR_NOMEMORY;
     
      if ( !retcode )
      { 
         this->getConnectionHandle( &pMySQL );

         sqlTrace(szST_use_result, iST_use_result);
         this->MySQLResult = (*pmysql_use_result) ( pMySQL );

         sqlTrace(szST_fetch_field, iST_fetch_field);
         this->MySQLFields = (*pmysql_fetch_field) ( this->MySQLResult );
      }


      for ( index = 0; !retcode && index < this->uColumns; index++ )
      {

         (pCols+index)->uColNum       = index;
         (pCols+index)->pszColName    = (pCHAR) malloc(strlen(MySQLFields[index].name)+1);

         if ( (pCols+index)->pszColName )
            strcpy((pCols+index)->pszColName, MySQLFields[index].name);
         else
            retcode = DBXERR_NOMEMORY;

         if ( !retcode &&  MySQLFields[index].table )
         {
            (pCols+index)->pszTblName = (pCHAR) malloc(strlen(MySQLFields[index].table)+1);

            if ( (pCols+index)->pszTblName )
               strcpy((pCols+index)->pszTblName, MySQLFields[index].table);
            else
               retcode = DBXERR_NOMEMORY;

          }


          if ( !retcode )
          {
             (pCols+index)->iPrecision   = MySQLFields[index].length;
             (pCols+index)->iScale       = MySQLFields[index].decimals;
             (pCols+index)->uLength      = MySQLFields[index].length;
             (pCols+index)->uPhyType     = MySQLFields[index].type;
             (pCols+index)->uFlags       = MySQLFields[index].flags;
             (pCols+index)->uOffset      = uRecordBufferSize;

             //Set the logical data type
             retcode = this->setLogicalTypeInfo ( (pCols+index) );

             (pCols+index)->uNullOffset  = uRecordBufferSize + 
                                           (pCols+index)->uLength;
             uRecordBufferSize += (pCols+index)->uLength + sizeof(INT32);

          }

       }//for

   }

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::getNativeSQL (
    pBYTE pszSQL,
    ppBYTE ppszNativeSQL
)
{
   SQLResult retcode  = SQL_SUCCESS;
   pCHAR pszNativeSQL = NULL;

   if ( !retcode )
   { 
      pszNativeSQL = new CHAR[strlen((pCHAR)pszSQL) + 1];
      assert(pszNativeSQL);

      if ( pszNativeSQL )
      {
         strcpy(pszNativeSQL, (pCHAR)pszSQL);
         *ppszNativeSQL = (pBYTE)pszNativeSQL;
      }
      else
         retcode = DBXERR_NOMEMORY;
   }

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::describeParameters (
)
{
   SQLResult retcode = SQL_SUCCESS;

   // allocate parameter list

   this->pParamList  = new Parameter[this->uParams];
   assert(this->pParamList);

   if ( this->pParamList )
      memset(this->pParamList, 0x0, sizeof(Parameter) * this->uParams);
   else
      retcode = DBXERR_NOMEMORY;

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::prepare (
   pCHAR  pszSQL,
   UINT16 uParamCount )
{
   SQLResult retcode = SQL_SUCCESS;
   pBYTE     pszNativeSQL = NULL;

   this->uParams = uParamCount;
   retcode = getNativeSQL((pBYTE)pszSQL, &pszNativeSQL);

   if ( !retcode )
   {
      this->pszSQL = new CHAR[ strlen( (pCHAR) pszNativeSQL ) + 1] ;
      if ( this->pszSQL )
         strcpy(this->pszSQL, (pCHAR)pszNativeSQL);
      else
         retcode = DBXERR_NOMEMORY;

      if ( !retcode && this->uParams )
         retcode = this->describeParameters();
  
      if ( pszNativeSQL )
         delete []pszNativeSQL;
   }

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::putString (
   pParameter pParam, 
   pVOID      pData, 
   UINT32     ulLength
)
{
   SQLResult retcode = SQL_SUCCESS;
   UINT32 uLen = 0;  

   uLen = ulLength < strlen((pCHAR) pData) ? strlen((pCHAR) pData) : ulLength;
 
   pParam->pData = new BYTE[uLen];
   assert(pParam->pData);
   strncpy((pCHAR)pParam->pData, (pCHAR)pData, strlen((pCHAR)pData));
   pParam->ulLength = strlen((pCHAR)pData);

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::putBytes (
   pParameter pParam, 
   pVOID      pData, 
   UINT32     ulLength
)
{
   SQLResult retcode = SQL_SUCCESS;

   pParam->pData = new BYTE[ulLength];
   assert(pParam->pData);
   memcpy((pCHAR)pParam->pData, (pCHAR)pData, ulLength);
   pParam->ulLength = ulLength;

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::putShort (
   pParameter pParam, 
   pVOID      pData, 
   UINT32     ulLength
)
{
   SQLResult retcode = SQL_SUCCESS;
   CHAR   szTemp[50] = " ";

   sprintf(szTemp,"%d", *(pINT16)pData );
   
   pParam->pData = new BYTE[strlen(szTemp)];
   assert(pParam->pData);

   strncpy((pCHAR) pParam->pData, szTemp, strlen(szTemp) );

   pParam->ulLength = strlen(szTemp);

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::putLong (
   pParameter pParam, 
   pVOID      pData, 
   UINT32     ulLength
)
{
   SQLResult retcode = SQL_SUCCESS;
   CHAR   szTemp[50] = " ";

   sprintf(szTemp,"%d", *(pINT32)pData );
   
   pParam->pData = new BYTE[strlen(szTemp)];
   assert(pParam->pData);

   strncpy((pCHAR) pParam->pData, szTemp, strlen(szTemp) );

   pParam->ulLength = strlen(szTemp);

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::putDouble (
   pParameter pParam, 
   pVOID      pData, 
   UINT32     ulLength
)
{
   SQLResult retcode = SQL_SUCCESS;
   CHAR   szTemp[50] = " ";

   switch ( ulLength )
   {
      case 4:
         sprintf(szTemp,"%f", *(float*)pData );
         break;

      case 8:
         sprintf(szTemp,"%f", *(pDFLOAT)pData );
         break;

      default:
         retcode = DBXERR_INVALIDLEN;
         break;
   }

   if ( !retcode )
   {
      pParam->pData = new BYTE[strlen(szTemp)];
      assert(pParam->pData);

      strncpy((pCHAR) pParam->pData, szTemp, strlen(szTemp) );

      pParam->ulLength = strlen(szTemp);

   }

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::putTimeStamp (
   pParameter pParam, 
   pVOID      pData, 
   UINT32     ulLength
)
{
   SQLResult retcode = SQL_SUCCESS;
   CHAR   szTemp[50] = " ";
   SQLLTIMESTAMP tStamp = *(pSQLLTIMESTAMP)pData;

   sprintf(szTemp, "%d-%d-%d %d:%d:%d",tStamp.year,tStamp.month,tStamp.day,tStamp.hour,tStamp.minute,tStamp.second ); 

   pParam->pData = new BYTE[strlen(szTemp)];
   assert(pParam->pData);

   strncpy((pCHAR) pParam->pData, szTemp, strlen(szTemp) );

   pParam->ulLength = strlen(szTemp);

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::putTime (
   pParameter pParam, 
   pVOID      pData, 
   UINT32     ulLength
)
{
   SQLResult retcode = SQL_SUCCESS;
   CHAR   szTemp[50] = " ";
   UINT16 hour    = 0;
   UINT16 minute  = 0;
   UINT16 msecond = 0;
   TIME time = *(pTIME)pData;
   
   LsTimeDecode(time, &hour, &minute, &msecond);
 
   sprintf(szTemp, "%d:%d:%d", hour, minute, msecond/1000 ); 
 
   pParam->pData = new BYTE[strlen(szTemp)];
   assert(pParam->pData);

   strncpy((pCHAR) pParam->pData, szTemp, strlen(szTemp) );

   pParam->ulLength = strlen(szTemp);

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::putDate (
   pParameter pParam, 
   pVOID      pData, 
   UINT32     ulLength
)
{
   SQLResult retcode = SQL_SUCCESS;
   CHAR   szTemp[50] = " ";
   UINT16 month = 0;
   UINT16 day   = 0;
   INT16  year  = 0;
   DBIDATE date = *(pDBIDATE)pData;
  
   LsDateDecode(date, &month, &day, &year);
 
   sprintf(szTemp, "%d-%d-%d",year,month,day ); 
 
   pParam->pData = new BYTE[strlen(szTemp)];
   assert(pParam->pData);

   strncpy((pCHAR) pParam->pData, szTemp, strlen(szTemp) );

   pParam->ulLength = strlen(szTemp);

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

INT32 BcdToString(
   pFMTBcd  pBcd,
   pCHAR    pDest,
   INT16    iUnits1,
   INT16    iUnits2 )
{
   BYTE     iLoop;
   BYTE     iNumDigits;
   BOOL     bDataStart = FALSE;
   pCHAR    pStr = pDest;

   //
   // First, is the number negative?
   //
   if( pBcd -> iSignSpecialPlaces & ( 1 << 7 ) )
   {
      *pDest++ = '-';
      *pDest = 0;
   }
   else
      *pDest = 0;

   //
   // Now, loop through the whole part of the bcd number
   //
   // use lower 6 bits of iSignSpecialPlaces.
   iNumDigits = pBcd->iPrecision - ( pBcd->iSignSpecialPlaces & 63 );
   for( iLoop = 0; iLoop < iNumDigits; iLoop++ )
   {
      CHAR  cVal;

      if( iLoop % 2 )
         // lower 4 bits only
         cVal = ( pBcd -> iFraction[ ( iLoop - 1 ) /  2 ] & 15 ) + '0';
      else
         // upper 4 bits only
         cVal = ( pBcd -> iFraction[ iLoop / 2 ] >> 4 ) + '0';

      //
      // This little test is used to strip leading '0' chars.
      //
      if( bDataStart )
         *pDest++ = cVal;
      else
      if( cVal != '0' )
      {
         *pDest++ = cVal;
         bDataStart = TRUE;
      }
   }

   //
   // If no data is stored yet, add a leading '0'.
   //
   if( !bDataStart )
      *pDest++ = '0';

   *pDest++ = '.';

   //
   // Now, loop through the fractional part
   //
   for( iLoop = iNumDigits; iLoop < pBcd->iPrecision; iLoop++, pDest++ )
   {
      if( iLoop % 2 )
         // lower 4 bits only
         *pDest = ( pBcd -> iFraction[ ( iLoop - 1 ) /  2 ] & 15 ) + '0';
      else
         // upper 4 bits only
         *pDest = ( pBcd -> iFraction[ iLoop / 2 ] >> 4 ) + '0';
   }

   //
   // If trailing char is decimal point, add a '0'.
   //
   if( *(pDest-1) == '.' )
      *pDest++ = '0';

   *pDest = 0;

   //
   // Right trim '0' chars
   //
   iLoop = strlen( pStr );
   pDest = &pStr[ iLoop-1 ];
   while( *pDest == '0' && ( iLoop > 0 ) )
   {
      //
      // If prior char is a decimal point, leave the trailing '0' char.
      //
      if( *(pDest-1) == '.' )
         break;

      *pDest-- = 0;
      iLoop--;
   }

   return DBXERR_NONE;
}

////////////////////////////////////////////////////////////////////////////////

INT32 BcdToShort (
   pBYTE           pSource,
   pBYTE           pDest,
   INT16           iUnits1,
   INT16           iUnits2
)
{
   CHAR        szTmp[20];
   pCHAR       pszTmp;

   //
   // NOTE: We are hard-coding iUnits1 here. IB doesn't preserve the original
   // precision of BCD fields. The precision we are hard-coding is based on
   // the physical field type
   //
   BcdToString( (pFMTBcd)pSource, szTmp, 5, -iUnits2 );

   //
   // scan for the '.'
   //
   pszTmp = strchr( szTmp, '.' );

   //
   // Is there more data than can be stored at this precision?
   //
   if( ( ( pszTmp - szTmp ) + (-iUnits2) - ( ( szTmp[0] == '-' ) ? 1:0 ) ) > 5 )
      return DBXERR_INVALIDXLATION;

   //
   // Strip data to -iUnits2 to the right of the '.'
   //
   while( iUnits2 )
   {
      *pszTmp = *(pszTmp+1);
      if( !*(pszTmp+1) )
         break;
      iUnits2++;
      pszTmp++;
   }

   //
   // Zero pad the number (if needed).
   //
   while( iUnits2 < 0 )
   {
      *(pszTmp++) = '0';
      iUnits2++;
   }
   *pszTmp = 0;

   //
   // Convert to native format
   //
   *(pINT16)pDest = atol( szTmp );

  return DBXERR_NONE;
}

////////////////////////////////////////////////////////////////////////////////

INT32 BcdToLong (
   pBYTE           pSource,
   pBYTE           pDest,
   INT16           iUnits1,
   INT16           iUnits2
)
{
   CHAR        szTmp[20];
   pCHAR       pszTmp;

   //
   // NOTE: We are hard-coding iUnits1 here. IB doesn't preserve the original
   // precision of BCD fields. The precision we are hard-coding is based on
   // the physical field type
   //
   BcdToString( (pFMTBcd)pSource, szTmp, 9, -iUnits2 );

   //
   // scan for the '.'
   //
   pszTmp = strchr( szTmp, '.' );

   //
   // Is there more data than can be stored at this precision?
   //
   if( ( ( pszTmp - szTmp ) + (-iUnits2) - ( ( szTmp[0] == '-' ) ? 1:0 ) ) > 9 )
      return DBXERR_INVALIDXLATION;

   //
   // Strip data to -iUnits2 to the right of the '.'
   //
   while( iUnits2 )
   {
      *pszTmp = *(pszTmp+1);
      if( !*(pszTmp+1) )
         break;
      iUnits2++;
      pszTmp++;
   }

   //
   // Zero pad the number (if needed).
   //
   while( iUnits2 < 0 )
   {
      *(pszTmp++) = '0';
      iUnits2++;
   }
   *pszTmp = 0;

   //
   // Convert to native format
   //
   *(pINT32)pDest = atol( szTmp );

   return DBXERR_NONE;
}

////////////////////////////////////////////////////////////////////////////////

INT32 BcdToDouble (
   pBYTE           pSource,
   pBYTE           pDest,
   INT16           iUnits1,
   INT16           iUnits2
)
{
   CHAR        szTmp[40];

   BcdToString( (pFMTBcd)pSource, szTmp, 15, -iUnits2 );
   *(double *)pDest = atof( szTmp );

   return DBXERR_NONE;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::putBcd (
   pParameter pParam, 
   pVOID      pData, 
   UINT32     ulLength
)
{
   SQLResult retcode = SQL_SUCCESS;
   CHAR   szTemp[40] = " ";

   retcode = BcdToString( (pFMTBcd)pData, &szTemp[0], pParam->lPrecision, pParam->lScale);

   pParam->pData = new BYTE[strlen(szTemp)];
   assert(pParam->pData);

   strncpy((pCHAR) pParam->pData, szTemp, strlen(szTemp) );

   pParam->ulLength = strlen(szTemp);

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::putBlob (
   pParameter pParam, 
   pVOID      pData, 
   UINT32     ulLength
)
{
   SQLResult retcode = SQL_SUCCESS;

   pParam->pData = new BYTE[ulLength];
   assert(pParam->pData);

   memcpy(pParam->pData, pData, ulLength);

   pParam->ulLength = ulLength;

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::setParameter (
   UINT16     uParamNumber,
   UINT16     uChildPos,
   STMTParamType ePType,
   UINT16     uLogType,
   UINT16     uSubType,
   INT32      lPrecision,
   INT32      lScale,
   UINT32     ulLength,
   pVOID      pBuffer,
   BOOL       bIsNull )
{
   SQLResult retcode  = SQL_SUCCESS;
   INT32 iValue       = 0;
  
   //Check for valid param type 
   if ( ePType != paramIN )
      return DBXERR_INVALIDPARAM;

   pParameter pParam  = &this->pParamList[uParamNumber - 1];

   //Initialize parameter members

   if ( bIsNull == TRUE )
      pParam->iInd    = SQLX_NULL_DATA;
   else
      pParam->iInd    = 0;

   pParam->eParamType = ePType;
   pParam->lPrecision = lPrecision;
   pParam->lScale     = lScale;

   //increment  bound params
   this->uBoundParams ++;
 
   switch (uLogType)
   {
      case fldZSTRING:

         pParam->uSubType = uSubType;
         pParam->uPhyType = FIELD_TYPE_STRING;

         if (pParam->iInd != -1)
            retcode = putString(pParam, pBuffer, ulLength) ;

         break;

      case fldVARBYTES:

         pParam->uSubType = uSubType;
         pParam->uPhyType = FIELD_TYPE_STRING;

         if (pParam->iInd != -1)
            retcode = putBytes(pParam, pBuffer, ulLength) ;

         break;

      case fldINT16:

         pParam->uPhyType = FIELD_TYPE_SHORT;
         pParam->uSubType = uSubType;

         if (pParam->iInd != -1)
            retcode = putShort(pParam, pBuffer, ulLength) ;

         break;


      case fldINT32:

         pParam->uPhyType = FIELD_TYPE_LONG;
         pParam->uSubType = uSubType;

         if (pParam->iInd != -1)
            retcode = putLong(pParam, pBuffer, ulLength) ;

         break;


      case fldFLOAT:

         if ( ulLength == 4 )
         {
            pParam->uPhyType = FIELD_TYPE_FLOAT;
         }
         else
            pParam->uPhyType = FIELD_TYPE_DOUBLE;

         pParam->uSubType = uSubType;

         if (pParam->iInd != -1)
            retcode = putDouble(pParam, pBuffer, ulLength) ;

         break;

      
      case fldDATETIME:

         pParam->uPhyType = FIELD_TYPE_DATE;
         pParam->uSubType = uSubType;

         if (pParam->iInd != -1)
            retcode = putTimeStamp(pParam, pBuffer, ulLength) ;

         break;


      case fldTIME:

         pParam->uPhyType = FIELD_TYPE_TIME;
         pParam->uSubType = uSubType;

         if (pParam->iInd != -1)
            retcode = putTime(pParam, pBuffer, ulLength) ;

         break;


      case fldDATE:

         pParam->uPhyType = FIELD_TYPE_DATE;
         pParam->uSubType = uSubType;

         if (pParam->iInd != -1)
            retcode = putDate(pParam, pBuffer, ulLength) ;

         break;
 
      
      case fldBCD:
         
         switch ( lPrecision )
         {
            case 5:
               pParam->uPhyType = FIELD_TYPE_SHORT;
               break;

            case 9:
               pParam->uPhyType = FIELD_TYPE_LONG;
               break;

            default:
               pParam->uPhyType = FIELD_TYPE_DOUBLE;
               break;
         }

         if (pParam->iInd != -1)
            retcode = putBcd(pParam, pBuffer, ulLength);

         break;

      case fldBLOB:

         iValue = TRUE;

         if ( uSubType == fldstMEMO )
         {
            pParam->uPhyType = FIELD_TYPE_BLOB;
            
            //Text type
         }
         else
         {
            pParam->uPhyType = FIELD_TYPE_BLOB;

            //BLOB type
         }

         pParam->uSubType = uSubType;

         if (pParam->iInd != -1)
            retcode = putBlob(pParam, pBuffer, ulLength);
        
         break;
          
      default: 
         retcode = DBXERR_INVALIDFLDTYPE;    
         break;

   }
   
   return retcode;

}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::execute (
   ppSQLCursor ppCur)
{
   SQLResult retcode = SQL_SUCCESS;

   if (ppCur)
      *ppCur = NULL;

   if ( this->uParams == this->uBoundParams )
      retcode = exec(ppCur);
   else
      retcode = DBXERR_INVALIDPARAM;

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCommand::getParameter (
   UINT16  uParameterNumber,
   UINT16  uChildPos,
   pVOID   pData,
   UINT32  ulLength,
   pINT32  piInd )
{
   SQLResult retcode = SQL_SUCCESS;
   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

VOID MYSQLCommand::sqlTrace ( 
   pCHAR szMessage, UINT16 uMsgId )
{

   if ( pConn )
   {
      pConn->sqlTrace(szMessage, uMsgId);
   }
}

////////////////////////////////////////////////////////////////////////////////

