
/****************************************************************************

dbExpress - Cross-Platform, database independent data access interface

Copyright (c) 2000, 2001 Borland Software Corporation

This file may be distributed and/or modified under the terms of the GNU
General Public License (GPL) version 2 as published by the Free Software

Software Foundation and appearing at
http://www.borland.com/kylix/gpl.html.

Licensees holding a valid Borland No-Nonsense License for this Software
may use this file in accordance with such license, which appears in the file
license.txt that came with this Software.

****************************************************************************/

#include "mysqlobj.h"

#ifdef LINUX
extern nl_catd ResCatalog;
#endif
////////////////////////////////////////////////////////////////////////////////

MYSQLConnection::MYSQLConnection (
)
{
   //Initialize all private members

   pConnection = NULL;
   pCallBack   = NULL;
   pszError    = NULL;
   ulBlobSize  = 0;
   iRefCount   = 0;
   szHostName[0] = '\0';
   
   memset((pVOID)&mySQL,'\0',sizeof(mySQL));
   memset((pVOID)&traceDesc, '\0',sizeof(traceDesc));


}

////////////////////////////////////////////////////////////////////////////////

MYSQLConnection::~MYSQLConnection (
)
{
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLConnection::connect (
         pCHAR pszDbName,
         pCHAR pszUserName, 
         pCHAR pszPassword )
{
   SQLResult retcode = SQL_SUCCESS;

   sqlTrace(szST_init, iST_init);  
   (*pmysql_init)(&this->mySQL); 
 
   sqlTrace(szST_real_connect, iST_real_connect);  
   this->pConnection = (*pmysql_real_connect) ( &this->mySQL, szHostName, 
                          pszUserName, pszPassword, pszDbName, 0, NULL, 0 );

   if (this->pConnection == NULL )
      retcode = DBXERR_INVALIDUSRPASS;
  
   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLConnection::disconnect ()
{
   SQLResult retcode = SQL_SUCCESS;

   if ( this->pConnection )
   {
      sqlTrace(szST_close, iST_close);  
      (*pmysql_close) (this->pConnection);

      this->pConnection = NULL;
   }

   return retcode;
}
   

////////////////////////////////////////////////////////////////////////////////

UINT32 MYSQLConnection::Release()
{
   UINT32 ulCount = 0;

   iRefCount --;

   ulCount = iRefCount;
   if ( iRefCount == 0 )
   {
      //delete this object
      delete this;
   }

   return ulCount;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLConnection::getSQLCommand(ppSQLCommand ppComm)
{
   SQLResult retcode  = SQL_SUCCESS;
   pMYSQLCommand pMYSQLComm = NULL;

   if ( ppComm )
   {
      pMYSQLComm = new MYSQLCommand();
      assert(pMYSQLComm);

      if ( pMYSQLComm )
      {
         pMYSQLComm->AddRef();
         pMYSQLComm->setOption(eCommBlobSize, this->ulBlobSize);
         pMYSQLComm->setConnection(this);
         *ppComm = pMYSQLComm;
      }
      else
         retcode = DBXERR_NOMEMORY;
   }

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLConnection::setOption (
         eSQLConnectOption eCOption, 
         INT32             lValue )
{
   SQLResult retcode = SQL_SUCCESS;

   switch (eCOption)
   {
      case eConnBlobSize:
         this->ulBlobSize = lValue;
         break;

      case eConnHostName:

         if ( strlen((pCHAR)lValue) <= MYSQL_MAXHOSTNAME )
         {
            memset(this->szHostName,'\0',sizeof(szHostName) );
            strncpy(this->szHostName,(pCHAR)lValue, strlen((pCHAR) lValue) );
         }
         else
            retcode = SQLX_ERROR;

         break;

      case eConnCallBack:
         this->pCallBack = (pfCallBack) lValue;
         break;

      case eConnCallBackInfo:
         this->iCBInfo = lValue;
         break;

      default:
         retcode = DBXERR_NOTSUPPORTED;
         break;
   }

   return retcode;
}


////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLConnection::getOption (
         eSQLConnectOption eCOption,
         pINT32            plValue,
         INT16             iMaxLen,
         pINT16            piLen )
{
   SQLResult retcode = SQL_SUCCESS;

   switch (eCOption)
   {
      case eConnBlobSize:
         *plValue = this->ulBlobSize;
         break;

      case eConnHostName:

         if ( strlen( szHostName ) <= iMaxLen )
         {
            strncpy( (pCHAR)plValue, szHostName, strlen(szHostName) );
         }
         else
            *(pCHAR)plValue = '\0';

         *piLen = strlen( szHostName );

         break;

      case eConnNativeHandle:

         *plValue = (INT32)&this->mySQL;
         *piLen   = sizeof(this->mySQL);
         break;

      case eConnMaxActiveComm:
         *plValue = 1;
         break;

      case eConnObjectQuoteChar:
         *(pCHAR)plValue = '\0';
         break;

      default:
         retcode = DBXERR_NOTSUPPORTED;
         break;
   }

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLConnection::getConnectionHandle (
   MYSQL** ppConnection )
{
   SQLResult retcode = SQL_SUCCESS;

   if ( this->pConnection )
   {
      *ppConnection = this->pConnection;
   }
   else
      retcode = DBXERR_INVALIDHNDL;

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLConnection::getErrorMessageLen (
   pUINT16 puErrorLen )
{
   SQLResult retcode = SQL_SUCCESS;
  
   sqlTrace(szST_error, iST_error);  
   this->pszError = mysql_error( &this->mySQL );

   *puErrorLen    = strlen( this->pszError);

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLConnection::getErrorMessage (
   pBYTE pszErrorMessage )
{
  SQLResult retcode = SQL_SUCCESS;

  strcpy((pCHAR)pszErrorMessage,this->pszError);

  return retcode;
}


////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLConnection::sqlTrace( pCHAR pszMessage, UINT16 uMsgId )
{
   SQLResult retcode  = SQL_SUCCESS;
   CBRType cbr        = cbrCONTINUE;
   UINT16  uLen       = 0;
   pCHAR   pszResMessage = NULL;

   if ( !retcode && pCallBack )
   {
      memset((pVOID)&traceDesc,'\0',sizeof(traceDesc) );

      #ifdef LINUX
      pszResMessage = catgets(ResCatalog, 1, uMsgId, pszMessage);
      uLen = strlen(pszResMessage) > DBXMAXTRACELEN ? 
             DBXMAXTRACELEN: strlen(pszResMessage);
      strncpy(&traceDesc.szTrace[0], pszResMessage, uLen);
      #endif

      #ifdef WINDOWS
      uLen = strlen(pszMessage) > DBXMAXTRACELEN ? 
             DBXMAXTRACELEN: strlen(pszMessage);
      strncpy(&traceDesc.szTrace[0], pszMessage, uLen);
      #endif
      traceDesc.iCBInfo = this->iCBInfo;

      //Execute trace callback
      cbr = (*pCallBack) ( cbTRACE, (pVOID) &traceDesc );
    
   }

   return retcode;

} 

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLConnection::getSQLMetaData (
      ppSQLMetaData ppMeta)
{
   SQLResult retcode = SQL_SUCCESS;

   if ( ppMeta )
   {
      *ppMeta = new MYSMetaData( this );
      assert( *ppMeta );

      (*ppMeta)->AddRef( );
   }

   return retcode;
}
///////////////////////////////////////////////////////////////////////////////
