
/****************************************************************************

dbExpress - Cross-Platform, database independent data access interface

Copyright (c) 2000, 2001 Borland Software Corporation

This file may be distributed and/or modified under the terms of the GNU
General Public License (GPL) version 2 as published by the Free Software

Software Foundation and appearing at
http://www.borland.com/kylix/gpl.html.

Licensees holding a valid Borland No-Nonsense License for this Software
may use this file in accordance with such license, which appears in the file
license.txt that came with this Software.

****************************************************************************/

#include "mysqlobj.h"

////////////////////////////////////////////////////////////////////////////////

MYSQLCursor::MYSQLCursor(pMYSQLCommand pComm)
{
    this->pComm = pComm;
    iRefCount   = 0;
}

////////////////////////////////////////////////////////////////////////////////

MYSQLCursor::~MYSQLCursor()
{
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::getColumnCount (
    pUINT16 puColumns ) 
{
   SQLResult retcode = SQL_SUCCESS;

   retcode = this->pComm->numResultColumns(puColumns);

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::getColumnNameLength (
   UINT16 uColumnNumber, pUINT16 uLen )
{
   SQLResult retcode = SQL_SUCCESS;
   pColumn pCol      = NULL;

   retcode = this->pComm->getColumn(uColumnNumber, &pCol);

   if (!retcode )
      *uLen = strlen(pCol->pszColName) + 1;

   return retcode; 
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::getColumnName (
   UINT16 uColumnNumber, 
   pCHAR  pColumnName)
{
 
   SQLResult retcode = SQL_SUCCESS;
   pColumn pCol      = NULL;

   retcode = this->pComm->getColumn(uColumnNumber, &pCol);

   if ( !retcode )
   {
      memcpy(pColumnName, pCol->pszColName, strlen(pCol->pszColName));

      pColumnName[strlen(pCol->pszColName)] = 0;
   }

   return retcode;

}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::getColumnType (
   UINT16 uColumnNumber,
   pUINT16 puType,
   pUINT16 puSubType )
{
   SQLResult retcode = SQL_SUCCESS;
   pColumn pCol      = NULL;

   retcode = this->pComm->getColumn(uColumnNumber, &pCol);

   if (!retcode )
   {
      *puType    = pCol->uLogType;
      *puSubType = pCol->uLogSubType;
   }

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::getColumnPrecision (
   UINT16 uColumnNumber,
   pINT16 piPrecision )
{
   SQLResult retcode = SQL_SUCCESS; 
   pColumn pCol      = NULL;

   retcode = this->pComm->getColumn(uColumnNumber, &pCol);

   if ( !retcode )
      *piPrecision = pCol->iLogPrecision;

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::getColumnScale (
   UINT16 uColumnNumber,
   pINT16 piScale )
{
   SQLResult retcode = SQL_SUCCESS; 
   pColumn pCol      = NULL;

   retcode = this->pComm->getColumn(uColumnNumber, &pCol);

   if ( !retcode )
      *piScale = pCol->iLogScale;

   return retcode;

}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::isBlobSizeExact (
   UINT16 uColumnNumber,
   pBOOL  pbBlobSizeExact )
{
   SQLResult retcode = SQL_SUCCESS;

   *pbBlobSizeExact = TRUE;
   return retcode;
}
 
////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::isNullable (
   UINT16 uColumnNumber,
   pBOOL  pbNullable )
{
   SQLResult retcode = SQL_SUCCESS; 
   pColumn pCol      = NULL;

   retcode = this->pComm->getColumn(uColumnNumber, &pCol);

   //Check the column attribute
   if ( !retcode )
   {
      if ( pCol->uFlags & NOT_NULL_FLAG )
         *pbNullable = FALSE;
      else
         *pbNullable = TRUE;
   }
 
   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::isAutoIncrement (
   UINT16 uColumnNumber,
   pBOOL  pbAutoIncr )
{
   SQLResult retcode = SQL_SUCCESS; 
   pColumn pCol      = NULL;

   retcode = this->pComm->getColumn(uColumnNumber, &pCol);
   
   //Check the column attribute
   if ( !retcode )
   {
      if ( pCol->uFlags & AUTO_INCREMENT_FLAG )
         *pbAutoIncr = TRUE;
      else
         *pbAutoIncr = FALSE;

   }

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::isReadOnly (
   UINT16 uColumnNumber,
   pBOOL pbReadOnly )
{
   SQLResult retcode = SQL_SUCCESS; 
   pColumn pCol      = NULL;

   this->pComm->getColumn(uColumnNumber, &pCol);

   //Check the column attribute
   if ( !retcode ) 
      *pbReadOnly = FALSE;

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::isSearchable(
   UINT16 uColumnNumber,
   pBOOL  pbSearchable )
{
   SQLResult retcode = SQL_SUCCESS; 
   pColumn pCol      = NULL;

   retcode = this->pComm->getColumn(uColumnNumber, &pCol);

   if ( !retcode )
   {
      switch(pCol->uPhyType)
      {
         case FIELD_TYPE_BLOB:
            *pbSearchable = FALSE;
             break;
         
         default:
            *pbSearchable = TRUE;
            break;
      }
   }
 
   return retcode;

}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::getColumnLength(
   UINT16 uColumnNumber,
   pUINT32 pulLength)
{
   SQLResult retcode = SQL_SUCCESS;
   pColumn pCol      = NULL;

   retcode = this->pComm->getColumn(uColumnNumber, &pCol);
 
   if ( !retcode ) 
      *pulLength = pCol->uLogLength;

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::next()
{
    SQLResult retcode = SQL_SUCCESS;

    retcode  = this->pComm->fetch();

    return retcode;
}

////////////////////////////////////////////////////////////////////////////////

UINT32 MYSQLCursor::Release()
{
   SQLResult retcode = SQL_SUCCESS;
   UINT32 ulCount = 0;

   iRefCount --;

   ulCount = iRefCount;
   if ( iRefCount == 0 )
   {
      retcode = this->pComm->close();
      delete this;
   }

   return ulCount;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::getErrorMessageLen (
   pUINT16 puErrorLen )
{
   SQLResult retcode = SQL_SUCCESS;
  
   retcode = this->pComm->getErrorMessageLen( puErrorLen );

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::getErrorMessage (
   pBYTE pszErrorMessage )
{
  SQLResult retcode = SQL_SUCCESS;

  retcode = this->pComm->getErrorMessage( pszErrorMessage );

  return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::getString(
   UINT16 uColumnNumber, 
   pCHAR pData,
   pBOOL pbIsNull)
{
   SQLResult retcode   = SQL_SUCCESS;
   pColumn pCol        = NULL;
   MYSQL_ROW MySQLRow;
   pUINT32   pDataLen = NULL;
   UINT32    uLen     = 0;

   retcode = this->pComm->getColumn(uColumnNumber, &pCol);

   if ( !retcode )
      if ( pCol->uLogType != fldZSTRING )
         retcode = DBXERR_INVALIDXLATION;

   if ( !retcode )
      retcode = this->pComm->getRow (&MySQLRow );

   if ( !retcode )
      retcode = this->pComm->getRowDataLength (&pDataLen );

   if ( !retcode )
   {
      if ( MySQLRow[uColumnNumber - 1] )
      {
         *pbIsNull = FALSE;

         //Check if the data length is more than the metadata MySQL gave
         uLen = pDataLen[uColumnNumber -1] > (pCol->uLogLength - 1) ? 
                (pCol->uLogLength - 1) : pDataLen[uColumnNumber-1];

         strncpy( (pCHAR)pData, MySQLRow[uColumnNumber -1], uLen);
         *(pData + uLen) = '\0';
      }
      else
         *pbIsNull = TRUE;

   }

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::getShort(
   UINT16 uColumnNumber,
   pINT16 pData,
   pBOOL pbIsNull)
{
   SQLResult retcode   = SQL_SUCCESS;
   pColumn pCol        = NULL;
   MYSQL_ROW MySQLRow;
   pUINT32   pDataLen  = NULL;
   CHAR temp[50]       = " ";

   retcode = this->pComm->getColumn(uColumnNumber, &pCol);

   if ( !retcode )
      if ( pCol->uLogType != fldINT16 )
         retcode = DBXERR_INVALIDXLATION;

   if ( !retcode )
      retcode = this->pComm->getRow (&MySQLRow );

   if ( !retcode )
      retcode = this->pComm->getRowDataLength (&pDataLen );


   if ( !retcode )
   {
      if ( MySQLRow[uColumnNumber -1] )
      {
         *pbIsNull = FALSE;
         strncpy( (pCHAR)temp, MySQLRow[uColumnNumber -1], pDataLen[uColumnNumber -1] );
         *(pINT16)pData = atoi(temp);
     
      }
      else
         *pbIsNull = TRUE;

   }

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::getLong(
   UINT16 uColumnNumber,
   pINT32 pData, 
   pBOOL pbIsNull)
{
   SQLResult retcode   = SQL_SUCCESS;
   pColumn pCol        = NULL;
   MYSQL_ROW MySQLRow;
   pUINT32   pDataLen  = NULL;
   CHAR temp[50]       = " ";

   retcode = this->pComm->getColumn(uColumnNumber, &pCol);

   if ( !retcode )
      if ( pCol->uLogType != fldINT32 )
         retcode = DBXERR_INVALIDXLATION;

   if ( !retcode )
      retcode = this->pComm->getRow (&MySQLRow );

   if ( !retcode )
      retcode = this->pComm->getRowDataLength (&pDataLen );


   if ( !retcode )
   {
      if ( MySQLRow[uColumnNumber -1] )
      {
         *pbIsNull = FALSE;
         strncpy( (pCHAR)temp, MySQLRow[uColumnNumber -1], pDataLen[uColumnNumber -1] );
         *(pINT32)pData = atol(temp);
     
      }
      else
         *pbIsNull = TRUE;

   }

   return retcode;

}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::getDouble(
   UINT16 uColumnNumber,
   pDFLOAT pData,
   pBOOL pbIsNull)
{
   SQLResult retcode   = SQL_SUCCESS;
   pColumn pCol        = NULL;
   MYSQL_ROW MySQLRow;
   pUINT32   pDataLen  = NULL;
   CHAR temp[50]       = " ";

   retcode = this->pComm->getColumn(uColumnNumber, &pCol);

   if ( !retcode )
      if ( pCol->uLogType != fldFLOAT )
         retcode = DBXERR_INVALIDXLATION;

   if ( !retcode )
      retcode = this->pComm->getRow (&MySQLRow );

   if ( !retcode )
      retcode = this->pComm->getRowDataLength (&pDataLen );


   if ( !retcode )
   {
      if ( MySQLRow[uColumnNumber -1] )
      {
         *pbIsNull = FALSE;
         strncpy( (pCHAR)temp, MySQLRow[uColumnNumber -1], pDataLen[uColumnNumber -1] );
         *(pDFLOAT)pData = atof(temp);
     
      }
      else
         *pbIsNull = TRUE;

   }

   return retcode;

}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::getBcd(
   UINT16  uColumnNumber,
   pFMTBcd pData,
   pBOOL   pbIsNull)
{
   SQLResult retcode   = SQL_SUCCESS;
   pColumn pCol        = NULL;
   MYSQL_ROW MySQLRow;
   pUINT32   pDataLen  = NULL;
   CHAR temp[50]       = " ";

   retcode = this->pComm->getColumn(uColumnNumber, &pCol);

   if ( !retcode )
      if ( pCol->uLogType != fldBCD )
         retcode = DBXERR_INVALIDXLATION;

   if ( !retcode )
      retcode = this->pComm->getRow (&MySQLRow );

   if ( !retcode )
      retcode = this->pComm->getRowDataLength (&pDataLen );


   if ( !retcode )
   {
      if ( MySQLRow[uColumnNumber -1] )
      {
         *pbIsNull = FALSE;
         strncpy( (pCHAR)temp, MySQLRow[uColumnNumber -1], pDataLen[uColumnNumber -1] );
         retcode = StringToBcd ( &temp[0], (pFMTBcd) pData, 15, 0 );
     
      }
      else
         *pbIsNull = TRUE;

   }


   return retcode;

}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::getTimeStamp(
   UINT16 uColumnNumber,
   pSQLLTIMESTAMP pData, 
   pBOOL pbIsNull)
{
   SQLResult retcode   = SQL_SUCCESS;
   pColumn pCol        = NULL;
   MYSQL_ROW MySQLRow;
   pUINT32   pDataLen  = NULL;
   CHAR temp[50]       = " ";
   CHAR szYear[5]      = " ";
   CHAR szMonth[3]     = " ";
   CHAR szDay[3]       = " ";
   CHAR szHour[3]      = " ";
   CHAR szMinute[3]    = " ";
   CHAR szSecond[3]    = " ";
   pSQLLTIMESTAMP pTimeStamp = (pSQLLTIMESTAMP) pData;
   pTimeStamp->fraction      = 0;

   retcode = this->pComm->getColumn(uColumnNumber, &pCol);

   if ( !retcode )
      if ( pCol->uLogType != fldDATETIME )
         retcode = DBXERR_INVALIDXLATION;

   if ( !retcode )
      retcode = this->pComm->getRow (&MySQLRow );

   if ( !retcode )
      retcode = this->pComm->getRowDataLength (&pDataLen );


   if ( !retcode )
   {
      if ( MySQLRow[uColumnNumber -1] )
      {
         *pbIsNull = FALSE;
         strncpy( (pCHAR)temp, MySQLRow[uColumnNumber -1], pDataLen[uColumnNumber -1] );
         
         switch ( pCol->uLength )
         {
            case 14: //YYYYMMDDHHMMSS
               
               strncpy((pCHAR)&szYear[0], (pCHAR)temp, 4); 
               strncpy((pCHAR)&szMonth[0], (pCHAR)temp+4, 2);
               strncpy((pCHAR)&szDay[0], (pCHAR)temp+6, 2);
               strncpy((pCHAR)&szHour[0], (pCHAR)temp+8, 2);
	       strncpy((pCHAR)&szMinute[0], (pCHAR)temp+10, 2);
               strncpy((pCHAR)&szSecond[0], (pCHAR)temp+12, 2);
 
               pTimeStamp->year   = atoi(szYear);
               pTimeStamp->month  = atoi(szMonth);
               pTimeStamp->day    = atoi(szDay);
               pTimeStamp->hour   = atoi(szHour);
               pTimeStamp->minute = atoi(szMinute);
               pTimeStamp->second = atoi(szSecond);
               break;

            case 12: //YYMMDDHHMMSS

               strncpy((pCHAR)&szYear[0], (pCHAR)temp, 2); 
               strncpy((pCHAR)&szMonth[0], (pCHAR)temp+2, 2);
               strncpy((pCHAR)&szDay[0], (pCHAR)temp+4, 2);
               strncpy((pCHAR)&szHour[0], (pCHAR)temp+6, 2);
	       strncpy((pCHAR)&szMinute[0], (pCHAR)temp+8, 2);
               strncpy((pCHAR)&szSecond[0], (pCHAR)temp+10, 2);
 
               pTimeStamp->year   = atoi(szYear);
               pTimeStamp->month  = atoi(szMonth);
               pTimeStamp->day    = atoi(szDay);
               pTimeStamp->hour   = atoi(szHour);
               pTimeStamp->minute = atoi(szMinute);
               pTimeStamp->second = atoi(szSecond);
               break;

            case 10: //YYMMDDHHMM

               strncpy((pCHAR)&szYear[0], (pCHAR)temp, 2); 
               strncpy((pCHAR)&szMonth[0], (pCHAR)temp+2, 2);
               strncpy((pCHAR)&szDay[0], (pCHAR)temp+4, 2);
               strncpy((pCHAR)&szHour[0], (pCHAR)temp+6, 2);
               strncpy((pCHAR)&szMinute[0], (pCHAR)temp+8, 2);
 
               pTimeStamp->year   = atoi(szYear);
               pTimeStamp->month  = atoi(szMonth);
               pTimeStamp->day    = atoi(szDay);
               pTimeStamp->hour   = atoi(szHour);
               pTimeStamp->minute = atoi(szMinute);
               pTimeStamp->second = 0;
               break;

            case  8: //YYYYMMDD

               strncpy((pCHAR)&szYear[0], (pCHAR)temp, 4); 
               strncpy((pCHAR)&szMonth[0], (pCHAR)temp+4, 2);
               strncpy((pCHAR)&szDay[0], (pCHAR)temp+6, 2);
 
               pTimeStamp->year   = atoi(szYear);
               pTimeStamp->month  = atoi(szMonth);
               pTimeStamp->day    = atoi(szDay);
               pTimeStamp->hour   = 0; 
               pTimeStamp->minute = 0;
               pTimeStamp->second = 0;

               break;

            case  6: //YYMMDD

               strncpy((pCHAR)&szYear[0], (pCHAR)temp, 2); 
               strncpy((pCHAR)&szMonth[0], (pCHAR)temp+2, 2);
               strncpy((pCHAR)&szDay[0], (pCHAR)temp+4, 2);
 
               pTimeStamp->year   = atoi(szYear);
               pTimeStamp->month  = atoi(szMonth);
               pTimeStamp->day    = atoi(szDay);
               pTimeStamp->hour   = 0; 
               pTimeStamp->minute = 0;
               pTimeStamp->second = 0;
               break;

            case  4: //YYMM

               strncpy((pCHAR)&szYear[0], (pCHAR)temp, 2); 
               strncpy((pCHAR)&szMonth[0], (pCHAR)temp+2, 2);
 
               pTimeStamp->year   = atoi(&szYear[0]);
               pTimeStamp->month  = atoi(szMonth);
               pTimeStamp->day    = 0; 
               pTimeStamp->hour   = 0; 
               pTimeStamp->minute = 0;
               pTimeStamp->second = 0;
               break;

            case  2: //YY

               pTimeStamp->year = atoi(temp);
               pTimeStamp->month  = 0;
               pTimeStamp->day    = 0; 
               pTimeStamp->hour   = 0; 
               pTimeStamp->minute = 0;
               pTimeStamp->second = 0;
               break;

            case 19: //DATETIME

               strncpy((pCHAR)&szYear[0], (pCHAR)temp, 4); 
               strncpy((pCHAR)&szMonth[0], (pCHAR)temp+5, 2);
               strncpy((pCHAR)&szDay[0], (pCHAR)temp+8, 2);
               strncpy((pCHAR)&szHour[0], (pCHAR)temp+11, 2);
	       strncpy((pCHAR)&szMinute[0], (pCHAR)temp+14, 2);
               strncpy((pCHAR)&szSecond[0], (pCHAR)temp+17, 2);
 
               pTimeStamp->year   = atoi(szYear);
               pTimeStamp->month  = atoi(szMonth);
               pTimeStamp->day    = atoi(szDay);
               pTimeStamp->hour   = atoi(szHour);
               pTimeStamp->minute = atoi(szMinute);
               pTimeStamp->second = atoi(szSecond);
               break;

            default:
               assert("Invalid TimeStamp/DateTime size");
               break;
        }

     
      }
      else
         *pbIsNull = TRUE;

   }

   return retcode;

}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::getTime(
   UINT16 uColumnNumber,
   pINT32 pData, 
   pBOOL pbIsNull)
{
   SQLResult retcode   = SQL_SUCCESS;
   pColumn pCol        = NULL;
   MYSQL_ROW MySQLRow;
   pUINT32   pDataLen  = NULL;
   CHAR temp[50]       = " ";
   CHAR szHour[3]      = " ";
   CHAR szMinute[3]    = " ";
   CHAR szSecond[3]    = " ";
   SQLLTIME Time;       


   retcode = this->pComm->getColumn(uColumnNumber, &pCol);
   
   if ( !retcode )
      if ( pCol->uLogType != fldTIME )
         retcode = DBXERR_INVALIDXLATION;

   if ( !retcode )
      retcode = this->pComm->getRow (&MySQLRow );

   if ( !retcode )
      retcode = this->pComm->getRowDataLength (&pDataLen );


   if ( !retcode )
   {
      if ( MySQLRow[uColumnNumber -1] )
      {
         *pbIsNull = FALSE;
         strncpy( (pCHAR)temp, MySQLRow[uColumnNumber -1], pDataLen[uColumnNumber -1] );
         strncpy((pCHAR)&szHour[0], (pCHAR)temp, 2);
	 strncpy((pCHAR)&szMinute[0], (pCHAR)temp+3, 2);
         strncpy((pCHAR)&szSecond[0], (pCHAR)temp+6, 2);
 
         Time.hour   = atoi(szHour);
         Time.minute = atoi(szMinute);
         Time.second = atoi(szSecond);

         retcode  = LsTimeEncode( Time.hour, Time.minute, Time.second * 1000, pData );

      }
      else
      {
         *pData    = 0;
         *pbIsNull = TRUE;
      }

   }

   return retcode;

}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::getDate(
   UINT16 uColumnNumber,
   pINT32 pData, 
   pBOOL  pbIsNull)
{
   SQLResult retcode   = SQL_SUCCESS;
   pColumn pCol        = NULL;
   MYSQL_ROW MySQLRow;
   pUINT32   pDataLen  = NULL;
   CHAR temp[50]       = " ";
   CHAR szYear[5]      = " ";
   CHAR szMonth[3]     = " ";
   CHAR szDay[3]       = " ";
   SQLLDATE  Date;

   retcode = this->pComm->getColumn(uColumnNumber, &pCol);

   if ( !retcode )
      if ( pCol->uLogType != fldDATE )
         retcode = DBXERR_INVALIDXLATION;

   if ( !retcode )
      retcode = this->pComm->getRow (&MySQLRow );

   if ( !retcode )
      retcode = this->pComm->getRowDataLength (&pDataLen );


   if ( !retcode )
   {
      if ( MySQLRow[uColumnNumber -1] )
      {
         *pbIsNull = FALSE;
         strncpy( (pCHAR)temp, MySQLRow[uColumnNumber -1], pDataLen[uColumnNumber -1] );
         
         strncpy((pCHAR)&szYear[0], (pCHAR)temp, 4); 
         strncpy((pCHAR)&szMonth[0], (pCHAR)temp+5, 2);
         strncpy((pCHAR)&szDay[0], (pCHAR)temp+8, 2);
 
         Date.year   = atoi(szYear);
         Date.month  = atoi(szMonth);
         Date.day    = atoi(szDay);

         *pData = LsDateEncode( Date.month, Date.day, Date.year );
      }
      else
      {
         *pData    = 0;
         *pbIsNull = TRUE;
      }
   }

   return retcode;

}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::getBytes(
   UINT16 uColumnNumber,
   pBYTE  pData, 
   pBOOL  pbIsNull)
{
   SQLResult retcode   = SQL_SUCCESS;
   pColumn pCol        = NULL;
   MYSQL_ROW MySQLRow;
   pUINT32   pDataLen  = NULL;

   retcode = this->pComm->getColumn(uColumnNumber, &pCol);

   if ( !retcode )
      if ( pCol->uLogType != fldVARBYTES )
         retcode = DBXERR_INVALIDXLATION;

   if ( !retcode )
      retcode = this->pComm->getRow (&MySQLRow );

   if ( !retcode )
      retcode = this->pComm->getRowDataLength (&pDataLen );


   if ( !retcode )
   {
      if ( MySQLRow[uColumnNumber -1] )
      {
         *pbIsNull = FALSE;
         *(pUINT16)pData = pDataLen[uColumnNumber -1];
         memcpy(pData+sizeof(UINT16), MySQLRow[uColumnNumber -1], pDataLen[uColumnNumber -1] );
      }
      else
         *pbIsNull = TRUE;
   }

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::getBlobSize(
   UINT16 uColumnNumber,
   pUINT32 pulLength,
   pBOOL pbIsNull )
{
   SQLResult retcode   = SQL_SUCCESS;
   pColumn pCol        = NULL;
   MYSQL_ROW MySQLRow;
   pUINT32   pDataLen  = NULL;

   retcode = this->pComm->getColumn(uColumnNumber, &pCol);

   if ( !retcode )
      if ( pCol->uLogType  != fldBLOB )
         retcode = DBXERR_INVALIDFLDTYPE;

   if ( !retcode )
      retcode = this->pComm->getRow (&MySQLRow );

   if ( !retcode )
      retcode = this->pComm->getRowDataLength (&pDataLen );

   if ( !retcode )
   {
      if ( MySQLRow[uColumnNumber -1] )
      {
         *pbIsNull = FALSE;
         *pulLength = pDataLen[uColumnNumber -1];
      }
      else
         *pbIsNull = TRUE;
   }
   
   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLCursor::getBlob(
   UINT16 uColumnNumber,
   pVOID pData,
   pBOOL pbIsNull,
   UINT32 ulLength)
{
   SQLResult retcode   = SQL_SUCCESS;
   pColumn pCol        = NULL;
   MYSQL_ROW MySQLRow;
   pUINT32   pDataLen  = NULL;

   retcode = this->pComm->getColumn(uColumnNumber, &pCol);

   if ( !retcode )
      if ( pCol->uLogType  != fldBLOB )
         retcode = DBXERR_INVALIDXLATION;

   if ( !retcode )
      retcode = this->pComm->getRow (&MySQLRow );

   if ( !retcode )
      retcode = this->pComm->getRowDataLength (&pDataLen );


   if ( !retcode )
   {
      if ( MySQLRow[uColumnNumber -1] )
      {
         *pbIsNull = FALSE;
         ulLength = ulLength > pDataLen[uColumnNumber -1] ? pDataLen[uColumnNumber -1] : ulLength ;  
         memcpy((pBYTE)pData, MySQLRow[uColumnNumber -1], ulLength );
          
         *((pBYTE)pData+ulLength) = '\0';
      }
      else
         *pbIsNull = TRUE;
   }

   return retcode;

}

////////////////////////////////////////////////////////////////////////////////

// This function does no effort to validate bogus strings. Following is what is and
// isn't checked:
//    - Exponential form is currently invalid. This is checked for by looking for
//      'e' | 'E' in the string.
//    - A leading '+' | '-' is valid and will set the sign bit.
//    - The last '.' found will be used to determine the decimal point location
//      (no error will be returned for multiple decimal points).
//    - No error will be returned for embedded non-digit characters.
//    - Leading whitespace will be ignored.
//
INT32 StringToBcd(
   pCHAR    pSource,
   pFMTBcd  pBcd,
   INT16    iUnits1,
   INT16    iUnits2 )
{
   BOOL     bNeg = FALSE;
   INT16    iLoop;
   INT16    iNumDigits;
   INT16    iDecimalPos;
   pCHAR    pszTmp;

   //
   // Strip leading whitespace.
   //
   while( isspace( *pSource ) )
      pSource++;

   //
   // Strip trailing whitespace.
   //
   pszTmp = &pSource[ strlen( pSource ) - 1 ];
   while( isspace( *pszTmp ) )
      pszTmp--;
   pszTmp[1] = 0;

   //
   // Is the number negative?
   //
   //if( ( bNeg = ( *pSource == '-' ) ) || ( *pSource == '+' ) )
   if ( *pSource == '-' ) 
      bNeg = TRUE;
   if ( ( *pSource == '-' )  || ( *pSource == '+' ) )
      pSource++;

   //
   // Clear structure.
   //
   memset( pBcd->iFraction, 0, sizeof( pBcd->iFraction ));

   //
   // Start stuffing the nibbles.
   //
   iNumDigits = strlen( pSource );
   iDecimalPos = 0;
   for( iLoop = 0; iLoop < iNumDigits; iLoop++, pSource++ )
   {
      if( *pSource == '.' )
      {
         if( iLoop == 0 )
         {
            //
            // If the string starts with the decimal point, stuff a 0 at the
            // start of the BCD value.
            //
            iDecimalPos = 1;
            pBcd -> iFraction[ 0 ] |= 0;
            continue;
         }
         else
         {
            //
            // Store the location of the decimal.
            //
            iDecimalPos = iLoop;

            //
            // Skip over the decimal point character.
            //
            iNumDigits--;
            pSource++;
         }
      }
      else
      //
      // Disallow translation of data stored in exponential form.
      //
      if( ( *pSource == 'e' ) || ( *pSource == 'E' ) )
      {
         memset( pBcd, 0,  sizeof( *pBcd ));
         return DBXERR_INVALIDXLATION;
      }
      else
      //
      // If any other character (ie. a ',',...) skip it.
      //
      if( !isdigit( *pSource ) )
      {
         iNumDigits--;
         pSource++;
      }

      //
      // 'OR' the data into the appropriate nibble.
      //
      if( iLoop % 2 )
         pBcd -> iFraction[ ( iLoop - 1 ) /  2 ] |= *pSource - '0';
      else
         pBcd -> iFraction[ iLoop / 2 ] |= ( *pSource - '0' ) << 4;
   }

   //
   // Set precision, sign, and scale (ignore special (ie. set to 0)).
   //
   pBcd -> iPrecision = (BYTE)iNumDigits;
   if( iDecimalPos )
      pBcd -> iSignSpecialPlaces = ( ( bNeg ? 1 : 0 ) << 7 ) + (BYTE)( iNumDigits - iDecimalPos );
   else
      pBcd -> iSignSpecialPlaces = ( bNeg ? 1 : 0 ) << 7;

   return DBXERR_NONE;
}

////////////////////////////////////////////////////////////////////////////////

