
/****************************************************************************

dbExpress - Cross-Platform, database independent data access interface

Copyright (c) 2000, 2001 Borland Software Corporation

This file may be distributed and/or modified under the terms of the GNU
General Public License (GPL) version 2 as published by the Free Software

Software Foundation and appearing at
http://www.borland.com/kylix/gpl.html.

Licensees holding a valid Borland No-Nonsense License for this Software
may use this file in accordance with such license, which appears in the file
license.txt that came with this Software.

****************************************************************************/

#include "mysqlobj.h"

typedef MYSQL*	STDCALL (*tmysql_init) (MYSQL *mysql);
typedef MYSQL*	STDCALL (*tmysql_connect) (MYSQL *mysql, const char *host,
		   const char *user, const char *passwd);
typedef MYSQL*	STDCALL (*tmysql_real_connect) (MYSQL *mysql, const char *host,
		   const char *user,
		   const char *passwd,
		   const char *db,
		   unsigned int port,
		   const char *unix_socket,
		   unsigned int clientflag);
typedef void	STDCALL (*tmysql_close) (MYSQL *sock);
typedef int	STDCALL (*tmysql_select_db) (MYSQL *mysql, const char *db);
typedef int	STDCALL (*tmysql_query) (MYSQL *mysql, const char *q);
typedef int	STDCALL (*tmysql_real_query) (MYSQL *mysql, const char *q,
                   unsigned int length);
typedef int	STDCALL (*tmysql_create_db) (MYSQL *mysql, const char *DB);
typedef int	STDCALL (*tmysql_drop_db) (MYSQL *mysql, const char *DB);
typedef int	STDCALL (*tmysql_shutdown) (MYSQL *mysql);
typedef int	STDCALL (*tmysql_dump_debug_info) (MYSQL *mysql);
typedef int	STDCALL (*tmysql_refresh) (MYSQL *mysql,
		   unsigned int refresh_options);
typedef int	STDCALL (*tmysql_kill) (MYSQL *mysql,unsigned long pid);
typedef int	STDCALL (*tmysql_ping) (MYSQL *mysql);
typedef char *	STDCALL (*tmysql_stat) (MYSQL *mysql);
typedef char *	STDCALL (*tmysql_get_server_info) (MYSQL *mysql);
typedef char *	STDCALL (*tmysql_get_client_info) (void);
typedef char *	STDCALL (*tmysql_get_host_info) (MYSQL *mysql);
typedef unsigned int	STDCALL (*tmysql_get_proto_info) (MYSQL *mysql);
typedef MYSQL_RES *	STDCALL (*tmysql_list_dbs) (MYSQL *mysql,const char *wild);
typedef MYSQL_RES *	STDCALL (*tmysql_list_tables) (MYSQL *mysql,const char *wild);
typedef MYSQL_RES *	STDCALL (*tmysql_list_fields) (MYSQL *mysql, const char *table,
					 const char *wild);
typedef MYSQL_RES *	STDCALL (*tmysql_list_processes) (MYSQL *mysql);
typedef MYSQL_RES *	STDCALL (*tmysql_store_result) (MYSQL *mysql);
typedef MYSQL_RES *	STDCALL (*tmysql_use_result) (MYSQL *mysql);
typedef int	STDCALL (*tmysql_options) (MYSQL *mysql,enum mysql_option option,
				     const char *arg);
typedef void	STDCALL (*tmysql_free_result) (MYSQL_RES *result);
typedef void	STDCALL (*tmysql_data_seek) (MYSQL_RES *result,unsigned int offset);
typedef MYSQL_ROW_OFFSET STDCALL (*tmysql_row_seek) (MYSQL_RES *result, MYSQL_ROW_OFFSET);
typedef MYSQL_FIELD_OFFSET STDCALL (*tmysql_field_seek) (MYSQL_RES *result,
			      MYSQL_FIELD_OFFSET offset);
typedef MYSQL_ROW	STDCALL (*tmysql_fetch_row) (MYSQL_RES *result);
typedef unsigned long *	STDCALL (*tmysql_fetch_lengths) (MYSQL_RES *result);
typedef MYSQL_FIELD *	STDCALL (*tmysql_fetch_field) (MYSQL_RES *result);
typedef unsigned int	STDCALL (*tmysql_escape_string) (char *to,const char *from,
			   unsigned int from_length);
typedef void	STDCALL (*tmysql_debug) (const char *debug);

tmysql_init pmysql_init;
tmysql_connect pmysql_connect;
tmysql_real_connect pmysql_real_connect;
tmysql_close pmysql_close;
tmysql_select_db pmysql_select_db;
tmysql_query pmysql_query;
tmysql_real_query pmysql_real_query;
tmysql_create_db pmysql_create_db;
tmysql_drop_db pmysql_drop_db;
tmysql_shutdown pmysql_shutdown;
tmysql_dump_debug_info pmysql_dump_debug_info;
tmysql_refresh pmysql_refresh;
tmysql_kill pmysql_kill;
tmysql_ping pmysql_ping;
tmysql_stat pmysql_stat;
tmysql_get_server_info pmysql_get_server_info;
tmysql_get_client_info pmysql_get_client_info;
tmysql_get_host_info pmysql_get_host_info;
tmysql_get_proto_info pmysql_get_proto_info;
tmysql_list_dbs pmysql_list_dbs;
tmysql_list_tables pmysql_list_tables;
tmysql_list_fields pmysql_list_fields;
tmysql_list_processes pmysql_list_processes;
tmysql_store_result pmysql_store_result;
tmysql_use_result pmysql_use_result;
tmysql_options pmysql_options;
tmysql_free_result pmysql_free_result;
tmysql_data_seek pmysql_data_seek;
tmysql_row_seek pmysql_row_seek;
tmysql_fetch_row pmysql_fetch_row;
tmysql_fetch_lengths pmysql_fetch_lengths;
tmysql_fetch_field pmysql_fetch_field;
tmysql_escape_string pmysql_escape_string;
tmysql_debug pmysql_debug;

#ifdef LINUX
pVOID hDLL = NULL;
nl_catd ResCatalog = NULL;
#endif

#ifdef WINDOWS
HMODULE hDLL = NULL;
#endif

////////////////////////////////////////////////////////////////////////////////

SQLResult SQLInit( pCHAR pszVendorLib, pCHAR pszResFile )
{

   SQLResult retcode = SQL_SUCCESS;

   #ifdef LINUX
   hDLL = dlopen(pszVendorLib, RTLD_NOW);

   //Initialize locale
   setlocale(LC_ALL,"");
   ResCatalog = catopen(pszResFile, NL_CAT_LOCALE);
   #endif

   #ifdef WINDOWS
   hDLL = LoadLibrary(pszVendorLib);
   #endif

   if ( !hDLL )
   {
      retcode = SQLX_INITFAIL;

   }
   else
   {
      #ifdef LINUX

         pmysql_init                = (tmysql_init) dlsym(hDLL, "mysql_init");
         pmysql_connect             = (tmysql_connect) dlsym(hDLL, "mysql_connect");
         pmysql_real_connect        = (tmysql_real_connect) dlsym(hDLL, "mysql_real_connect");
         pmysql_close               = (tmysql_close) dlsym(hDLL, "mysql_close");
         pmysql_real_query          = (tmysql_real_query) dlsym(hDLL, "mysql_real_query");
         pmysql_use_result          = (tmysql_use_result) dlsym(hDLL, "mysql_use_result");
         pmysql_fetch_field         = (tmysql_fetch_field) dlsym(hDLL, "mysql_fetch_field");
         pmysql_fetch_row           = (tmysql_fetch_row) dlsym(hDLL, "mysql_fetch_row");
         pmysql_fetch_lengths       = (tmysql_fetch_lengths) dlsym(hDLL, "mysql_fetch_lengths");
         pmysql_free_result         = (tmysql_free_result) dlsym(hDLL, "mysql_free_result");
         pmysql_escape_string       = (tmysql_escape_string) dlsym(hDLL, "mysql_escape_string");

      #endif

      #ifdef WINDOWS

         pmysql_init                = (tmysql_init) GetProcAddress(hDLL, "mysql_init");
         pmysql_connect             = (tmysql_connect) GetProcAddress(hDLL, "mysql_connect");
         pmysql_real_connect        = (tmysql_real_connect) GetProcAddress(hDLL, "mysql_real_connect");
         pmysql_close               = (tmysql_close) GetProcAddress(hDLL, "mysql_close");
         pmysql_real_query          = (tmysql_real_query) GetProcAddress(hDLL, "mysql_real_query");
         pmysql_use_result          = (tmysql_use_result) GetProcAddress(hDLL, "mysql_use_result");
         pmysql_fetch_field         = (tmysql_fetch_field) GetProcAddress(hDLL, "mysql_fetch_field");
         pmysql_fetch_row           = (tmysql_fetch_row) GetProcAddress(hDLL, "mysql_fetch_row");
         pmysql_fetch_lengths       = (tmysql_fetch_lengths) GetProcAddress(hDLL, "mysql_fetch_lengths");
         pmysql_free_result         = (tmysql_free_result) GetProcAddress(hDLL, "mysql_free_result");
         pmysql_escape_string       = (tmysql_escape_string) GetProcAddress(hDLL, "mysql_escape_string");

      #endif
      
      if ( !pmysql_init ||
           !pmysql_connect ||
           !pmysql_real_connect ||
           !pmysql_close ||
           !pmysql_real_query ||
           !pmysql_use_result ||
           !pmysql_fetch_field ||
           !pmysql_fetch_row   ||
           !pmysql_fetch_lengths ||
           !pmysql_free_result   ||
           !pmysql_escape_string    
      )
      {
         retcode = SQLX_INITFAIL;
      }
         

   }
   
   return retcode;

}

////////////////////////////////////////////////////////////////////////////////

SQLResult  getSQLDriverMYSQL ( pCHAR pszVendorLib, pCHAR pszResFile, ppSQLDriver ppDrvr)
{
   SQLResult retcode = SQL_SUCCESS;

   if ( ppDrvr )
   {
      *ppDrvr  = new MYSQLDriver();
      (*ppDrvr)->AddRef();
      retcode  = SQLInit(pszVendorLib, pszResFile);
   }
   else
      retcode = DBXERR_INVALIDHNDL;

  return retcode;
}

////////////////////////////////////////////////////////////////////////////////

MYSQLDriver::MYSQLDriver()
{
   iRefCount = 0;
}

////////////////////////////////////////////////////////////////////////////////

MYSQLDriver::~MYSQLDriver()
{
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLDriver::getSQLConnection(ppSQLConnection ppConn)
{
   SQLResult retcode = SQL_SUCCESS;

   pSQLConnection pConn = new MYSQLConnection();
   assert(pConn);

   if ( pConn && ppConn )
   {
      pConn->AddRef();
      pConn->setOption(eConnBlobSize, 1024);

      *ppConn = pConn;
   }
   else
      retcode = DBXERR_NOMEMORY;

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLDriver::setOption (
         eSQLDriverOption  eDrvOption, 
         INT32             lValue )
{
   SQLResult retcode = SQL_SUCCESS;

   switch (eDrvOption)
   {
      case eDrvRestrict:
         break;
      default:
         retcode = DBXERR_NOTSUPPORTED;
         break;
   }

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

SQLResult MYSQLDriver::getOption (
         eSQLDriverOption  eDrvOption,
         pINT32            plValue,
         INT16             iMaxLen,
         pINT16            piLen )
{
   SQLResult retcode = SQL_SUCCESS;

   switch (eDrvOption)
   {

      default:
         retcode = DBXERR_NOTSUPPORTED;
         break;
   }

   return retcode;
}

////////////////////////////////////////////////////////////////////////////////

UINT32 MYSQLDriver::Release()
{
   UINT32 ulCount = 0;

   iRefCount--;

   ulCount = iRefCount;
   if ( iRefCount == 0 )
   {
      delete this;
   }
   return ulCount;
}

////////////////////////////////////////////////////////////////////////////////

