/****************************************************************************

dbExpress - Cross-Platform, database independent data access interface

Copyright (c) 2000, 2001 Borland Software Corporation

This file may be distributed and/or modified under the terms of the GNU
General Public License (GPL) version 2 as published by the Free Software

Software Foundation and appearing at
http://www.borland.com/kylix/gpl.html.

Licensees holding a valid Borland No-Nonsense License for this Software
may use this file in accordance with such license, which appears in the file
license.txt that came with this Software.

****************************************************************************/

#include "myschema.h"
#include "myschcon.h"
#include <stdlib.h>
#include <string.h>

#ifdef WINDOWS
  #define StrCmpi(f1, f2) strcmpi((pCHAR) f1, (pCHAR) f2)
#else
  #define StrCmpi(f1, f2) strcasecmp((pCHAR) f1, (pCHAR) f2)
#endif
/*--------------------------------------------------------------
 1. todo: local and temp tables.
 2. todo: getDatabaseName from Ramesh when available.
 3. todo: finish and check getOption() 
---------------------------------------------------------------*/
/*--------------------------------------------------------------
/ class DBMetaData method implementation
/--------------------------------------------------------------*/

INT32 BcdToStr(
   pFMTBcd  pBcd,
   pCHAR    pDest,
   INT16    iUnits1,
   INT16    iUnits2 )
{
   BYTE     iLoop;
   BYTE     iNumDigits;
   BOOL     bDataStart = FALSE;
   pCHAR    pStr = pDest;

   //
   // First, is the number negative?
   //
   if( pBcd -> iSignSpecialPlaces & ( 1 << 7 ) )
   {
      *pDest++ = '-';
      *pDest = 0;
   }
   else
      *pDest = 0;

   //
   // Now, loop through the whole part of the bcd number
   //
   // use lower 6 bits of iSignSpecialPlaces.
   iNumDigits = pBcd->iPrecision - ( pBcd->iSignSpecialPlaces & 63 );
   for( iLoop = 0; iLoop < iNumDigits; iLoop++ )
   {
      CHAR  cVal;

      if( iLoop % 2 )
         // lower 4 bits only
         cVal = ( pBcd -> iFraction[ ( iLoop - 1 ) /  2 ] & 15 ) + '0';
      else
         // upper 4 bits only
         cVal = ( pBcd -> iFraction[ iLoop / 2 ] >> 4 ) + '0';

      //
      // This little test is used to strip leading '0' chars.
      //
      if( bDataStart )
         *pDest++ = cVal;
      else
      if( cVal != '0' )
      {
         *pDest++ = cVal;
         bDataStart = TRUE;
      }
   }

   //
   // If no data is stored yet, add a leading '0'.
   //
   if( !bDataStart )
      *pDest++ = '0';

   *pDest++ = '.';

   //
   // Now, loop through the fractional part
   //
   for( iLoop = iNumDigits; iLoop < pBcd->iPrecision; iLoop++, pDest++ )
   {
      if( iLoop % 2 )
         // lower 4 bits only
         *pDest = ( pBcd -> iFraction[ ( iLoop - 1 ) /  2 ] & 15 ) + '0';
      else
         // upper 4 bits only
         *pDest = ( pBcd -> iFraction[ iLoop / 2 ] >> 4 ) + '0';
   }

   //
   // If trailing char is decimal point, add a '0'.
   //
   if( *(pDest-1) == '.' )
      *pDest++ = '0';

   *pDest = 0;

   //
   // Right trim '0' chars
   //
   iLoop = strlen( pStr );
   pDest = &pStr[ iLoop-1 ];
   while( *pDest == '0' && ( iLoop > 0 ) )
   {
      //
      // If prior char is a decimal point, leave the trailing '0' char.
      //
      if( *(pDest-1) == '.' )
         break;

      *pDest-- = 0;
      iLoop--;
   }

   return DBXERR_NONE;
}

MYSMetaData::MYSMetaData( pSQLConnection pConnection ) : SQLMetaData( )
{
   this->pConnection = pConnection;
   iRefCount = 0;
   lSubType = 0;
   SetPropsToDefault( );
   pszErrorMessage = NULL;
}

MYSMetaData::~MYSMetaData( )
{
   if( pszErrorMessage ) delete [] pszErrorMessage;
   if( pszMetaDatabaseName ) delete [] pszMetaDatabaseName;
   if( pszMetaSchemaName ) delete [] pszMetaSchemaName;
   if( pszMetaDatabaseVersion ) delete [] pszMetaDatabaseVersion;
}

SQLResult MYSMetaData::getErrorMessage( pBYTE pszError )
{
   if( pszError && pszErrorMessage )
      strcpy( (pCHAR) pszError, pszErrorMessage );
   return SQL_SUCCESS;
}

SQLResult MYSMetaData::getErrorMessageLen ( pUINT16 puErrorLen )
{
   if( pszErrorMessage )
      *puErrorLen = strlen( pszErrorMessage );
   else
      *puErrorLen = 0;
   return SQL_SUCCESS;
}

void MYSMetaData::SetPropsToDefault( void )
{
   pszMetaSchemaName = NULL;
   pszMetaDatabaseName = NULL;
   pszMetaDatabaseVersion = NULL;
}

SQLResult MYSMetaData::setOption( eSQLMetaDataOption eDOption, 
        INT32 lValue )
{
   SQLResult rslt = SQL_SUCCESS;
   if( eDOption == eMetaSchemaName )
   {
      pCHAR pTemp = (pCHAR) lValue;
      if( pszMetaSchemaName ) delete [] pszMetaSchemaName;
      pszMetaSchemaName = new CHAR[ strlen( pTemp ) + 1 ];
      strcpy( pszMetaSchemaName, pTemp );
   }
   else if( eDOption == eMetaDatabaseName )
   {
      pCHAR pTemp = (pCHAR) lValue;
      if( pszMetaDatabaseName ) delete [] pszMetaDatabaseName;
      pszMetaDatabaseName = new CHAR[ strlen( pTemp ) + 1 ];
      strcpy( pszMetaDatabaseName, pTemp );
   }
   return rslt;
}

SQLResult MYSMetaData::getOption( eSQLMetaDataOption eDOption, pINT32 plValue,
                           INT16 iMaxLength, pINT16 piLength )
{
   SQLResult rslt = SQL_SUCCESS;
   switch( eDOption )
   {
      case eMetaCatalogName:
         *piLength = (INT16) 0;
         *plValue = 0;
         break;
      case eMetaDatabaseName:
         rslt = pConnection->getOption( eConnHostName, plValue, iMaxLength, piLength );
         break;
      case eMetaDatabaseVersion:
         rslt = pConnection->getOption( eConnServerVersion, plValue, iMaxLength, piLength );
         break;
      case eMetaTransactionIsoLevel:
         rslt = pConnection->getOption( eConnTxnIsoLevel, plValue, iMaxLength, piLength );
         break;
      case eMetaSupportsTransaction:
         *piLength = sizeof( BOOL );
         *plValue = FALSE;
         break;
      case eMetaSupportsTransactions:
         *piLength = sizeof( BOOL );
         *plValue = FALSE;
         break;
      case eMetaMaxColumnsInTable:
        *piLength = sizeof( INT32 );
         *plValue = 0;
         break;
      case eMetaMaxRowSize:
         *piLength = sizeof( INT32 );
         *plValue = 0; 
         break;
      case eMetaObjectQuoteChar:
         rslt = pConnection->getOption( eConnObjectQuoteChar, plValue, sizeof(INT32), piLength);
         break;
      case eMetaProcSupportsCursor:
         *piLength = sizeof( BOOL );
         *plValue = FALSE;
         break;
      case eMetaProcSupportsCursors:
         *piLength = sizeof( BOOL );
         *plValue = FALSE;
         break;
      case eMetaSchemaName:
         if( pszMetaSchemaName )
         {
            *piLength = (INT16) (strlen( pszMetaSchemaName ) + 1);
            strcpy( (pCHAR) plValue, pszMetaSchemaName );
         }
         else
         {
            *piLength = (INT16) 0;
            *plValue = 0;
         }
         break;
      default: 
         rslt = DBXERR_NOTSUPPORTED;
         break;
   }
   return rslt;
}


BOOL MYSMetaData::HasInvalidOption( void )
{
   BOOL bRetval = FALSE;
   switch( eObjectType )
   {
      case eExtTypeView:
      case eExtTypeSynonym:
      case eExtTypeProcedure:
      case eExtTypeRole:
      case eExtTypeUDT:
      case eExtTypeProcParams:
         bRetval = TRUE;
         break;
      case eExtTypeTable:
         bRetval = FALSE;
         break;
      case eExtTypeColumns:
         bRetval = ( eSQLRowId & lSubType ) || 
                   ( eSQLRowVersion & lSubType ) ||
                   ( lSubType > (int) eSQLDefault );
         break;
      case eExtTypeIndices:
         bRetval = lSubType > (int) (eSQLUnique | eSQLPrimaryKey);
         break;
   }
   return bRetval;
}

SQLResult MYSMetaData::BuildQuery( pCHAR * ppszQuery, pCHAR pszScope,
                            pCHAR pszWildCard )
{
   SQLResult rslt = SQL_SUCCESS;
   pCHAR pBase;
   UINT32 iLen = strlen( ExtendedObjectQuery[ eObjectType ] );
   pCHAR pszQuery = NULL;
   if( iLen == 0 )
      rslt = DBXERR_NOTSUPPORTED;
   else
      pBase = ExtendedObjectQuery[ eObjectType ];  // not yet allocated
   if( pszScope && *pszScope )
   {
      iLen = iLen + strlen( pszScope );
      pszQuery = new CHAR[ iLen + 1 ];
      sprintf( pszQuery, pBase, pszScope );
   }
   else
   {
      pszQuery = new CHAR[ iLen + 1 ];
      strcpy( pszQuery, pBase );
   }
   if( !rslt )
      *ppszQuery = pszQuery;
   else
      *ppszQuery = NULL;
   return rslt;
}


SQLResult MYSMetaData::ExecuteQuery( pSQLCursor * ppCursor, pCHAR pszScope,
                            pCHAR pszWildCard )
{
   SQLResult rslt = SQL_SUCCESS;
   MYSMetadataCursor * pObjCursor = NULL;
   SQLCursor * pCursor = NULL;
   SQLCommand * pCommand = NULL;
   pCHAR pszQuery = NULL;
   if( HasInvalidOption( ) )
      rslt = DBXERR_NOTSUPPORTED;
   else if( !pConnection ) 
      rslt = SQL_NULL_DATA;
   else
   {
      rslt = pConnection->getSQLCommand( &pCommand );
      if( !rslt )
      {
         if( lSubType || (int) eObjectType > (int) eExtTypeUDT ) 
            rslt = BuildQuery( &pszQuery, pszScope, pszWildCard );
         else
         {  // ObjectList Cursor
            pszQuery = ObjectQuery[ eObjectType ];
            if( ! (*pszQuery) )
            {
               rslt = DBXERR_NOTSUPPORTED;
               pszQuery = NULL;
            }
         }
         if( !rslt )
            rslt = pCommand->prepare( pszQuery, 0 );
         if( !rslt )
            rslt = pCommand->execute( &pCursor );
         if( rslt )              // otherwise, 
         {                       // pCommand is Released when Cursor is Released
            GetCommandError( pCommand, &pszErrorMessage );
            if( pCommand ) pCommand->Release( );  
         }
         else
         {  // SQLCursor and SQLCommand Released in MYSMetadataCursor::Release()
            pObjCursor = new MYSMetadataCursor( pCursor, pCommand, eObjectType, lSubType, pszScope, pszWildCard );
         }
         if( pObjCursor ) pObjCursor->AddRef( );
         *ppCursor = pObjCursor;
         if( (lSubType || (int) eObjectType > (int) eExtTypeUDT) && pszQuery )  
            delete [] pszQuery;      // else not allocated
      }
   }
   return rslt;
}

SQLResult MYSMetaData::getObjectList( eSQLObjectType eObjType, ppSQLCursor ppCur )
{
   SQLResult rslt = SQL_SUCCESS;
   this->lSubType = 0;
   this->eObjectType = (eExtendedObjectType) eObjType;
   if( (eExtendedObjectType) eObjType == eExtTypeDataType )
   {
      // dataType Object: uses hard-coded value, no cursor from server.
      *ppCur = new MYSMetadataCursor( NULL, NULL, eObjectType, lSubType, NULL, NULL );
      if( *ppCur ) (*ppCur)->AddRef( );
   }
   else
      rslt = ExecuteQuery( ppCur, NULL, NULL );
   return rslt;
}

SQLResult MYSMetaData::getTables( pCHAR pszTableName, UINT32 uTableType, 
                     ppSQLCursor ppCur )
{
   if( uTableType )
      this->lSubType = uTableType;
   else
      this->lSubType = 0 | eSQLTable;
   this->eObjectType = eExtTypeTable;
   return ExecuteQuery( ppCur, NULL, pszTableName );
}

SQLResult MYSMetaData::getProcedures( pCHAR pszProcName, UINT32 uProcType, 
                     ppSQLCursor ppCur )
{
   if( uProcType )
      this->lSubType = uProcType;
   else
      this->lSubType = 0 | eSQLProcedure;
   this->eObjectType = eExtTypeProcedure;
   return ExecuteQuery( ppCur, NULL, pszProcName );
}

SQLResult MYSMetaData::getColumns( pCHAR pszTableName, pCHAR pszColumnName,
                     UINT32 uColType, ppSQLCursor ppCur )
{
   this->lSubType = uColType;
   this->eObjectType = eExtTypeColumns;
   return ExecuteQuery( ppCur, pszTableName, pszColumnName );
}

SQLResult MYSMetaData::getProcedureParams( pCHAR pszProcName, pCHAR pszParamName,
                     ppSQLCursor ppCur )
{
   this->lSubType = 0 | eSQLProcedure;
   this->eObjectType = eExtTypeProcParams;
   return ExecuteQuery( ppCur, pszProcName, pszParamName );
}

SQLResult MYSMetaData::getIndices( pCHAR pszTableName, UINT32 uIndexType,
                     ppSQLCursor ppCur )
{
   this->lSubType = uIndexType;
   this->eObjectType = eExtTypeIndices;
   return ExecuteQuery( ppCur, pszTableName, NULL );
}

UINT32 MYSMetaData::Release()
{
   iRefCount --;
   if( iRefCount == 0 )
      delete this;
   else
      return iRefCount;
   return 0;
}

/*--------------------------------------------------------------
/ class MYSMetadataCursor method implementation
/--------------------------------------------------------------*/

SQLResult MYSMetadataCursor::CalcDataTypeInfo( UINT16 uColumn, pBYTE pData )
{
   SQLResult rslt = DBXERR_NOTSUPPORTED;
   pCHAR pszFullColumn = new CHAR[ MYSMAXCOLUMNLENGTH + 10 ];
   BOOL bIsNull = FALSE;
   rslt = pSourceCursor->getString( (UINT16) MYSFieldTypeCol, pszFullColumn, &bIsNull );
   if( !rslt && !bIsNull )
   {
      INT16 iIndexOfColName;
      pCHAR pszColumnName, pTemp;
      pszColumnName = new CHAR[ strlen( pszFullColumn ) + 1 ];
      strcpy( pszColumnName, pszFullColumn );
      pTemp = strstr( pszColumnName, "(" );
      if( pTemp ) *pTemp = 0;
      iIndexOfColName = getIndexForTypeColumnName( pszColumnName );
      if( iIndexOfColName != UNKNOWN_COL_LENGTH )
      {
         UINT16 iLogFldType = LkUpFldLogType[ iIndexOfColName ];
         UINT16 iValue = 0;
         switch( uColumn )
         {
            case ColObjDataTypeCol:
               *((pINT16) pData ) = (INT16) iLogFldType;
               break;
            case ColObjTypeNameCol:
               strcpy( (pCHAR) pData, pszColumnName );
               break;
            case ColObjSubtypeCol:
               if( iLogFldType == fldINT16 && pSourceCursor )
               {
                  rslt = pSourceCursor->getString( (UINT16) MYSExtra, pszFullColumn, &bIsNull );
                  if( !rslt && (strcmp( pszFullColumn, MYSAutoInc ) == 0) )
                     *((pINT16) pData ) = (INT16) fldstAUTOINC;
                  else
                     *((pINT16) pData ) = (INT16) LkUpFldSubType[ iIndexOfColName ];
               }
               else
                  *((pINT16) pData) = (INT16) LkUpFldSubType[ iIndexOfColName ];
               break;
            case ColObjLenCol:
            case ColObjPrecCol:
               if( iLogFldType == fldZSTRING || iLogFldType == fldBCD )
               {
                  pTemp = strstr( pszFullColumn, "(" );
                  if( pTemp )
                     iValue = atoi( pTemp + 1 );
                  if( iValue )
                     if( iLogFldType == fldZSTRING && uColumn == ColObjLenCol )
                        iValue = INT16( iValue+1 );
               }
               else if( uColumn == ColObjLenCol )
                  iValue = (INT16) LkUpFldLength[ iIndexOfColName ];
               else
                  iValue = (INT16) LkUpFldPrecision[ iIndexOfColName ];
               if( uColumn == ColObjLenCol )
                  *((pINT32) pData ) = (INT32) iValue;
               else
                  *((pINT16) pData ) = iValue;
               break;
            case ColObjScaleCol:
               if( iLogFldType == fldBCD )
               {
                  pTemp = strstr( pszFullColumn, "(" );
                  if( pTemp ) pTemp = strstr( pTemp, "," );
                  if( pTemp )
                     iValue = atoi( pTemp + 1 );
               }
               else
                  iValue = (INT16) LkUpFldScale[ iIndexOfColName ];
               *((pINT16) pData ) = iValue;
               break;
         }
      delete [] pszColumnName;
      }
   }
   delete [] pszFullColumn;
   return rslt;
}

MYSMetadataCursor::MYSMetadataCursor( const pSQLCursor pSource, const pSQLCommand pCommand,
              eExtendedObjectType eType, UINT32 lSubType, pCHAR pszName,
              pCHAR pszWild )
{
   iRefCount = 0;
   iRowIndex = 0;
   iColCount = 0;
   this->pSourceCursor = pSource;
   this->pCommand = pCommand;
   this->eObjectType = eType;
   this->lSubType = lSubType;
   if( pszName && *pszName )
   {
      this->pszNameParam = new CHAR[ strlen( pszName ) + 1 ];
      strcpy( this->pszNameParam, pszName );
   }
   else
      this->pszNameParam = NULL;
   if( pszWild && *pszWild )
   {
      this->pszWildCard = new CHAR[ strlen( pszWild ) + 1 ];
      strcpy( pszWildCard, pszWild );
   }
   else
      pszWildCard = NULL;
   bIsObjectList = ( (lSubType == 0)
            && ((int) eObjectType <= (int) eExtTypeUDT ) );
   pszErrorMessage = NULL;
}

MYSMetadataCursor::~MYSMetadataCursor()
{
   if( pszErrorMessage ) delete [] pszErrorMessage;
   if( pszNameParam ) delete [] pszNameParam;
   if( pszWildCard ) delete [] pszWildCard;
}


UINT32 MYSMetadataCursor::getColLengthFromAction( UINT32 uColumnNumber, BOOL bPrecision )
{
   INT32 iRetval;
   eGetColumnAction eAction;
   if( !bIsObjectList )  
      eAction = ColumnActionType[ eObjectType ] [uColumnNumber - 1 ];
   else
      eAction = ColumnActionType[ 0 ] [uColumnNumber - 1 ];
   iRetval = GetStandardColLengthFromAction( eAction, uColumnNumber, bPrecision );
   if( iRetval == UNKNOWN_COL_LENGTH )
   {
      switch( eAction )
      {
         case eUseParamName:
            iRetval = (UINT32) (strlen( pszNameParam ) + 1 ); 
            break;
         case eCalcDataTypeName:
            iRetval = (UINT32) MYSMAXTYPENAME_SIZE + 1;
            break;
         case eCalcDatabase:
            iRetval = (UINT32) MYSMAXNAMELEN + 1;
            break;
         case eGetFromServer:
         {
            SQLResult rslt = UNKNOWN_COL_LENGTH;
            switch( eObjectType )
            {
               case eExtTypeIndices:
                  switch( uColumnNumber )
                  {
                     case IdxObjSortOrdCol:
                        rslt = pSourceCursor->getColumnLength( (UINT16) MYSColAscDescCol, (pUINT32) &iRetval );
                        break;
                     default:
                        iRetval = MYSMAXCOLUMNLENGTH;
                        break;
                  }
                  break;
               default:
                  iRetval = MYSMAXCOLUMNLENGTH;
                  break;
            }
            break;
         }
         default:
            iRetval = MYSMAXCOLUMNLENGTH;
      }
   }
   if( iRetval && bPrecision ) iRetval --;
   return iRetval;
}

SQLResult MYSMetadataCursor::getColumnCount (pUINT16 puColumns)
{
   *puColumns = 0;
   if( bIsObjectList )
      *puColumns = ObjTblColCount;  // getObjectList(): 4 columns
   else
   {
      switch( eObjectType )
      {
         case eExtTypeTable:
            *puColumns = TblObjColCount;
            break;
         case eExtTypeColumns:
            *puColumns = ColObjColCount;
            break;
         case eExtTypeIndices:
            *puColumns = IdxObjColCount;
            break;
      }
   }
   return SQL_SUCCESS;
}

// verify that uColumnNumber is valid
SQLResult MYSMetadataCursor::checkColumnCount( UINT16 uColumnNumber )
{
   UINT16 iColumns = 0;
   SQLResult rslt = getColumnCount( &iColumns );
   if( uColumnNumber == 0 || uColumnNumber > iColumns )
      rslt = DBXERR_OUTOFRANGE;
   return rslt;
}

// used by ResultSet Metadata methods (getColumnNameLength(), etc.)
// to set hard-coded default values defined in SchConst.h
SQLResult MYSMetadataCursor::getDefaultColumnValues( UINT16 uColumnNumber,
     pUINT16 piNameLength, pCHAR pszColumnName, pUINT16 puType, 
     pUINT16 puSubType, pUINT32 pulLength, pINT16 piPrecision, 
     pINT16 piScale )
{
   SQLResult rslt = checkColumnCount( uColumnNumber );
   if( rslt ) return rslt;
   if( bIsObjectList )
   {
      if( pszColumnName ) strcpy( pszColumnName, ObjTblColName[ uColumnNumber - 1 ] );
      if( piNameLength ) *piNameLength = ObjTblColNameLen[ uColumnNumber - 1 ];
      if( puType ) *puType = ObjTblColType[ uColumnNumber - 1 ];
      if( puSubType ) *puSubType = ObjTblColSubType[ uColumnNumber - 1 ];
      if( piPrecision ) *piPrecision = ObjTblColLen[ uColumnNumber - 1 ];
      if( piScale ) *piScale = ObjTblColScale[ uColumnNumber - 1 ];
      if( pulLength ) *pulLength = ObjTblColLen[ uColumnNumber - 1 ];
   }
   else
   {
      switch( eObjectType )
      {
         case eExtTypeTable:
            if( pszColumnName ) strcpy( pszColumnName, TblObjColName[ uColumnNumber - 1 ] );
            if( piNameLength ) *piNameLength = TblObjColNameLen[ uColumnNumber - 1 ];
            if( puType ) *puType = TblObjColType[ uColumnNumber - 1 ];
            if( puSubType ) *puSubType = TblObjColSubType[ uColumnNumber - 1 ];
            if( piPrecision ) *piPrecision = TblObjColPrec[ uColumnNumber - 1 ];
            if( piScale ) *piScale = TblObjColScale[ uColumnNumber - 1 ];
            if( pulLength ) *pulLength = TblObjColLen[ uColumnNumber - 1 ];
            break;
         case eExtTypeColumns:
            if( pszColumnName ) strcpy( pszColumnName, ColObjColName[ uColumnNumber - 1 ] );
            if( piNameLength ) *piNameLength = ColObjColNameLen[ uColumnNumber - 1 ];
            if( puType ) *puType = ColObjColType[ uColumnNumber - 1 ];
            if( puSubType ) *puSubType = ColObjColSubType[ uColumnNumber - 1 ];
            if( piPrecision ) *piPrecision = ColObjColPrec[ uColumnNumber - 1 ];
            if( piScale ) *piScale = ColObjColScale[ uColumnNumber - 1 ];
            if( pulLength ) *pulLength = ColObjColLen[ uColumnNumber - 1 ];
            break;
         case eExtTypeIndices:
            if( pszColumnName ) strcpy( pszColumnName, IdxObjColName[ uColumnNumber - 1 ] );
            if( piNameLength ) *piNameLength = IdxObjColNameLen[ uColumnNumber - 1 ];
            if( puType ) *puType = IdxObjColType[ uColumnNumber - 1 ];
            if( puSubType ) *puSubType = IdxObjColSubType[ uColumnNumber - 1 ];
            if( piPrecision ) *piPrecision = IdxObjColPrec[ uColumnNumber - 1 ];
            if( piScale ) *piScale = IdxObjColScale[ uColumnNumber - 1 ];
            if( pulLength ) *pulLength = IdxObjColLen[ uColumnNumber - 1 ];
            break;
      }
   }
   if( ((pulLength) && (*pulLength == UNKNOWN_COL_LENGTH)) || 
       ((piPrecision) && (*piPrecision == UNKNOWN_COL_LENGTH)) ) // retrieve from server
   {
      if( eObjectType == eExtTypeDataType )
      {
         if( pulLength ) *pulLength = MYSMAXTYPENAME_SIZE + 1;
         if( piPrecision ) *piPrecision = MYSMAXTYPENAME_SIZE;
      }
      else
      {
         if( pulLength ) *pulLength = getColLengthFromAction( uColumnNumber, FALSE );
         if( piPrecision ) *piPrecision = (UINT16) getColLengthFromAction( uColumnNumber, TRUE );
      }
   }
   return rslt;
}

SQLResult MYSMetadataCursor::getColumnNameLength (
   UINT16  uColumnNumber,
   pUINT16 puLen )
{
   return getDefaultColumnValues( uColumnNumber, puLen, NULL, NULL, NULL, 
                            NULL, NULL, NULL );
}
SQLResult MYSMetadataCursor::getColumnName (
   UINT16 uColumnNumber, 
   pCHAR  pszColumnName)
{
   return getDefaultColumnValues( uColumnNumber, NULL, pszColumnName, NULL, 
                            NULL, NULL, NULL, NULL );
}
SQLResult MYSMetadataCursor::getColumnType (
   UINT16  uColumnNumber,
   pUINT16 puType,
   pUINT16 puSubType )
{
   return getDefaultColumnValues( uColumnNumber, NULL, NULL, puType, puSubType,
                            NULL, NULL, NULL );

}
SQLResult MYSMetadataCursor::getColumnLength (
   UINT16  uColumnNumber,
   pUINT32 pulLength )
{
   SQLResult rslt = getDefaultColumnValues( uColumnNumber, NULL, NULL, 
                               NULL, NULL, pulLength, NULL, NULL );
   if( !rslt )
   {
      if( *pulLength == 0 )
         *pulLength = getColLengthFromAction( uColumnNumber, FALSE );
   }
   return rslt;
}

SQLResult MYSMetadataCursor::getColumnPrecision (
   UINT16 uColumnNumber,
   pINT16 piPrecision )
{
   SQLResult rslt = getDefaultColumnValues( uColumnNumber, NULL, NULL, 
                               NULL, NULL, NULL, piPrecision, NULL );
   if( !rslt )
   {
      if( *piPrecision == UNKNOWN_COL_LENGTH )  // place holders or empty string values
      {
         eGetColumnAction eAction;
         if( (lSubType > 0 ) || (int) eObjectType > (int) eExtTypeUDT )  
            eAction = ColumnActionType[ eObjectType ] [uColumnNumber - 1 ];
         else
            eAction = ColumnActionType[ 0 ] [uColumnNumber - 1 ];
         if( eAction == eGetFromServer )
            *piPrecision = (UINT16) getColLengthFromAction( uColumnNumber, TRUE );
      }
   }
   return rslt;
}

SQLResult MYSMetadataCursor::getColumnScale (
   UINT16 uColumnNumber,
   pINT16 piScale )
{
   SQLResult rslt = getDefaultColumnValues( uColumnNumber, NULL, NULL, NULL, NULL, 
                            NULL, NULL, piScale );
   return rslt;
}

SQLResult MYSMetadataCursor::isNullable (
   UINT16 uColumnNumber,
   pBOOL  pbNullable )
{
   SQLResult rslt = checkColumnCount( uColumnNumber );
   if( !rslt ) 
      *pbNullable = FALSE;
   return rslt;
}
SQLResult MYSMetadataCursor::isAutoIncrement (
   UINT16 uColumnNumber,
   pBOOL  pbAutoIncr )
{
   SQLResult rslt = checkColumnCount( uColumnNumber );
   if( !rslt ) 
      *pbAutoIncr = FALSE;
   return rslt;
}

SQLResult MYSMetadataCursor::isReadOnly (
   UINT16 uColumnNumber,
   pBOOL  pbReadOnly )
{
   SQLResult rslt = checkColumnCount( uColumnNumber );
   if( !rslt ) 
      *pbReadOnly = TRUE;
   return rslt;
}

SQLResult MYSMetadataCursor::isSearchable(
   UINT16 uColumnNumber,
   pBOOL  pbSearchable )
{
   SQLResult rslt = checkColumnCount( uColumnNumber );
   if( !rslt ) 
      *pbSearchable = TRUE;
   return rslt;
}

SQLResult MYSMetadataCursor::next()
{
   SQLResult rslt;
   if( eObjectType == eExtTypeDataType )  // no Source Cursor
      rslt = (iRowIndex < MaxLkUpFldTypes) ? SQL_SUCCESS : DBXERR_EOF;
   else if( pSourceCursor == NULL )
      rslt = DBXERR_EOF;
   else
   {
      rslt = pSourceCursor->next( );
      iColCount++;
      if( rslt && rslt != DBXERR_EOF )
         GetCommandError( pCommand, &pszErrorMessage );
      else
      {
         if( pszWildCard && *pszWildCard )
         {
            BOOL bIsNull;
            CHAR pszName[ MYSMAXCOLUMNLENGTH ];
            strcpy( pszName, "" );
            while( TRUE )
            {
               switch( eObjectType )
               {
                  case eExtTypeTable:
                  case eExtTypeColumns:
                     pSourceCursor->getString( MYSNameCol, pszName, &bIsNull );
                     break;
                  case eExtTypeIndices:
                     pSourceCursor->getString( MYSIndexNameCol, pszName, &bIsNull );
                     break;
               }
               if( ! strcmp( pszName, pszWildCard ) )
                  break;
               else if( pSourceCursor->next( ) )
                  break;
               iColCount++;
            }
         }
         if( lSubType && ( eObjectType != eExtTypeTable ) ) 
         {
            BOOL bValue = FALSE;
            while( !rslt && !bValue )
            {
               switch( eObjectType )
               {
                  case eExtTypeColumns:
                     if( lSubType & eSQLAutoIncr )
                        rslt = calcIsAutoIncr( &bValue );
                     if( !rslt && !bValue && (lSubType & eSQLDefault) )
                        rslt = calcIsDefaultCol( &bValue );
                     break;
                  case eExtTypeIndices:
                     if( (lSubType & eSQLUnique) && (lSubType & eSQLNonUnique) ) 
                        bValue = TRUE;  // both unique and non-unique requested
                     else if( (lSubType & eSQLUnique) )
                        rslt = calcIsIndexUnique( &bValue );
                     else if( (lSubType & eSQLNonUnique) )
                     {
                        rslt = calcIsIndexUnique( &bValue );
                        if( !rslt ) bValue = !bValue;
                     }
                     if( (!bValue) && (!rslt) && (lSubType & eSQLPrimaryKey ) )
                        rslt = calcIsIndexPrimary( &bValue );
                     break;
               }  // if sought type not found, go on to next record.
               if( !rslt && !bValue ) rslt = pSourceCursor->next( );
               iColCount++;
            }
         }
      }
   }
   iRowIndex++;
   return rslt;
}

UINT32 MYSMetadataCursor::Release()
{
   SQLResult rslt = SQL_SUCCESS;
   iRefCount --;
   if( iRefCount == 0 )
      {
      iRowIndex = 0;
      if( pSourceCursor )  // no Source Cursor for hard-coded tables
      {
         rslt = pSourceCursor->Release( );
         if( rslt )
           pCommand->Release( );
         else
           rslt = pCommand->Release( );
      }
      delete this;
      return 0;
   }
   return rslt;
}


SQLResult MYSMetadataCursor::free()
{
   return SQL_SUCCESS;
}

SQLResult MYSMetadataCursor::getShortFromServer(
   UINT16 uColumnNumber,
   pINT16 pData,
   pBOOL  pbIsNull )
{
   SQLResult rslt = DBXERR_NOTSUPPORTED;
   switch( eObjectType )
   {
      case eExtTypeColumns:
         if( uColumnNumber == ColObjNullCol )
         {  // Nullable is a string field
            pCHAR pszNullString = new CHAR[ MYSMAXNAMELEN ];
            rslt = pSourceCursor->getString( MYSNullableCol, pszNullString, pbIsNull );
            if( !rslt )
            {
               if( StrCmpi( MYSNULLABLEString, pszNullString ) == 0 )
                  *pData = (INT16) TRUE;
               else
                  *pData = (INT16) FALSE; 
            }
            delete [] pszNullString;
         }
         break;
      case eExtTypeIndices:
      {
         if( uColumnNumber == IdxObjColPosCol )
         {
            FMTBcd BcdVal;
            char szValue[38];
            rslt = pSourceCursor->getBcd( MYSIndexOrderCol, &BcdVal, pbIsNull );
            if( !rslt && !*pbIsNull )
            {
               BcdToStr( &BcdVal, szValue, 0, 0 );
               *pData = (INT16) (atoi( szValue ));
            }
         }
         break;
      }
   }
   return rslt;
}


SQLResult MYSMetadataCursor::getStringFromServer(
   UINT16 uColumnNumber,
   pCHAR  pData,
   pBOOL  pbIsNull )
{
   SQLResult rslt = DBXERR_NOTSUPPORTED;
   if( bIsObjectList )
      rslt = pSourceCursor->getString( MYSNameCol, pData, pbIsNull );
   else
   {
      switch( eObjectType )
      {
         case eExtTypeTable:  // only 1 field returned
            rslt = pSourceCursor->getString( MYSNameCol, pData, pbIsNull );
            break;
         case eExtTypeColumns:
            rslt = pSourceCursor->getString( MYSNameCol, pData, pbIsNull );
            break;
         case eExtTypeIndices:
         {
            UINT16 uSrcColumn = 0;
            if( uColumnNumber == IdxObjIdxNameCol )
               uSrcColumn = MYSIndexNameCol;
            else if( uColumnNumber == IdxObjColNameCol )
               uSrcColumn = MYSIndexColNameCol;
            else if( uColumnNumber == IdxObjSortOrdCol )
               uSrcColumn = MYSColAscDescCol;
            if( uSrcColumn > 0 )
               rslt = pSourceCursor->getString( uSrcColumn, pData, pbIsNull );
            break;
         }
      }
   }
   return rslt;
}

// for getIndices, sets pData to Ascending or Descending value
SQLResult MYSMetadataCursor::getString (
   UINT16 uColumnNumber,
   pCHAR  pData,
   pBOOL  pbIsNull )
{
   SQLResult rslt = checkColumnCount( uColumnNumber );
   if( !rslt )
   {
      eGetColumnAction eAction;
      if( (lSubType > 0 ) || (int) eObjectType > (int) eExtTypeUDT )  
         eAction = ColumnActionType[ eObjectType ] [uColumnNumber - 1 ];
      else
         eAction = ColumnActionType[ 0 ] [uColumnNumber - 1 ];
      switch( eAction )
      {
         case eGetFromServer:
            if( eObjectType == eExtTypeDataType )
               strcpy( pData, LkUpFldTypeNames[ iRowIndex - 1] );
            else
               rslt = getStringFromServer( uColumnNumber, pData, pbIsNull );
            break;
         case eUseNullString:
            *pbIsNull = FALSE;
            strcpy( pData, NULLSTRING );
            break;
         case eUseParamName:
            *pbIsNull = FALSE;
            strcpy( pData, pszNameParam );
            break;
         case eCalcDataTypeName:
            *pbIsNull = FALSE;
            rslt = CalcDataTypeInfo( uColumnNumber, (pBYTE) pData );
            break;
      }
   }
   return rslt;
}

INT16 MYSMetadataCursor::getIndexForTypeColumnName( pCHAR pszColumnName )
{
   INT16 iRetval = UNKNOWN_COL_LENGTH;
   for( INT16 i = 0; i < MaxLkUpFldTypes; i++ )
   {
      if( StrCmpi( pszColumnName, LkUpFldTypeNames[ i ] ) == 0 )
      {
         iRetval = i;
         break;
      }
   }
   return iRetval;
}

SQLResult MYSMetadataCursor::getShort(
   UINT16 uColumnNumber,
   pINT16 pData,
   pBOOL  pbIsNull )
{
   SQLResult rslt = checkColumnCount( uColumnNumber );
   if( !rslt )
   {
      eGetColumnAction eAction;
      if( (lSubType > 0 ) || (int) eObjectType > (int) eExtTypeUDT )  
         eAction = ColumnActionType[ eObjectType ] [uColumnNumber - 1 ];
      else
         eAction = ColumnActionType[ 0 ] [uColumnNumber - 1 ];
      switch( eAction )
      {
         case eGetFromServer:
            rslt = getShortFromServer( uColumnNumber, pData, pbIsNull );
            break;
         case eCalcTypeInfo:
            *pbIsNull = FALSE;
            rslt = CalcDataTypeInfo( uColumnNumber, (pBYTE) pData );
            break;
         case eCalcColPosition:
            *pbIsNull = FALSE;
            *pData = (INT16) (iColCount - 1);
            break;
      }
   }
   return rslt;
}

SQLResult MYSMetadataCursor::calcIsIndexPrimary( pBOOL pbIsPrimary )
{
   BOOL bIsNull;
   pCHAR pszKeyName = new CHAR[ MYSMAXCOLUMNLENGTH + 1 ];
   SQLResult rslt = pSourceCursor->getString( MYSIndexNameCol, pszKeyName, &bIsNull );
   if( !rslt )
      *pbIsPrimary = StrCmpi( pszKeyName, MYSPRIMARY ) == 0;
   delete [] pszKeyName;
   return rslt;
}
SQLResult MYSMetadataCursor::calcIsIndexUnique( pBOOL pbIsUnique )
{
   BOOL bIsNull;
   SQLResult rslt;
   INT16 iNotUnique = 0;
   FMTBcd BcdVal;
   char szValue[38];
   rslt = pSourceCursor->getBcd( MYSNonUniqueCol, &BcdVal, &bIsNull );
   if( !rslt && !bIsNull )
   {
      BcdToStr( &BcdVal, szValue, 0, 0 );
      iNotUnique = (INT16) (atoi( szValue ));
      *pbIsUnique = (iNotUnique == 0);
   }
   return rslt;
}

SQLResult MYSMetadataCursor::calcIndexType( 
   UINT16 uColumnNumber,
   pINT32 pData,
   pBOOL  pbIsNull )
{
   UINT32 iValue = 0;
   BOOL bValue = FALSE;
   SQLResult rslt = calcIsIndexPrimary( &bValue );
   if( !rslt )
   {
      if( bValue )
         iValue |= eSQLPrimaryKey;
      rslt = calcIsIndexUnique( &bValue );
      if( !rslt )
      {
         if( !bValue )
            iValue |= eSQLNonUnique;
         else
            iValue |= eSQLUnique;
         *pbIsNull = FALSE;
      }
      *pData = iValue;
   }
   return rslt;
}

SQLResult MYSMetadataCursor::calcIsDefaultCol( pBOOL pbIsDefault )
{
   BOOL bIsNull;
   pCHAR pszValue = new CHAR[ MYSMAXCOLUMNLENGTH ];
   SQLResult rslt = pSourceCursor->getString( (UINT16) MYSDefaultCol, pszValue, &bIsNull );
   if( !rslt )
      *pbIsDefault = !bIsNull;
   delete [] pszValue;
   return rslt;
}

SQLResult MYSMetadataCursor::calcIsAutoIncr( pBOOL pbIsAutoInc )
{
   BOOL bIsNull;
   pCHAR pszValue = new CHAR[ MYSMAXCOLUMNLENGTH ];
   SQLResult rslt = pSourceCursor->getString( (UINT16) MYSExtra, pszValue, &bIsNull );
   if( !rslt )
      *pbIsAutoInc = strcmp( pszValue, MYSAutoInc ) == 0;
   delete [] pszValue;
   return rslt;
}

SQLResult MYSMetadataCursor::calcColumnType( 
   UINT16 uColumnNumber,
   pINT32 pData,
   pBOOL  pbIsNull )
{
   UINT32 iValue = 0;
   BOOL bValue;   // no need to initialize
   SQLResult rslt = calcIsAutoIncr( &bValue );
   if( !rslt )
   {
      if( bValue )
         iValue |= eSQLAutoIncr;
      rslt = calcIsDefaultCol( &bValue );
      if( !rslt )
      {
         if( bValue )
            iValue |= eSQLDefault;
         *pData = iValue;
         *pbIsNull = FALSE;
      }
   }
   return rslt;
}
                  
SQLResult MYSMetadataCursor::calcTableType( 
   UINT16 uColumnNumber,
   pINT32 pData,
   pBOOL  pbIsNull )
{
   UINT32 iValue = 0 | eSQLTable;
   *((INT32 *) pData) = iValue; 
   return SQL_SUCCESS;
}

// sets pData as pInt32 containing eSQLTableType/eSQLColumnType/
// eSQLProcedureType or eSQLIndicesType  values ORed.
SQLResult MYSMetadataCursor::calcType(
   UINT16 uColumnNumber,
   pINT32 pData,
   pBOOL  pbIsNull )
{
  SQLResult rslt = SQL_SUCCESS;
  *pbIsNull = FALSE;
  if( eObjectType == eExtTypeTable )
    rslt = calcTableType( uColumnNumber, pData, pbIsNull );
  else if( eObjectType == eExtTypeIndices )
    rslt = calcIndexType( uColumnNumber, pData, pbIsNull );
  else if( eObjectType == eExtTypeColumns )
    rslt = calcColumnType( uColumnNumber, pData, pbIsNull );
  return rslt;
}

SQLResult MYSMetadataCursor::getLong(
   UINT16 uColumnNumber,
   pINT32 pData,
   pBOOL  pbIsNull )
{
   SQLResult rslt = checkColumnCount( uColumnNumber );
   if( !rslt )
   {
      eGetColumnAction eAction;
      if( !bIsObjectList )
         eAction = ColumnActionType[ eObjectType ] [uColumnNumber - 1 ];
      else
         eAction = ColumnActionType[ 0 ] [uColumnNumber - 1 ];
      switch( eAction )
      {
         case eGetRecno:
            *pbIsNull = FALSE;
            *((INT32*)pData) = iRowIndex;
            break;
         case eGetFromServer:
         {
            INT16 iValue;
            rslt = getShortFromServer( uColumnNumber, &iValue, pbIsNull );
            *pData = (INT32) iValue;
            break;
         }
         case eCalcType:
            calcType( uColumnNumber, pData, pbIsNull );
            break;
         case eCalcTypeInfo:  
            *pbIsNull = FALSE;
            CalcDataTypeInfo( uColumnNumber, (pBYTE) pData );
            break;
         default:
            rslt = DBXERR_NOTSUPPORTED;
            break;
      }
   }
   return rslt;
}
SQLResult MYSMetadataCursor::getErrorMessage( pBYTE pszError )
{
   if( pszError && pszErrorMessage )
      strcpy( (pCHAR) pszError, pszErrorMessage );
   return SQL_SUCCESS;
}

SQLResult MYSMetadataCursor::getErrorMessageLen ( pUINT16 puErrorLen )
{
   if( pszErrorMessage )
      *puErrorLen = strlen( pszErrorMessage );
   else
      *puErrorLen = 0;
   return SQL_SUCCESS;
}
