
/****************************************************************************

dbExpress - Cross-Platform, database independent data access interface

Copyright (c) 2000, 2001 Borland Software Corporation

This file may be distributed and/or modified under the terms of the GNU
General Public License (GPL) version 2 as published by the Free Software

Software Foundation and appearing at
http://www.borland.com/kylix/gpl.html.

Licensees holding a valid Borland No-Nonsense License for this Software
may use this file in accordance with such license, which appears in the file
license.txt that came with this Software.

****************************************************************************/

#ifndef MYSSCHEMA_H
#define MYSSCHEMA_H
#include <string.h>
#include <stdio.h>
#include <dbxpress.h>

enum eExtendedObjectType
{
   eExtTypeDatabase  = 0,
   eExtTypeDataType  = 1,
   eExtTypeTable     = 2,
   eExtTypeView      = 3,
   eExtTypeSynonym   = 4,
   eExtTypeProcedure = 5,
   eExtTypeUser      = 6,
   eExtTypeRole      = 7,
   eExtTypeUDT       = 8,
   eExtTypeColumns   = 9,
   eExtTypeProcParams = 10,
   eExtTypeIndices    = 11
};

class MYSMetaData : public SQLMetaData
{
   private:
      UINT32 iRefCount;
      pSQLConnection pConnection;
      eExtendedObjectType eObjectType;      // Metadata Type to be retrieved
      UINT32 lSubType;                      //if 0, an Object Table
      pCHAR pszMetaSchemaName;              //read/write 
      pCHAR pszMetaDatabaseName;            //readonly
      pCHAR pszMetaDatabaseVersion;         //readonly
      pCHAR pszErrorMessage;
      // for internal use
      void SetPropsToDefault( void );
      BOOL HasInvalidOption( void );
      SQLResult GCCSTDC BuildQuery( pCHAR * ppszQuery, pCHAR pszScope,
                            pCHAR pszWildCard );
      SQLResult GCCSTDC ExecuteQuery( pSQLCursor * ppCursor, pCHAR pszScope,
                            pCHAR pszWildCard );

   public:

      INT32  GCCSTDC QueryInterface( const GUID &riid, ppVOID ppv ) { 
/*
        // Without an implementation of QueryInterface() returning valid data in ppv,
        // there is a high risk of crashing if the resulting interface pointer is attempted to be used.
        // The return codes should be equivalent to the COM versions of HRESULT values as
        // clients (Delphi for instance) may automatically check these return values and raise
        // exceptions if they are failures.  I'd recommend an equivalent to S_OK for success,
        // and an equivalent to E_NOINTERFACE for a bad interface cast.
        if (ppv)
        {
          *ppv = dynamic_cast<SQLMetaData *>(this);
          AddRef(); 
          return 0; // change to be a specific success code...  S_OK? 
        }
        else
          return -1; // change to be a specific error code...  E_NOINTERFACE? 
*/
        return 0;
      };
      UINT32 GCCSTDC AddRef( void ) { return ++iRefCount; };
      UINT32 GCCSTDC Release( void );
      SQLResult GCCSTDC  free( );

      MYSMetaData::MYSMetaData( pSQLConnection pConnection );
      MYSMetaData::~MYSMetaData( );
      //Set/Get options for database metadata properties
      SQLResult GCCSTDC  setOption( eSQLMetaDataOption eMOption, INT32 lValue );

      SQLResult GCCSTDC  getOption( eSQLMetaDataOption eDOption, pINT32 plValue,
                           INT16 iMaxLength, pINT16 piLength );

      // Retrieve Metadata Cursors
      SQLResult GCCSTDC  getObjectList( eSQLObjectType eObjType, ppSQLCursor ppCur );

      SQLResult GCCSTDC  getTables( pCHAR pszTableName, UINT32 uTableType, 
                           ppSQLCursor ppCur );

      SQLResult GCCSTDC  getProcedures( pCHAR pszProcName, UINT32 uProcType, 
                           ppSQLCursor ppCur );

      SQLResult GCCSTDC  getColumns( pCHAR pszTableName, pCHAR pszColumnName,
                           UINT32 uColType, ppSQLCursor ppCur );

      SQLResult GCCSTDC  getProcedureParams( pCHAR pszProcName, pCHAR pszParamName,
                           ppSQLCursor ppCur );
   
      SQLResult GCCSTDC  getIndices( pCHAR pszTableName, UINT32 uIndexType,
                           ppSQLCursor ppCur );

     SQLResult GCCSTDC getErrorMessage( pBYTE pszError );

     SQLResult GCCSTDC getErrorMessageLen ( pUINT16 puErrorLen );
};

class MYSMetadataCursor : public SQLCursor
{
   private:
     UINT32 iRefCount;
     pCHAR pszNameParam;                   // holder for TableName/ProcName
     pCHAR pszWildCard;                    // Wildcard holder
     BOOL bIsObjectList;
     eExtendedObjectType eObjectType;
     UINT32 lSubType;                      // stores ORed eSQLSubtype enums
     INT16 iRowIndex;                      // maintain current row for RECNO column
     INT16 iColCount;                      // maintain current row for RECNO column
     SQLCommand * pCommand;                // to be able free SQLCommand 
     SQLCursor * pSourceCursor;            // when done with Source Cursor
     // check if type of Columns is same as type of Source Cursor
     BOOL checkType( UINT16 uColumnNumber, UINT16 iExpType, pUINT16 piType = NULL );
     // verify that uColumnNumber is valid
     SQLResult GCCSTDC  checkColumnCount( UINT16 uColumnNumber );
     // next 5 methods used to 
     // set pData as pInt32 containing eSQLTableType/eSQLColumnType/
     // eSQLProcedureType or eSQLIndicesType  values ORed.
     SQLResult GCCSTDC  calcType( UINT16 uColumnNumber, pINT32 pData, pBOOL  pbIsNull );
     SQLResult GCCSTDC  calcTableType( UINT16 uColumnNumber, pINT32 pData, pBOOL pbIsNull );
     SQLResult GCCSTDC  calcProcedureType( UINT16 uColumnNumber, pINT32 pData, pBOOL pbIsNull ); 
     SQLResult GCCSTDC  calcIndexType( UINT16 uColumnNumber, pINT32 pData, pBOOL pbIsNull );
     SQLResult GCCSTDC  calcIsDefaultCol( pBOOL pbIsDefault );
     SQLResult GCCSTDC  calcIsAutoIncr( pBOOL pbIsAutoInc );
     SQLResult GCCSTDC  calcIsIndexPrimary( pBOOL pbIsPrimary );
     SQLResult GCCSTDC  calcIsIndexUnique( pBOOL pbIsUnique );
     SQLResult GCCSTDC  calcColumnType( UINT16 uColumnNumber, pINT32 pData, pBOOL  pbIsNull );
     // Based on string pszColumnName, returns Index into DataTypeLkUP arrays
     INT16 getIndexForTypeColumnName( pCHAR pszColumnName );
     SQLResult GCCSTDC  CalcDataTypeInfo( UINT16 uColumn, pBYTE pData );
     // for Column and ProcParams: returns # of Column-Length Column
     UINT16 getLengthField( void );
     // used by ResultSet Metadata methods (getColumnNameLength(), etc.)
     // to set hard-coded default values defined in SchConst.h
     SQLResult GCCSTDC  getDefaultColumnValues( UINT16 uColumnNumber,
         pUINT16 piNameLength, pCHAR pszColumnName, pUINT16 puType, 
         pUINT16 piSubType, pUINT32 pulLength, pINT16 piPrecision, 
         pINT16 piScale );
     // for getIndices, sets pData to Ascending or Descending value
     SQLResult GCCSTDC  getAscDescValue( UINT16 uColumn, pCHAR pData, pBOOL pbIsNull );
     // used to get ColumnLength as Int16 from the Column-Length Column
     // for current row, from Source Cursor
     SQLResult GCCSTDC  calcColumnLength( pINT16 piLength );
     SQLResult GCCSTDC  getStringFromServer(
        UINT16 uColumnNumber,
        pCHAR  pData,
        pBOOL  pbIsNull );
     SQLResult GCCSTDC  getShortFromServer(
        UINT16 uColumnNumber,
        pINT16 pData,
        pBOOL  pbIsNull );
     SQLResult GCCSTDC  getLongFromServer(
        UINT16 uColumnNumber,
        pINT32 pData,
        pBOOL  pbIsNull );
     UINT32 getColLengthFromAction( UINT32 uColumnNumber, BOOL bPrecision );
     pCHAR pszErrorMessage;

   public:
      INT32  GCCSTDC QueryInterface( const GUID &riid, ppVOID ppv ) { 
/*
        // Without an implementation of QueryInterface() returning valid data in ppv,
        // there is a high risk of crashing if the resulting interface pointer is attempted to be used.
        // The return codes should be equivalent to the COM versions of HRESULT values as
        // clients (Delphi for instance) may automatically check these return values and raise
        // exceptions if they are failures.  I'd recommend an equivalent to S_OK for success,
        // and an equivalent to E_NOINTERFACE for a bad interface cast.
        if (ppv)
        {
          *ppv = dynamic_cast<SQLCursor *>(this);
          AddRef(); 
          return 0; // change to be a specific success code...  S_OK? 
        }
        else
          return -1; // change to be a specific error code...  E_NOINTERFACE? 
*/
        return 0;
      };
      UINT32 GCCSTDC AddRef( void ) { return ++iRefCount; };
      UINT32 GCCSTDC Release( void );
      SQLResult GCCSTDC  free( );

      MYSMetadataCursor( const pSQLCursor pSource, const pSQLCommand pCommand, 
                       eExtendedObjectType eType, UINT32 iSubType, pCHAR pszName,
                       pCHAR pszWildCard );
      ~MYSMetadataCursor();

      //Metadata access methods
      SQLResult GCCSTDC  getColumnCount (pUINT16 puColumns);
      SQLResult GCCSTDC  getColumnNameLength (
        UINT16  uColumnNumber,
        pUINT16 puLen );
      SQLResult GCCSTDC  getColumnName (
        UINT16 uColumnNumber, 
        pCHAR  pColumnName);
      SQLResult GCCSTDC  getColumnType (
        UINT16  uColumnNumber,
        pUINT16 puType,
        pUINT16 puSubType );
      SQLResult GCCSTDC   getColumnLength (
        UINT16  uColumnNumber,
        pUINT32 pulLength );
      SQLResult GCCSTDC  getColumnPrecision (
        UINT16 uColumnNumber,
        pINT16 piPrecision );
      SQLResult GCCSTDC  getColumnScale (
        UINT16 uColumnNumber,
        pINT16 piScale );
      SQLResult GCCSTDC  isNullable (
        UINT16 uColumnNumber,
        pBOOL  pbNullable );
      SQLResult GCCSTDC  isAutoIncrement (
        UINT16 uColumnNumber,
        pBOOL  pbAutoIncr );
      SQLResult GCCSTDC  isReadOnly (
        UINT16 uColumnNumber,
        pBOOL  pbReadOnly );
      SQLResult GCCSTDC  isSearchable(
        UINT16 uColumnNumber,
        pBOOL  pbSearchable );
  
      SQLResult GCCSTDC  next();
  
//   all others will return errors
      SQLResult GCCSTDC  getString (
        UINT16 uColumnNumber,
        pCHAR  pData,
        pBOOL  pbIsNull );
  
      SQLResult GCCSTDC  getShort (
        UINT16 uColumnNumber,
        pINT16 pData,
        pBOOL  pbIsNull );
  
      SQLResult GCCSTDC  getLong (
        UINT16 uColumnNumber,
        pINT32 pData,
        pBOOL  pbIsNull );

     SQLResult GCCSTDC getErrorMessage( pBYTE pszError );

     SQLResult GCCSTDC getErrorMessageLen ( pUINT16 puErrorLen );
};  
#endif

