
/****************************************************************************

dbExpress - Cross-Platform, database independent data access interface

Copyright (c) 2000, 2001 Borland Software Corporation

This file may be distributed and/or modified under the terms of the GNU
General Public License (GPL) version 2 as published by the Free Software

Software Foundation and appearing at
http://www.borland.com/kylix/gpl.html.

Licensees holding a valid Borland No-Nonsense License for this Software
may use this file in accordance with such license, which appears in the file
license.txt that came with this Software.

****************************************************************************/

#ifndef MYSQLOBJ_H
#define MYSQLOBJ_H

#ifdef LINUX
#include <dlfcn.h>
#include <locale.h>
#include <nl_types.h>
#endif

#ifdef WINDOWS
#include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <assert.h>
#include <string.h>
#include <ctype.h>

#include <dbxpress.h>
#include <sqlx.h>
#include <mysql.h>
#include "myschema.h"

#define MYSQL_MAXPARAMNAME 64
#define MYSQL_MAXHOSTNAME 256

typedef struct Column
{
   UINT16  uColNum;
   pCHAR   pszColName;
   pCHAR   pszTblName;

   UINT16  uLogType;
   UINT16  uLogSubType;
   INT16   iLogPrecision;
   INT16   iLogScale;
   UINT32  uLogLength;

   UINT16  uPhyType;
   INT16   iPrecision;
   INT16   iScale;
   UINT32  uFlags;
   UINT32  uOffset;
   UINT32  uLength;
   UINT32  uNullOffset;
   UINT32  uAttribute;

}Column;

typedef Column* pColumn;

typedef pColumn* ppColumn;

typedef struct Parameter
{
    UINT16            uParamNumber;
    pCHAR             szName[MYSQL_MAXPARAMNAME];
    STMTParamType     eParamType;
    UINT16            uPhyType;
    UINT16            uSubType;
    INT32             lPrecision;
    INT32             lScale;
    pBYTE             pData;
    UINT32            ulLength;
    INT16             iInd;
} Parameter;

typedef Parameter* pParameter;

typedef pParameter* ppParameter;

//MySQL Vendor calls

extern MYSQL*	STDCALL (*pmysql_init) (MYSQL *mysql);
extern MYSQL*	STDCALL (*pmysql_connect) (MYSQL *mysql, const char *host,
		   const char *user, const char *passwd);
extern MYSQL*	STDCALL (*pmysql_real_connect) (MYSQL *mysql, const char *host,
		   const char *user,
		   const char *passwd,
		   const char *db,
		   unsigned int port,
		   const char *unix_socket,
		   unsigned int clientflag);
extern void	STDCALL (*pmysql_close) (MYSQL *sock);
extern int	STDCALL (*pmysql_select_db) (MYSQL *mysql, const char *db);
extern int	STDCALL (*pmysql_query) (MYSQL *mysql, const char *q);
extern int	STDCALL (*pmysql_real_query) (MYSQL *mysql, const char *q,
                   unsigned int length);
extern int	STDCALL (*pmysql_create_db) (MYSQL *mysql, const char *DB);
extern int	STDCALL (*pmysql_drop_db) (MYSQL *mysql, const char *DB);
extern int	STDCALL (*pmysql_shutdown) (MYSQL *mysql);
extern int	STDCALL (*pmysql_dump_debug_info) (MYSQL *mysql);
extern int	STDCALL (*pmysql_refresh) (MYSQL *mysql,
		   unsigned int refresh_options);
extern int	STDCALL (*pmysql_kill) (MYSQL *mysql,unsigned long pid);
extern int	STDCALL (*pmysql_ping) (MYSQL *mysql);
extern char *	STDCALL (*pmysql_stat) (MYSQL *mysql);
extern char *	STDCALL (*pmysql_get_server_info) (MYSQL *mysql);
extern char *	STDCALL (*pmysql_get_client_info) (void);
extern char *	STDCALL (*pmysql_get_host_info) (MYSQL *mysql);
extern unsigned int	STDCALL (*pmysql_get_proto_info) (MYSQL *mysql);
extern MYSQL_RES *	STDCALL (*pmysql_list_dbs) (MYSQL *mysql,const char *wild);
extern MYSQL_RES *	STDCALL (*pmysql_list_tables) (MYSQL *mysql,const char *wild);
extern MYSQL_RES *	STDCALL (*pmysql_list_fields) (MYSQL *mysql, const char *table,
					 const char *wild);
extern MYSQL_RES *	STDCALL (*pmysql_list_processes) (MYSQL *mysql);
extern MYSQL_RES *	STDCALL (*pmysql_store_result) (MYSQL *mysql);
extern MYSQL_RES *	STDCALL (*pmysql_use_result) (MYSQL *mysql);
extern int	STDCALL (*pmysql_options) (MYSQL *mysql,enum mysql_option option,
				     const char *arg);
extern void	STDCALL (*pmysql_free_result) (MYSQL_RES *result);
extern void	STDCALL (*pmysql_data_seek) (MYSQL_RES *result,unsigned int offset);
extern MYSQL_ROW_OFFSET STDCALL (*pmysql_row_seek) (MYSQL_RES *result, MYSQL_ROW_OFFSET);
extern MYSQL_FIELD_OFFSET STDCALL (*pmysql_field_seek) (MYSQL_RES *result,
			      MYSQL_FIELD_OFFSET offset);
extern MYSQL_ROW	STDCALL (*pmysql_fetch_row) (MYSQL_RES *result);
extern unsigned long *	STDCALL (*pmysql_fetch_lengths) (MYSQL_RES *result);
extern MYSQL_FIELD *	STDCALL (*pmysql_fetch_field) (MYSQL_RES *result);
extern unsigned int	STDCALL (*pmysql_escape_string) (char *to,const char *from,
			   unsigned int from_length);
extern void	STDCALL (*pmysql_debug) (const char *debug);

DBIDATE LsDateEncode( UINT16, UINT16, INT16 );
VOID    LsDateDecode( DBIDATE, pUINT16, pUINT16, pINT16 );
INT32   LsTimeEncode( UINT16, UINT16, UINT16,  pTIME );
VOID    LsTimeDecode( TIME, pUINT16, pUINT16,  pUINT16 );

INT32 StringToBcd(
   pCHAR    pSource,
   pFMTBcd  pBcd,
   INT16    iUnits1,
   INT16    iUnits2 );


class MYSQLDriver : public SQLDriver
{
   public:

      INT32  GCCSTDC QueryInterface( const GUID &riid, ppVOID ppv ) {
/*
        // Without an implementation of QueryInterface() returning valid data in ppv,
        // there is a high risk of crashing if the resulting interface pointer is attempted to be used.
        // The return codes should be equivalent to the COM versions of HRESULT values as
        // clients (Delphi for instance) may automatically check these return values and raise
        // exceptions if they are failures.  I'd recommend an equivalent to S_OK for success,
        // and an equivalent to E_NOINTERFACE for a bad interface cast.
        if (ppv)
        {
          *ppv = dynamic_cast<SQLDriver *>(this);
          AddRef(); 
          return 0; // change to be a specific success code...  S_OK? 
        }
        else
          return -1; // change to be a specific error code...  E_NOINTERFACE? 
*/
        return 0;
      };
      UINT32 GCCSTDC AddRef() { return ++iRefCount; };
      UINT32 GCCSTDC Release();

      MYSQLDriver ();
      ~MYSQLDriver();

      SQLResult GCCSTDC  getSQLConnection (
         ppSQLConnection ppConn);

      SQLResult GCCSTDC  setOption (
         eSQLDriverOption  eDrvOption, 
         INT32             lValue );

      SQLResult GCCSTDC  getOption (
         eSQLDriverOption  eDrvOption,
         pINT32            plValue,
         INT16             iMaxLen,
         pINT16            piLen );


   private:

      INT32 iRefCount;
};


class MYSQLConnection : public SQLConnection
{

   public:

      INT32  GCCSTDC QueryInterface( const GUID &riid, ppVOID ppv ) { 
/*
        // Without an implementation of QueryInterface() returning valid data in ppv,
        // there is a high risk of crashing if the resulting interface pointer is attempted to be used.
        // The return codes should be equivalent to the COM versions of HRESULT values as
        // clients (Delphi for instance) may automatically check these return values and raise
        // exceptions if they are failures.  I'd recommend an equivalent to S_OK for success,
        // and an equivalent to E_NOINTERFACE for a bad interface cast.
        if (ppv)
        {
          *ppv = dynamic_cast<SQLConnection *>(this);
          AddRef(); 
          return 0; // change to be a specific success code...  S_OK? 
        }
        else
          return -1; // change to be a specific error code...  E_NOINTERFACE? 
*/
        return 0;
      };
      UINT32 GCCSTDC AddRef() { return ++iRefCount; };
      UINT32 GCCSTDC Release();

      MYSQLConnection ();
      ~MYSQLConnection();

      SQLResult GCCSTDC  connect (
         pCHAR pszServerName,
         pCHAR pszUserName, 
         pCHAR pszPassword );

      SQLResult GCCSTDC  disconnect ();

      SQLResult GCCSTDC  getSQLCommand(
         ppSQLCommand ppComm);

      SQLResult GCCSTDC  setOption (
         eSQLConnectOption eCOption, 
         INT32             lValue );

      SQLResult GCCSTDC  getOption (
         eSQLConnectOption eCOption,
         pINT32            plValue,
         INT16             iMaxLen,
         pINT16            piLen );

      SQLResult GCCSTDC  getErrorMessageLen (
         pUINT16 puErrorLen );

      SQLResult GCCSTDC  getErrorMessage (
         pBYTE pszError );

      SQLResult GCCSTDC  getConnectionHandle (
         MYSQL** ppConnection );

      SQLResult GCCSTDC  sqlTrace (
         pCHAR szMessage,
         UINT16 uMsgId );
     
      SQLResult GCCSTDC  getSQLMetaData (
         ppSQLMetaData ppMeta);

   private:

      MYSQL   mySQL;
      MYSQL   *pConnection;
      pCHAR   pszError;
      CHAR    szHostName[MYSQL_MAXHOSTNAME];

      UINT32  ulBlobSize;
      pfCallBack pCallBack;
      INT32  iCBInfo;
      TRACEDesc traceDesc;
      INT32 iRefCount;

};

typedef MYSQLConnection* pMYSQLConnection;


class MYSQLCommand : public SQLCommand
{
  public:

     INT32  GCCSTDC QueryInterface( const GUID &riid, ppVOID ppv ) { 
/*
        // Without an implementation of QueryInterface() returning valid data in ppv,
        // there is a high risk of crashing if the resulting interface pointer is attempted to be used.
        // The return codes should be equivalent to the COM versions of HRESULT values as
        // clients (Delphi for instance) may automatically check these return values and raise
        // exceptions if they are failures.  I'd recommend an equivalent to S_OK for success,
        // and an equivalent to E_NOINTERFACE for a bad interface cast.
        if (ppv)
        {
          *ppv = dynamic_cast<SQLCommand *>(this);
          AddRef(); 
          return 0; // change to be a specific success code...  S_OK? 
        }
        else
          return -1; // change to be a specific error code...  E_NOINTERFACE? 
*/
        return 0;
     };
     UINT32 GCCSTDC AddRef() { return ++iRefCount; };
     UINT32 GCCSTDC Release();

     MYSQLCommand();
     ~MYSQLCommand();

     SQLResult GCCSTDC  setOption (
        eSQLCommandOption eSOption,
        INT32             lValue );

     SQLResult GCCSTDC  getOption (
        eSQLCommandOption eSOption,
        pINT32            plValue,
        INT16             iMaxLen,
        pINT16            piLen );


     SQLResult GCCSTDC  close ();

     SQLResult GCCSTDC  getErrorMessageLen (
        pUINT16 puErrorLen );

     SQLResult GCCSTDC  getErrorMessage (
        pBYTE pszError );

     SQLResult GCCSTDC  fetch ();

     SQLResult GCCSTDC  extendedFetch (
        pUINT16     puRows );


     SQLResult GCCSTDC  prepare (
        pCHAR  pszSQL,
        UINT16 uParamCout );

     SQLResult GCCSTDC  setParameter (
        UINT16     uParamNumber,
        UINT16     uChildPos,
        STMTParamType ePType,
        UINT16     uLogType,
        UINT16     uSubType,
        INT32      lPrecision,
        INT32      lScale,
        UINT32     ulLength,
        pVOID      pBuffer,
        BOOL       bIsNull );

    SQLResult GCCSTDC  execute (
       ppSQLCursor ppCur);

    SQLResult GCCSTDC  executeImmediate (
       pCHAR pszSQL, 
       ppSQLCursor ppCur);

    SQLResult GCCSTDC  getRowsAffected (
       pINT32 plRows );

    VOID setConnection (
       pMYSQLConnection pConn );

    SQLResult GCCSTDC  numResultColumns (
       pUINT16    puColumns );

    SQLResult GCCSTDC  getColumn ( 
       UINT16 uColumn, 
       ppColumn ppCol);

    SQLResult GCCSTDC  getRowDataLength ( 
       ppUINT32  pDataLen);

    SQLResult GCCSTDC  getRow ( 
       MYSQL_ROW*  pRow);

    SQLResult GCCSTDC  getConnectionHandle ( 
       MYSQL** ppConnection );


   protected:

     SQLResult GCCSTDC  setLogicalTypeInfo( pColumn pCol );

     SQLResult GCCSTDC  exec (
       ppSQLCursor ppCur);

     SQLResult GCCSTDC  parse (
         pBYTE pszSQL );

     SQLResult GCCSTDC  describeColumns(); 

     SQLResult GCCSTDC  getNativeSQL (
        pBYTE pszSQL,
        ppBYTE ppszNativeSQL );

     SQLResult GCCSTDC  getParamQuery (
        ppCHAR  ppParamSQL,
        pUINT32 plLen );

     SQLResult GCCSTDC  getParamsLength (
        pUINT32 pulLength );

     SQLResult GCCSTDC  describeParameters ();
 
     SQLResult GCCSTDC  getParameter (
        UINT16  uParameterNumber,
        UINT16  uChildPos,
        pVOID   pData,
        UINT32  ulLength,
        pINT32  plInd );

     VOID GCCSTDC sqlTrace (
        pCHAR szMessage,
        UINT16 uMsgId );

     SQLResult GCCSTDC  putString (
          pParameter pParam, 
          pVOID      pData, 
          UINT32     ulLength );

     SQLResult GCCSTDC  putBytes (
          pParameter pParam, 
          pVOID      pData, 
          UINT32     ulLength );

     SQLResult GCCSTDC  putShort (
          pParameter pParam, 
          pVOID      pData, 
          UINT32     ulLength );

     SQLResult GCCSTDC  putLong (
          pParameter pParam, 
          pVOID      pData, 
          UINT32     ulLength );

     SQLResult GCCSTDC  putDouble (
          pParameter pParam, 
          pVOID      pData, 
          UINT32     ulLength );

     SQLResult GCCSTDC  putTimeStamp (
          pParameter pParam, 
          pVOID      pData, 
          UINT32     ulLength );

     SQLResult GCCSTDC  putTime (
          pParameter pParam, 
          pVOID      pData, 
          UINT32     ulLength );

      SQLResult GCCSTDC  putDate (
          pParameter pParam, 
          pVOID      pData, 
          UINT32     ulLength );

       SQLResult GCCSTDC  putBcd (
          pParameter pParam, 
          pVOID      pData, 
          UINT32     ulLength );

       SQLResult GCCSTDC  putBlob (
          pParameter pParam, 
          pVOID      pData, 
          UINT32     ulLength );

  private:

     pMYSQLConnection pConn;

     pCHAR         pszSQL;
     MYSQL_RES*    MySQLResult;
     MYSQL_FIELD*  MySQLFields;
     MYSQL_ROW     MySQLRow;
     pUINT32       pDataLength;

     pCHAR         pszError;
     INT32         iError;
     UINT16        uColumns;      
     pColumn       pCols;
     UINT32        uRecordBufferSize;
     UINT16        uParams;
     pParameter    pParamList;
     UINT16        uBoundParams;
     UINT32        ulBlobSize;
     INT32         iRefCount;


};

typedef MYSQLCommand* pMYSQLCommand;


class MYSQLCursor : public SQLCursor
{
   public:

      INT32 GCCSTDC  QueryInterface( const GUID &riid, ppVOID ppv ) { 
/*
        // Without an implementation of QueryInterface() returning valid data in ppv,
        // there is a high risk of crashing if the resulting interface pointer is attempted to be used.
        // The return codes should be equivalent to the COM versions of HRESULT values as
        // clients (Delphi for instance) may automatically check these return values and raise
        // exceptions if they are failures.  I'd recommend an equivalent to S_OK for success,
        // and an equivalent to E_NOINTERFACE for a bad interface cast.
        if (ppv)
        {
          *ppv = dynamic_cast<SQLCursor *>(this);
          AddRef(); 
          return 0; // change to be a specific success code...  S_OK? 
        }
        else
          return -1; // change to be a specific error code...  E_NOINTERFACE? 
*/
        return 0;
      };
      UINT32 GCCSTDC AddRef() { return ++iRefCount; };
      UINT32 GCCSTDC  Release();

      MYSQLCursor(pMYSQLCommand pComm);

      ~MYSQLCursor();

      SQLResult GCCSTDC  getColumnCount (
         pUINT16 puColumns ) ;

      SQLResult GCCSTDC  getColumnNameLength (
         UINT16  uColumnNumber,
         pUINT16 puLength );

      SQLResult GCCSTDC  getColumnName (
         UINT16 uColumnNumber, 
         pCHAR  pColumnName );
      
      SQLResult GCCSTDC  getColumnType (
         UINT16  uColumnNumber,
         pUINT16 puType,
         pUINT16 puSubType );

      SQLResult GCCSTDC   getColumnLength (
         UINT16  uColumnNumber,
         pUINT32 pulLength );

      SQLResult GCCSTDC  getColumnPrecision (
         UINT16  uColumnNumber,
         pINT16  piPrecision );

      SQLResult GCCSTDC  getColumnScale (
         UINT16 uColumnNumber,
         pINT16 piScale );

      SQLResult GCCSTDC  isBlobSizeExact (
         UINT16 uColumnNumber,
         pBOOL  pbBlobSizeExact );

      SQLResult GCCSTDC  isNullable (
         UINT16 uColumnNumber,
         pBOOL  pbNullable );

      SQLResult GCCSTDC  isAutoIncrement (
         UINT16 uColumnNumber,
         pBOOL  pbAutoIncr );

      SQLResult GCCSTDC  isReadOnly (
         UINT16 uColumnNumber,
         pBOOL  pbReadOnly );

      SQLResult GCCSTDC  isSearchable(
         UINT16 uColumnNumber,
         pBOOL  pbSearchable );
       
      SQLResult GCCSTDC  next();

     SQLResult GCCSTDC  getErrorMessageLen (
        pUINT16 puErrorLen );

     SQLResult GCCSTDC  getErrorMessage (
        pBYTE pszError );

      SQLResult GCCSTDC  getString (
         UINT16 uColumnNumber,
         pCHAR  pData,
         pBOOL  pbInd );

      SQLResult GCCSTDC  getShort (
         UINT16 uColumnNumber,
         pINT16 pData,
         pBOOL  pbInd );

      SQLResult GCCSTDC  getLong (
         UINT16 uColumnNumber,
         pINT32 pData,
         pBOOL  pbInd );

      SQLResult GCCSTDC  getDouble (
         UINT16  uColumnNumber,
         pDFLOAT pData,
         pBOOL   pbInd );

      SQLResult GCCSTDC  getBcd (
         UINT16  uColumnNumber,
         pFMTBcd pData,
         pBOOL   pbInd );

      SQLResult GCCSTDC  getTimeStamp (
         UINT16          uColumnNumber,
         pSQLLTIMESTAMP  pData,
         pBOOL           pbInd );

      SQLResult GCCSTDC  getTime (
         UINT16     uColumnNumber,
         pINT32     pData,
         pBOOL      pbInd );

      SQLResult GCCSTDC  getDate (
         UINT16     uColumnNumber,
         pINT32     pData,
         pBOOL      pbInd );

      SQLResult GCCSTDC  getBytes (
         UINT16   uColumnNumber,
         pBYTE    pData,
         pBOOL    pbInd );

      SQLResult GCCSTDC  getBlobSize (
         UINT16  uColumnNumber,
         pUINT32 pulLength,
         pBOOL   pbInd );

      SQLResult GCCSTDC  getBlob (
         UINT16 uColumnNumber,
         pVOID  pData,
         pBOOL  pbInd,
         UINT32 ulLength );

   private:

      pMYSQLCommand pComm;
      INT32 iRefCount;

};

typedef MYSQLCursor* pMYSQLCursor;

#endif

