/****************************************************************************

dbExpress - Cross-Platform, database independent data access interface

Copyright (c) 2000, 2001 Borland Software Corporation

This file may be distributed and/or modified under the terms of the GNU
General Public License (GPL) version 2 as published by the Free Software

Software Foundation and appearing at
http://www.borland.com/kylix/gpl.html.

Licensees holding a valid Borland No-Nonsense License for this Software
may use this file in accordance with such license, which appears in the file
license.txt that came with this Software.

****************************************************************************/


#ifndef DBXPRESS_H
#define DBXPRESS_H

#include "dbxtypes.h"

#ifdef LINUX
#define COMINTF __attribute__((com_interface))  
#define GCCSTDC __attribute__((__stdcall__))  
#endif

#ifdef WINDOWS
#define COMINTF
#define GCCSTDC __stdcall
#endif

typedef abstract SQLDriver*        pSQLDriver;
typedef abstract SQLConnection*    pSQLConnection;
typedef abstract SQLCommand*       pSQLCommand;
typedef abstract SQLCursor*        pSQLCursor;
typedef abstract SQLMetaData*      pSQLMetaData;

typedef pSQLDriver*                ppSQLDriver;
typedef pSQLConnection*            ppSQLConnection;
typedef pSQLCommand*               ppSQLCommand;
typedef pSQLCursor*                ppSQLCursor;
typedef pSQLMetaData*              ppSQLMetaData;

enum eSQLDriverOption 
{
   eDrvBlobSize     = 0,
   eDrvCallBack     = 1,                  
   eDrvCallBackInfo = 2,                  
   eDrvRestrict     = 3                 
};
 
enum eSQLConnectOption 
{
   eConnAutoCommit     = 0,
   eConnBlockingMode   = 1,
   eConnBlobSize       = 2,                   
   eConnRoleName       = 3,                   
   eConnWaitOnLocks    = 4,                   
   eConnCommitRetain   = 5,                   
   eConnTxnIsoLevel    = 6,
   eConnNativeHandle   = 7, 
   eConnServerVersion  = 8,
   eConnCallBack       = 9,
   eConnHostName       = 10,                  
   eConnDatabaseName   = 11,                  
   eConnCallBackInfo   = 12,                  
   eConnObjectMode     = 13,                  
   eConnMaxActiveComm  = 14,                  
   eConnServerCharSet  = 15,                 
   eConnSQLDialect     = 16,
   eConnRollbackRetain = 17,
   eConnObjectQuoteChar = 18                  
}; 

enum eSQLCommandOption 
{
   eCommRowsetSize     = 0,
   eCommBlobSize       = 1,
   eCommBlockRead      = 2,
   eCommBlockWrite     = 3,
   eCommParamCount     = 4,
   eCommNativeHandle   = 5,
   eCommCursorName     = 6,
   eCommStoredProc     = 7,
   eCommSQLDialect     = 8,
   eCommTransactionID  = 9

};

enum eSQLCursorOption 
{
   eCurObjectAttrName  = 0,
   eCurObjectTypeName  = 1,
   eCurParentFieldID   = 2
};

enum eSQLMetaDataOption 
{
   eMetaCatalogName          = 0,    //pCHAR, read/write
   eMetaSchemaName           = 1,    //pCHAR, read/write 
   eMetaDatabaseName         = 2,    //pCHAR, readonly
   eMetaDatabaseVersion      = 3,    //pCHAR, readonly
   eMetaTransactionIsoLevel  = 4,    //eXILType, readonly
   eMetaSupportsTransaction  = 5,    //BOOL, readonly
   eMetaMaxObjectNameLength  = 6,    //INT32, readonly
   eMetaMaxColumnsInTable    = 7,    //INT32, readonly
   eMetaMaxColumnsInSelect   = 8,    //INT32, readonly
   eMetaMaxRowSize           = 9,    //INT32, readonly
   eMetaMaxSQLLength         = 10,   //INT32, readonly
   eMetaObjectQuoteChar      = 11,   //pCHAR, readonly
   eMetaSQLEscapeChar        = 12,   //pCHAR, readonly
   eMetaProcSupportsCursor   = 13,   //BOOL, readonly
   eMetaProcSupportsCursors  = 14,   //BOOL, readonly
   eMetaSupportsTransactions = 15    //BOOL, readonly
};

enum eSQLObjectType
{
   eObjTypeDatabase  = 0,
   eObjTypeDataType  = 1,
   eObjTypeTable     = 2,
   eObjTypeView      = 3,
   eObjTypeSynonym   = 4,
   eObjTypeProcedure = 5,
   eObjTypeUser      = 6,
   eObjTypeRole      = 7,
   eObjTypeUDT       = 8
};

enum eSQLTableType
{
   eSQLTable       = 0x0001,
   eSQLView        = 0x0002,
   eSQLSystemTable = 0x0004,
   eSQLSynonym     = 0x0008,
   eSQLTempTable   = 0x0010,
   eSQLLocal       = 0x0020
};

enum eSQLProcType
{
   eSQLProcedure   = 0x0001,
   eSQLFunction    = 0x0002,
   eSQLPackage     = 0x0004,
   eSQLSysProcedure = 0x0008
};

enum eSQLColType
{
   eSQLRowId       = 0x0001,
   eSQLRowVersion  = 0x0002,
   eSQLAutoIncr    = 0x0004,
   eSQLDefault     = 0x0008
};

enum eSQLIndexType
{
   eSQLNonUnique   = 0x0001,
   eSQLUnique      = 0x0002,
   eSQLPrimaryKey  = 0x0004
};


//Loads the database vendor client and returns a new driver object

extern "C" SQLResult GCCSTDC getSQLDriverINTERBASE( 
             pCHAR pszVendorLib,
             pCHAR pszResFile,
             ppSQLDriver ppDrv );

extern "C" SQLResult GCCSTDC getSQLDriverMYSQL( 
             pCHAR pszVendorLib,
             pCHAR pszResFile,
             ppSQLDriver ppDrv );

extern "C" SQLResult GCCSTDC getSQLDriverDB2( 
             pCHAR pszVendorLib,
             pCHAR pszResFile,
             ppSQLDriver ppDrv );

extern "C" SQLResult GCCSTDC getSQLDriverORACLE( 
             pCHAR pszVendorLib,
             pCHAR pszResFile,
             ppSQLDriver ppDrv );

abstract COMINTF SQLDriver
{
   virtual INT32 GCCSTDC QueryInterface( const GUID &riid, ppVOID ppv ) = 0;
   virtual UINT32 GCCSTDC AddRef( ) = 0;
   //Destructs the SQLDriver object
   virtual UINT32 GCCSTDC Release( ) = 0;

   //Gets a new SQLConnection object
   virtual SQLResult GCCSTDC getSQLConnection (
      ppSQLConnection ppConn) = 0;

   //Set/Get options for database driver level properties
   virtual SQLResult GCCSTDC setOption (
      eSQLDriverOption  eDOption, 
      INT32             lValue )  = 0;

   virtual SQLResult GCCSTDC getOption (
      eSQLDriverOption  eDOption,
      pINT32            plValue,
      INT16             iMaxLength,
      pINT16            piLength )  = 0;

};


//SQLConnection implementation takes care of establishing a connection to a 
//database, getting a SQLCommand object for query and stored procedure
//processing, transactions and to set/get connection level properties

abstract COMINTF SQLConnection
{
   virtual INT32 GCCSTDC QueryInterface( const GUID &riid, ppVOID ppv ) = 0;
   virtual UINT32 GCCSTDC AddRef( ) = 0;
   //Destructs the SQLConnection object
   virtual UINT32 GCCSTDC Release( ) = 0;

   //Attach to a database server
   virtual SQLResult GCCSTDC connect (
      pCHAR pszServerName,
      pCHAR pszUserName, 
      pCHAR pszPassword ) = 0;

   //Detach from a database server
   virtual SQLResult GCCSTDC disconnect () = 0;

   //Destructs the SQLConnection object
  
   //Gets a new SQLCommand object 
   virtual SQLResult GCCSTDC getSQLCommand (
      ppSQLCommand ppComm) = 0;

   //Gets a new SQLMetaData object 
   virtual SQLResult GCCSTDC getSQLMetaData (
      ppSQLMetaData ppMeta) = 0;

   //Set/Get options for database connection level properties
   virtual SQLResult GCCSTDC setOption (
      eSQLConnectOption eCOption, 
      INT32             lValue )  = 0;

   virtual SQLResult GCCSTDC getOption (
      eSQLConnectOption eCOption,
      pINT32            plValue,
      INT16             iMaxLength,
      pINT16            piLength )  = 0;

   //Transaction support
   virtual SQLResult GCCSTDC beginTransaction(
      UINT32 ulTrasnID );

   virtual SQLResult GCCSTDC commit(
      UINT32 ulTrasnID );

   virtual SQLResult GCCSTDC rollback(
      UINT32 ulTrasnID );

   //Error handling
   virtual SQLResult GCCSTDC getErrorMessage (
      pBYTE pszError ) = 0;

   virtual SQLResult GCCSTDC getErrorMessageLen (
      pUINT16 puErrorLen ) = 0;

};


//SQLCommand takes care of processing a query or a stored procedure and
//returns a SQLCursor object if available.Also supports executing a prepared 
//query multiple times with parameter binding, returning multiple cursors 
//from stored procedure

abstract COMINTF SQLCommand
{
   virtual INT32  GCCSTDC QueryInterface( const GUID &riid, ppVOID ppv ) = 0;
   virtual UINT32 GCCSTDC AddRef( ) = 0;
   //Destructs the SQLCommand object
   virtual UINT32 GCCSTDC Release( ) = 0;

   //Set/Get options for command level properties
   virtual SQLResult GCCSTDC setOption (
      eSQLCommandOption eSOption,
      INT32             lValue ) = 0;

   virtual SQLResult GCCSTDC getOption (
      eSQLCommandOption eSOption,
      pINT32            plValue,
      INT16             iMaxLength,
      pINT16            piLength )  = 0;


   //Set/Get  methods for parameter binding support
   virtual SQLResult GCCSTDC setParameter (
      UINT16     uParameterNumber,
      UINT16     uChildPos,
      STMTParamType ePType,
      UINT16     uLogType,
      UINT16     uSubType,
      INT32      lMaxPrecision,
      INT32      lMaxScale,
      UINT32     ulLength,
      pVOID      pBuffer,
      BOOL       bIsNull ) = 0;


   virtual SQLResult GCCSTDC getParameter (
      UINT16  uParameterNumber,
      UINT16  uChildPos,
      pVOID   pData,
      UINT32  ulLength,
      pINT32  plInd ) = 0;


   //Prepare and Execute SQL and Stored Procedure
   virtual SQLResult GCCSTDC prepare (
      pCHAR  pszSQL,
      UINT16 uParamCount ) = 0;

   virtual SQLResult GCCSTDC execute (
     ppSQLCursor ppCur) = 0;


   //Direct execution of Query or Stored Procedure or DDL
   //No parsing will be done
   virtual SQLResult GCCSTDC executeImmediate (
      pCHAR pszSQL, 
      ppSQLCursor ppCur) = 0;


   //A new SQLCursor object will be returned if there is another result set 
   //available from a stored procedure execution
   virtual SQLResult GCCSTDC  getNextCursor (
      ppSQLCursor ppCur);

   //No.of rows affected as a result of INSERT/DELETE/UPDATE
   virtual SQLResult GCCSTDC getRowsAffected (
      pINT32 plRows) = 0;

   //Free resources used for parameter binding
   virtual SQLResult GCCSTDC close () = 0;


   //Error Handling
   virtual SQLResult GCCSTDC getErrorMessage (
      pBYTE pszError ) = 0;

   virtual SQLResult GCCSTDC getErrorMessageLen (
      pUINT16 puErrorLen ) = 0;

};


//SQLCursor holds metadata and data with respect to the executed query
//or stored procedure

abstract COMINTF SQLCursor
{
   virtual INT32  GCCSTDC  QueryInterface( const GUID &riid, ppVOID ppv ) = 0;
   virtual UINT32 GCCSTDC  AddRef( ) = 0;
   //Destructs the SQLCursor object
   virtual UINT32 GCCSTDC  Release( ) = 0;

   //Set/Get options for cursor level properties
   virtual SQLResult GCCSTDC setOption (
      eSQLCursorOption  eCurOption,
      INT32             lValue );

   virtual SQLResult GCCSTDC getOption (
      eSQLCursorOption  eCurOption,
      pINT32            plValue,
      INT16             iMaxLength,
      pINT16            piLength );

   //Error handling
   virtual SQLResult GCCSTDC getErrorMessage (
      pBYTE pszError );

   virtual SQLResult GCCSTDC getErrorMessageLen (
      pUINT16 puErrorLen );

   //Metadata access methods
   virtual SQLResult GCCSTDC getColumnCount (pUINT16 puColumns) = 0;

   virtual SQLResult GCCSTDC getColumnNameLength (
      UINT16  uColumnNumber,
      pUINT16 puLen ) = 0;

   virtual SQLResult GCCSTDC getColumnName (
      UINT16 uColumnNumber, 
      pCHAR  pColumnName) = 0;
      
   virtual SQLResult GCCSTDC getColumnType (
      UINT16  uColumnNumber,
      pUINT16 puType,
      pUINT16 puSubType ) = 0;

   virtual SQLResult GCCSTDC getColumnLength (
      UINT16  uColumnNumber,
      pUINT32 pulLength ) = 0;

   virtual SQLResult GCCSTDC getColumnPrecision (
      UINT16 uColumnNumber,
      pINT16 piPrecision ) = 0;

   virtual SQLResult GCCSTDC getColumnScale (
      UINT16 uColumnNumber,
      pINT16 piScale ) = 0;

   virtual SQLResult GCCSTDC isNullable (
      UINT16 uColumnNumber,
      pBOOL  pbNullable ) = 0;

   virtual SQLResult GCCSTDC  isAutoIncrement (
      UINT16 uColumnNumber,
      pBOOL  pbAutoIncr ) = 0;

   virtual SQLResult GCCSTDC isReadOnly (
      UINT16 uColumnNumber,
      pBOOL  pbReadOnly ) = 0;

   virtual SQLResult GCCSTDC isSearchable(
      UINT16 uColumnNumber,
      pBOOL  pbSearchable ) = 0;

   virtual SQLResult GCCSTDC isBlobSizeExact(
      UINT16 uColumnNumber,
      pBOOL  pbBlobExactSize ); 
      
   //Fetch the next record or the next set of records 
   virtual SQLResult GCCSTDC  next() = 0;


   //Data access methods
   virtual SQLResult GCCSTDC getString (
      UINT16 uColumnNumber,
      pCHAR  pData,
      pBOOL  pbIsNull );

   virtual SQLResult GCCSTDC getShort (
      UINT16 uColumnNumber,
      pINT16 pData,
      pBOOL  pbIsNull );

   virtual SQLResult GCCSTDC getLong (
      UINT16 uColumnNumber,
      pINT32 pData,
      pBOOL  pbIsNull );

   virtual SQLResult GCCSTDC getDouble (
      UINT16 uColumnNumber,
      pDFLOAT  pData,
      pBOOL  pbIsNull );

   virtual SQLResult GCCSTDC getBcd (
      UINT16 uColumnNumber,
      pFMTBcd  pData,
      pBOOL  pbIsNull );

   virtual SQLResult GCCSTDC getTimeStamp (
      UINT16          uColumnNumber,
      pSQLLTIMESTAMP  pData,
      pBOOL           pbIsNull ); 

   virtual SQLResult GCCSTDC getTime (
      UINT16     uColumnNumber,
      pINT32     pData,
      pBOOL      pbIsNull );

   virtual SQLResult GCCSTDC getDate (
      UINT16     uColumnNumber,
      pINT32     pData,
      pBOOL      pbIsNull );

   virtual SQLResult GCCSTDC getBytes (
      UINT16     uColumnNumber,
      pBYTE      pData,
      pBOOL      pbIsNull );

   virtual SQLResult GCCSTDC getBlobSize (
      UINT16  uColumnNumber,
      pUINT32 pulLength,
      pBOOL   bIsNull );

   virtual SQLResult GCCSTDC getBlob (
      UINT16 uColumnNumber,
      pVOID  pData,
      pBOOL  bIsNull,
      UINT32 ulLength );

};

//SQLMetaData returns info. about the various objects in a 
//database or metadata specific to a given database object.

abstract COMINTF SQLMetaData
{
   virtual INT32 GCCSTDC  QueryInterface( const GUID &riid, ppVOID ppv ) = 0;
   virtual UINT32 GCCSTDC AddRef( ) = 0;
   //Destructs the SQLMetaData object
   virtual UINT32 GCCSTDC Release( ) = 0;

   //Set/Get options for database metadata properties
   virtual SQLResult GCCSTDC  setOption (
      eSQLMetaDataOption  eMOption, 
      INT32               lValue ) = 0;

   virtual SQLResult GCCSTDC  getOption (
      eSQLMetaDataOption  eDOption,
      pINT32              plValue,
      INT16               iMaxLength,
      pINT16              piLength ) = 0;

   //Get Object list
   virtual SQLResult GCCSTDC  getObjectList (
      eSQLObjectType eObjType, 
      ppSQLCursor    ppCur) = 0;

   //Get Table list
   virtual SQLResult  GCCSTDC  getTables ( 
      pCHAR pszTableName,
      UINT32 uTableType, 
      ppSQLCursor ppCur) = 0;

   //Get Procedure list
   virtual SQLResult GCCSTDC  getProcedures ( 
      pCHAR pszProcName,
      UINT32 uProcType, 
      ppSQLCursor ppCur) = 0;

   //Get Column metadata for a given table 
   virtual SQLResult  GCCSTDC  getColumns (
      pCHAR pszTableName,
      pCHAR pszColumnName,
      UINT32 uColType, 
      ppSQLCursor ppCur) = 0;

   //Get Parameter metadata for a given Stored Procedure
   virtual SQLResult  GCCSTDC getProcedureParams (
      pCHAR pszProcName,
      pCHAR pszParamName,
      ppSQLCursor ppCur) = 0;
   
   //Get Index info associated with a given table   
   virtual SQLResult GCCSTDC  getIndices (
      pCHAR pszTableName,
      UINT32 uIndexType,
      ppSQLCursor ppCur) = 0;

   //Error handling
   virtual SQLResult GCCSTDC getErrorMessage (
      pBYTE pszError );

   virtual SQLResult GCCSTDC getErrorMessageLen (
      pUINT16 puErrorLen );

};

#endif
