
/****************************************************************************

dbExpress - Cross-Platform, database independent data access interface

Copyright (c) 2000, 2001 Borland Software Corporation

This file may be distributed and/or modified under the terms of the GNU
General Public License (GPL) version 2 as published by the Free Software

Software Foundation and appearing at
http://www.borland.com/kylix/gpl.html.

Licensees holding a valid Borland No-Nonsense License for this Software
may use this file in accordance with such license, which appears in the file
license.txt that came with this Software.

****************************************************************************/


#ifndef DBXTYPES_H
#define DBXTYPES_H

#include <time.h>

#define abstract struct

// Constants

#  define DBXMAXNAMELEN         31            // Name limit (table, field etc)
#  define DBXMAXTRACELEN        1024          // Max trace message length       
#  define FALSE 0
#  define TRUE 1

#define SQL_SUCCESS 0
#define SQL_ERROR  -1
#define SQL_NULL_DATA -1
#define SQL_NO_DATA 100

#ifndef NULL 
# define NULL (0)
#endif

#define VOID    void

typedef char CHAR;         
typedef void* pVOID;
typedef unsigned char       BYTE;

#define INT8    char
#define UINT8   unsigned char
#define INT16   short
#define UINT16  unsigned short
#define INT32   long
#define UINT32  unsigned long

typedef int BOOL;
typedef unsigned char BOOL8;
typedef short         BOOL16;
typedef double    DFLOAT;
typedef long      DBIDATE;
typedef long      TIME;
typedef struct tm CTIME;
typedef struct tm* pCTIME;


typedef struct
{
   INT16   year;
   UINT16  month;
   UINT16  day;
   UINT16  hour;
   UINT16  minute;
   UINT16  second;
   UINT32  fraction;

} CTIMESTAMP; 
 
typedef CTIMESTAMP* pCTIMESTAMP;
typedef CTIMESTAMP  SQLLTIMESTAMP;
typedef SQLLTIMESTAMP*  pSQLLTIMESTAMP;

typedef struct
{
   INT16   year;
   UINT16  month;
   UINT16  day;

} SQLLDATE; 
 
typedef SQLLDATE* pSQLLDATE;

typedef struct
{
   UINT16  hour;
   UINT16  minute;
   UINT16  second;
   UINT32  fraction;

} SQLLTIME; 

typedef SQLLTIME* pSQLLTIME;

typedef struct
{
   BYTE iPrecision;          //1..64 valid range
   BYTE iSignSpecialPlaces;  //sign:1, special:1, places:6
   BYTE iFraction[32];       //bcd nibbles, 00..99 ber byte, high nibbles 1s

} FMTBcd;

typedef FMTBcd* pFMTBcd;

typedef void** ppVOID;
typedef char*  pCHAR;
typedef char** ppCHAR;
typedef unsigned char** ppBYTE;
typedef const char* pCCHAR;
typedef unsigned char* pBYTE;
typedef short* pINT16;
typedef unsigned short *pUINT16;
typedef char* pINT8;
typedef unsigned char* pUINT8;
typedef long* pINT32;
typedef unsigned long* pUINT32;
typedef unsigned long** ppUINT32;
typedef unsigned char* pBOOL8;
typedef int* pBOOL;
typedef short* pBOOL16;
typedef double* pDFLOAT;
typedef double* pFLOAT;
typedef long* pDBIDATE;
typedef long* pTIME;
typedef double* pTIMESTAMP;


#define DATE      DBIDATE
#define pDATE     pDBIDATE

#ifndef GUID_DEFINED
#define GUID_DEFINED
typedef struct tagGUID
{
   UINT32 p1;
   UINT16 p2;
   UINT16 p3;
   BYTE p4[8];
} GUID;
#endif

typedef INT32  SQLResult;

// Driver Types
#define szORACLE        "ORACLE"
#define szDB2           "DB2"
#define szINTERBASE     "INTERBASE"
#define szMYSQL         "MYSQL"


// Field Types (Logical)
#define fldUNKNOWN      0
#define fldZSTRING      1                 // Null terminated string
#define fldDATE         2                 // Date     (32 bit)
#define fldBLOB         3                 // Blob
#define fldBOOL         4                 // Boolean  (16 bit)
#define fldINT16        5                 // 16 bit signed number
#define fldINT32        6                 // 32 bit signed number
#define fldFLOAT        7                 // 64 bit floating point
#define fldBCD          8                 // BCD
#define fldBYTES        9                 // Fixed number of bytes
#define fldTIME         10                // Time        (32 bit)
#define fldTIMESTAMP    11                // Time-stamp  (64 bit)
#define fldVARBYTES     15                // Length prefixed var bytes
#define fldCURSOR       17                // For Oracle Cursor type

// Object relational data types
#define fldADT          20                // ADT (Abstract data type)
#define fldARRAY        21                // VARRAY (Variable length array)
#define fldREF          22                // REF (Reference)
#define fldTABLE        23                // NESTED TABLE
#define fldDATETIME     24                // DATE TIME

#define MAXLOGFLDTYPES  25                // Number of logical fieldtypes


// Sub Types (Logical)

// fldZSTRING subtype
#define fldstFIXED         31             // CHAR

// fldFLOAT subtype
#define fldstMONEY         21             // Money

// fldBLOB subtypes
#define fldstMEMO          22             // Text Memo
#define fldstBINARY        23             // Binary data
#define fldstHMEMO         33             // CLOB
#define fldstHBINARY       34             // BLOB
#define fldstBFILE         36             // BFILE

// fldINT32 subtype
#define  fldstAUTOINC      29             // Auto Increment

// fldADT subtype
#define fldstADTNestedTable  35          // ADT for nested table 

// fldDATE subtype
#define fldstADTDATE       37            // DATE (OCIDate ) with in an ADT

typedef enum STMTFreeType
{
   stmtCLOSE    = 0, 
   stmtDROP     = 1
 
} STMTFreeType;

typedef enum STMTParamType
{
   paramUNKNOWN    = 0, // UNKNOWN (Error)
   paramIN         = 1, // Input parameter
   paramOUT        = 2, // Output parameter
   paramINOUT      = 3, // Input/Output parameter
   paramRET        = 4  // Procedure (or function) return

} STMTParamType;

typedef enum         // trace categories
{
   traceUNKNOWN   = 0x0000,
   traceQPREPARE  = 0x0001,   // prepared query statements
   traceQEXECUTE  = 0x0002,   // executed query statements
   traceERROR     = 0x0004,   // vendor errors
   traceSTMT      = 0x0008,   // statement ops (i.e. allocate, free)
   traceCONNECT   = 0x0010,   // connect / disconnect
   traceTRANSACT  = 0x0020,   // transaction
   traceBLOB      = 0x0040,   // blob i/o
   traceMISC      = 0x0080,   // misc.
   traceVENDOR    = 0x0100,   // vendor calls
   traceDATAIN    = 0x0200,   // parameter bound data
   traceDATAOUT   = 0x0400    // trace fetched data

} TRACECat;

typedef TRACECat *pTRACECat;

typedef struct       // trace callback info
{
   CHAR        szTrace[DBXMAXTRACELEN];// Trace message
   TRACECat    eTraceCat;              // Trace category
   INT32       iCBInfo;                // Client callback data
   UINT16      uTotalMsgLen;           // Trace message length

} TRACEDesc;

typedef TRACEDesc *pTRACEDesc;

// Transaction support
typedef enum                        // Transaction isolation levels
{
   xilREADCOMMITTED,                // Committed changes, no phantoms
   xilREPEATABLEREAD,               // Full read repeatability
   xilDIRTYREAD,                    // Uncommitted changes read
   xilCUSTOM                        // Database specific custom isolation level
} eXILType;

typedef enum                        // Transaction end control
{
   xendCOMMIT,                      // Commit transaction
   xendCOMMITKEEP,                  // Commit transaction, keep cursors
   xendABORT                        // Rollback transaction
} eXEnd;

typedef enum                         // Call back type
{
   cbTRACE        = 1               // Trace
} CBType;

typedef CBType* pCBType;

typedef enum                              // Call-back return type
{
   cbrUSEDEF        = 0,                  // Take default action
   cbrCONTINUE      = 1,                  // Continue
   cbrABORT         = 2,                  // Abort the operation
   cbrCHKINPUT      = 3,                  // Input given
   cbrYES           = 4,                  // Take requested action
   cbrNO            = 5,                  // Do not take requested action
   cbrPARTIALASSIST = 6,                  // Assist in completing the job
   cbrSKIP          = 7,                  // Skip this operation
   cbrRETRY         = 8                   // Retry this operation
} CBRType;

typedef CBRType* pCBRType;

typedef CBRType ( * pfCallBack)           // Call-back funtion pntr type
(
   CBType         ecbType,                // Callback type
   pVOID          pCbInfo                 // Call back info/Client Input
);

typedef pfCallBack* ppfCallBack;


//Error Codes

#define DBXERR_NONE                    0x0000
#define DBXERR_NOMEMORY                0x0001
#define DBXERR_INVALIDFLDTYPE          0x0002
#define DBXERR_INVALIDHNDL             0x0003
#define DBXERR_INVALIDTIME             0x0004
#define DBXERR_NOTSUPPORTED            0x0005  
#define DBXERR_INVALIDXLATION          0x0006
#define DBXERR_INVALIDPARAM            0x0007
#define DBXERR_OUTOFRANGE              0x0008
#define DBXERR_SQLPARAMNOTSET          0x0009
#define DBXERR_EOF                     0x000A
#define DBXERR_INVALIDUSRPASS          0x000B
#define DBXERR_INVALIDPRECISION        0x000C  
#define DBXERR_INVALIDLEN              0x000D
#define DBXERR_INVALIDTXNISOLEVEL      0x000E 
#define DBXERR_INVALIDTXNID            0x000F 
#define DBXERR_DUPLICATETXNID          0x0010 
#define DBXERR_DRIVERRESTRICTED        0x0011 
#define DBXERR_LOCALTRANSACTIVE        0x0012 

#endif

