
/****************************************************************************

dbExpress - Cross-Platform, database independent data access interface

Copyright (c) 2000, 2001 Borland Software Corporation

This file may be distributed and/or modified under the terms of the GNU
General Public License (GPL) version 2 as published by the Free Software

Software Foundation and appearing at
http://www.borland.com/kylix/gpl.html.

Licensees holding a valid Borland No-Nonsense License for this Software
may use this file in accordance with such license, which appears in the file
license.txt that came with this Software.

****************************************************************************/

#ifndef SQLX_H
#define SQLX_H

#include <dbxrc.h>
#include <dbxtypes.h>

#define SQLX_SUCCESS            0
#define SQLX_OK                 0
#define SQLX_SUCCESS_WITH_INFO  1
#define SQLX_NO_DATA_FOUND      100
#define SQLX_ERROR              -1     // Generic error
#define SQLX_INVALID_HANDLE     -2
#define SQLX_NOMEMORY           -3
#define SQLX_CONNECTFAIL        -4     // Failure to connect
#define SQLX_MULTIRESULTS       -5     // Multiple result sets
#define SQLX_CONNECTINUSE       -6     // Half-duplex connection in use
#define SQLX_INITFAIL           -7     // init failure
#define SQLX_SQLCOMMLOST        -8     // Lost communication w/ SQL server
#define SQLX_MULTICONNECT       -9     // Multiple connection attempted
#define SQLX_NOTSUPPORTED       -10    // Not supported
#define SQLX_BUSY               -11    // busy
#define SQLX_CANCELLED          -12    // operation cancelled
#define SQLX_INVALIDPARAM       -13    // invalid param
#define SQLX_INVALIDPARAMLEVEL  -14    // for Ora SP param level > 0
#define SQLX_UNKNOWNDATATYPE    -15    // unknown data type
#define SQLX_OPTRECLOCKRECDEL   -16    // optimistic lock fail because record deleted
#define SQLX_INVALIDREF         -17    // invalid ref
#define SQLX_NOSUCHTABLE        -18    // no such table

#define SQLX_PERMITS_NULLS      1
#define SQLX_NO_NULLS           0
#define SQLX_UNKNOWN            -1

#define SQLX_CLOSE              0
#define SQLX_DROP               1

#define SQLX_NULL_DATA          -1
#define SQLX_TRUNCATION         -2

#define SQLX_MAXCOLNAM          64


#define SQLVendorTrace(pTraceDesc, pszTraceMsg, MsgId, pfCallBack) \
   if (pTraceDesc && pfCallBack) \
      { pTraceDesc->eTraceCat = traceVENDOR; \
        SQLTrace ( pTraceDesc, pszTraceMsg, MsgId, FALSE, pfCallBack); }

////////////////////////////////////////////////////////////////////////////////
//   T Y P E D E F S
////////////////////////////////////////////////////////////////////////////////

typedef INT16       SQLX_RC;            // SQLX return code

typedef pVOID       SQLX_HDBC;          // database connection handle
typedef pVOID       SQLX_HSTMT;         // prepared statment handle
typedef pVOID       SQLX_HBLOB;         // blob handle


// The stmt attributes given reflect what may happen, not necessarily what
// will happen.  Thus, the attributes are given to indicate to SQLX what it
// is required to prepare for.  These have been invented to address
// server differences in statement, cursor, and transaction interdependencies.
// For example, in Sybase, overlapped cursors must be given separate DB
// connections, since only one active statement may exist per DB connection.
// InterBase and some other servers cause cursor closes at transaction
// commit, so statements that SQLD cannot shutdown and restart (due to
// lack of record addressability) must be protected from this.

typedef enum
   {
      stmtNONCURSOR,             // DDL, modification DML, xact/locking, etc...
      stmtNONOVERLAPPEDCURSOR,   // Immediately consumed cursor-oriented stmt
      stmtOVERLAPPEDCURSOR,      // Stmt may be active over future stmt execs
      stmtOVERLAPPEDXACT         // Stmt may be active over xact ends
   } SQLX_STMTType;

typedef enum
   {
      xoBLOBDATA,                // Blob data returned at QExec
      xoAUTOCOMMIT,              // auto commit mode
      xoTIMEOUT,                 // Statement timeout
      xoCONNECTTIMEOUT,          // Connection timeout
      xoTRANALLOWED,             // Informix db with logging
      xoTRANANSI,                // Informix db with logging mode ANSI
      xoXIL,                     // Transaction isolation level
      xoONLINE,                  // Informix specific OnLine vs. SE db
      xoSTMTTYPE,                // Stmt type
      xoSERVER_VERSION,          // Server version number
      xoCLIENT_VERSION,          // Client version number
      xoCURSORNAME,              // Cursor name
      xoASYNCHRONOUS,            // Asynchronous query exec.
      xoMAXROWSETSIZE,           // Maximum rowset size for block operations
      xoROWSETSIZE,              // Rowset size for block fetches
      xoBLOCKREAD,               // Block Fetch
      xoBLOCKWRITE,              // Block Write
      xoPARAMCOUNT,              // Number of parameters
      xoOUTPARAMCOUNT,           // Number of output parameters
      xoSTOREDPROCEDURE,         // Statement is stored procedure
      xoNATIVECONNECTHNDL,       // Native connection handle
      xoNATIVESTMTHNDL,          // Native stmt handle
      xoCONNECTACTIVE,           // Connection is active
      xoERROREXISTS,             // Native error exists
      xoSQLTRACEDESC,            // SQL trace desc
      xoLHOST,                   // MSSQL/Sybase client workstation name
      xoLAPP,                    // MSSQL/Sybase client application name
      xoLNATLANG,                // MSSQL/Sybase client National lang name
      xoTDSPACKETSIZE,           // MSSQL/Sybase TDS packet size
      xoCSCURSORROWS,            // Sybase CT-Lib CS_CURSOR_ROWS value
      xoDBNLS,                   // Informix DBNLS setting
      xoCOLLCHAR,                // Informix COLLCHAR setting
      xoNATIVESQLCA,             // Informix SQLCA structure
      xoEXECASCURSOR,            // Sybase CTLIB: Execute as a ct_cursor()
      xoTYPEFLAG_BCD,            // Oracle: NUMBER <-> strings.
      xoTYPEFLAG_ORAINT,         // oracle: NUMBER <-> integers.
      xoBLOBSIZE,                // Sybase/MSSQL : in query blob size.
      xoMAXDBPROCS,              // Sybase(DBLIB)/MSSQL max. no. of DBPROCESSes
      xoOBJECTMODE,              // ORACLE 8 object mode
      xoNESTEDCURSOR,            // ORACLE 8 Nested table cursor
      xo2PCSUPPORT,              // 2 PC support
      xo2PCTRANSID,              // 2 PC Transaction Id
      xoROLENAME,                // Interbase Role Name
      xoLOCKWAIT,                // IB lock resolution 
      xoCOMMITRETAIN,            // IB commit mode for explicit transactions
      xoCANCELQUERY,             // MSSQL:Use dbcanquery() instead of dbcancel()
      xoCALLBACK,                // Register CallBack
      xoCALLBACKINFO,            // Client CallBack Info
      xoSQLDIALECT,              // Interbase SQL Dialect
      xoTRANSID                  // Interbase transaction ID
   } SQLX_OPTION;

typedef enum
   {
      xtSTATUS,                  // Transaction status
      xtBEGIN,                   // Begin transaction
      xtCOMMIT,                  // Commit transaction
      xtCOMMIT_ALL,              // Commit all commitable transactions
      xtCOMMIT_KEEP,             // Commit, retain locks & cursors
      xtROLLBACK,                // Rollback transaction
      xtROLLBACK_KEEP            // Rollback, retain locks & cursors
   } SQLX_TCTL;

////////////////////////////////////////////////////////////////////////////////

INT32 SQLTrace (
   TRACEDesc         *pTraceDesc,       // Trace callback desc
   pCHAR             pszTraceMsg,       // Trace message
   UINT32            uMsgId,            // Trace Id
   BOOL              bNative,           // Native characters in message
   pfCallBack        pCallBack          // Client Callback
);

SQLX_RC SQLX_Startup (                 // SQLX startup thread
   pCHAR             pszVendInit,      // vendor init
   pCHAR             pszResFile        // Resource
);
SQLX_RC SQLX_Shutdown (                // SQLX shutdown thread
   void                                // no args
);

SQLX_RC SQLX_TaskInit (                // SQLX task initialization
   void                                // no args
);
SQLX_RC SQLX_TaskExit (                // SQLX task exit
   void                                // no args
);


///////////////////////
// connection functions
SQLX_RC SQLX_AllocConnect (             // allocate a connection handle
    SQLX_HDBC       *phDBC              // returned.  db connection handle
);
SQLX_RC SQLX_FreeConnect (              // free a connection handle
    SQLX_HDBC       hDBC                // db connection handle
);
SQLX_RC SQLX_Connect (                  // establish a connection
    SQLX_HDBC       hDBC,               // db connection handle
    pCHAR           pDbe,
    INT16           iDbeLen,
    pCHAR           pUid,
    INT16           iUidLen,
    pCHAR           pPwd,
    INT16           iPwdLen
);

SQLX_RC SQLX_Disconnect (               // close the connection
    SQLX_HDBC       hDBC                // db connection handle
);

SQLX_RC SQLX_SetConnectOption (         // Set connection options
    SQLX_HDBC       hDBC,
    SQLX_OPTION     cOption,            // Option id
    INT32           iValue              // On/Off/number
);

SQLX_RC SQLX_GetConnectOption (         // Get connection options
    SQLX_HDBC       hDBC,
    SQLX_OPTION     cOption,            // Option id
    pINT32          piLen,              // Length of result
    pINT32          piValue             // Result
);

SQLX_RC SQLX_Error (                    // return error information
    SQLX_HDBC       hDBC,               // db connection handle
    SQLX_HSTMT      hSTMT,              // statement handle
    pCHAR           pszSqlState,        // SQL State, if supported
    pUINT32         puNativeError,      // returned.  native error code
    pCHAR           pszNativeError,     // returned.  native error msg
    INT16           iErrorBufSize,      // size of error buffer
    pINT16          piErrorBufSize,     // returned.  bytes writen to buffer
    pBOOL           pbLastError         // returned.  this is the last error
);

SQLX_RC SQLX_Transact (
    SQLX_HDBC       hDBC,               // db connection handle
    SQLX_TCTL       eTCmd               // transaction control command
);

SQLX_RC SQLX_Transactions (
    SQLX_HDBC       hDBC,               // db connection handle
    SQLX_TCTL       eTCmd,              // transaction control command
    UINT32          ulTransID,          // transaction ID
    UINT32          uGID,               // XA Global ID (ORACLE 8i specific)
    eXILType        eTransIsoLevel      // transaction Isolation level
);

/////////////////////
// statement functions
SQLX_RC SQLX_SetStmtOption (            // Set stmt options
    SQLX_HSTMT      hSTMT,
    SQLX_OPTION     sOption,            // Option id
    INT32           iValue              // On/Off/number
);

SQLX_RC SQLX_GetStmtOption (            // Set stmt options
    SQLX_HSTMT      hSTMT,
    SQLX_OPTION     sOption,            // Option id
    pINT32          piLen,              // Length of result value
    pINT32          piValue             // returned. value
);

SQLX_RC SQLX_AllocStmt (                // allocate a statement handle
    SQLX_HDBC       hDBC,               // db connection handle
    SQLX_STMTType   eStmtType,          // NONCLONEABLE / CLONEABLE
    SQLX_HSTMT      *phSTMT             // returned.  statement handle
);

SQLX_RC SQLX_FreeStmt (                 // end processing on a statement handle
    SQLX_HSTMT      hSTMT,              // statement handle
    UINT16          uOption             // end option
);

SQLX_RC SQLX_Prepare (                  // prepare a statment for execution
    SQLX_HSTMT      hSTMT,              // statement handle
    pCHAR           pszSQL,             // sql statement
    INT32           iSQLLen,            // sql statement length
    BOOL            bPassthru           // Is this a pass through SQL
);

////////////////////
// execute functions
SQLX_RC SQLX_SetParam (                 // bind a value to a parameter marker
    SQLX_HSTMT      hSTMT,              // statement handle
    UINT16          uParam,             // param number
    const CHAR     *pszName,            // param name (if applicable)
    UINT16          uPHYType,           // physical datatype (not CLI)
    INT16           iXXX,               // N/A
    UINT32          uPrecision,         // "iUnits1"
    INT16           iScale,             // "iUnits2"
    INT32           iLength,            // my extra data (not CLI)
    pVOID           pBuff,
    pINT16          piLength,
    STMTParamType   eParamType
);

SQLX_RC SQLX_SetObjParam (              // bind an object to a parameter marker
    SQLX_HSTMT      hSTMT,              // statement handle
    UINT16          uParam              // param number
);

SQLX_RC SQLX_SetObjectParam (           // bind an object to a parameter marker
    SQLX_HSTMT      hSTMT,              // statement handle
    UINT16          uParam,             // param number
    const CHAR     *pszName,            // param name (if applicable)
    UINT16          uPHYType,           // physical datatype (not CLI)
    const CHAR     *pszTypeName,        // Object Type Name
    INT16           iExactAttr,         // Attribute index with resp to parent
    INT16           iAttr,              // Attribute index
    UINT32          uPrecision,         // "iUnits1"
    INT16           iScale,             // "iUnits2"
    INT32           iLength,            // size of object
    INT32           iInd,               // size of NULL indicator
    pVOID           pBuff,
    pINT16          piLength,
    pINT16          piOffset,
    STMTParamType   eParamType
);

SQLX_RC SQLX_SetAttrParam (             // bind an object attr
    SQLX_HSTMT      hSTMT,              // statement handle
    UINT16          uParam,             // param number
    const CHAR     *pszName,            // param name (if applicable)
    UINT16          uPHYType,           // physical datatype (not CLI)
    const CHAR     *pszTypeName,        // Object Type Name
    INT16           iExactAttr,         // Attribute index with resp to parent
    INT16           iAttr,              // Attribute index
    UINT32          uPrecision,         // "iUnits1"
    INT16           iScale,             // "iUnits2"
    INT32           iLength,            // size of object
    INT32           iInd,               // size of NULL indicator
    pVOID           pBuff,
    pINT16          piLength,
    pINT16          piOffset,
    STMTParamType   eParamType
);

SQLX_RC SQLX_ConstructObjParam (        // construct object param list
    SQLX_HSTMT      hSTMT,              // statement handle
    UINT16          uParam,             // param number
    const CHAR     *pszName,            // param name (if applicable)
    UINT16          uPHYType,           // physical datatype (not CLI)
    const CHAR     *pszTypeName,        // Object Type Name
    INT16           IsObject,           // Is this an object
    INT16           iAttr,              // Attribute index
    INT16           iCol,               // Attribute column no.
    UINT32          uPrecision,         // "iUnits1"
    INT16           iScale,             // "iUnits2"
    INT32           iLength,            // size of object
    INT32           iInd,               // size of NULL indicator
    pVOID           pBuff,
    pINT16          piLength,
    pINT16          piOffset,
    STMTParamType   eParamType
);

SQLX_RC SQLX_Execute (                  // execute a prepared statment
    SQLX_HSTMT      hSTMT,              // statement handle
    pBOOL           pbNoWait            // asynch. query execution?
);

SQLX_RC SQLX_ExecDirect (               // execute a statment 
    SQLX_HSTMT      hSTMT,              // statement handle
    pCHAR           pszSQL,             // SQL statement
    INT32           iLen                // statement length
);

SQLX_RC SQLX_ExtendedExecute (          // execute a prepared statment
    SQLX_HSTMT      hSTMT,              // statement handle
    pBOOL           pbNoWait,           // asynch. query execution?
    UINT16          uRows               // No of rows insert via block write
);
SQLX_RC SQLX_RowCount (                 // return rows effected by a statment
    SQLX_HSTMT      hSTMT,              // statement handle
    pINT32          piCount
);
SQLX_RC SQLX_GetNewCol (                // get a changed column of a record
    SQLX_HSTMT      hSTMT,              // statement handle
    UINT16          uCol,               // column id (1 based)
    UINT16          uPHYType,           // physical datatype (not CLI)
    INT16           iScale,             // scale of value
    INT32           iLength,            // my extra data (not CLI)
    pVOID           pBuff,              // returned.  new column value
    pINT16          piLength            // returned.  new column value length
);

///////////////////
// cursor functions
SQLX_RC SQLX_NumResultCols (            // execute a prepared statment
    SQLX_HSTMT      hSTMT,              // statement handle
    pINT16          piCols              // returned.  number of columns
);

SQLX_RC SQLX_NumResultObjCols (         // returns the count of object columns
    SQLX_HSTMT      hSTMT,              // statement handle
    pINT16          piCols              // returned.  number of object columns
);

SQLX_RC SQLX_TranslateType (
    SQLX_HDBC       hDBC,               // db connection handle
    pVOID           pSrcType,           // DB physical type
    UINT16          uSrcLen,            // DB physical len
    BOOL            bMetaData,          // DB type is from metadata
    pUINT16         puDstType           // Returned.  Lego physical type
);

SQLX_RC SQLX_TranslateData (            // Translate physical data formats
    UINT16          uSrcType,           // Physical type
    pVOID           pSrc,               // Physical buffer ptr
    UINT16          uSrcLen,            // Physical len
    UINT16          uDstType,           // Physical type
    pVOID           pDst,               // Returned. physical buffer ptr
    UINT16          uDstLen             // Physical len
);

SQLX_RC SQLX_TranslateDecimalData (     // Translate physical data formats
    UINT16          uSrcType,           // Physical type
    pVOID           pSrc,               // Physical buffer ptr
    UINT16          uSrcLen,            // Physical len
    UINT16          uDstType,           // Physical type
    pVOID           pDst,               // Physical buffer ptr
    UINT16          uDstLen,            // Physical len
    UINT16          iUnits2             // Decimal/Money Scale
);

SQLX_RC SQLX_TranslateDataPS(           // Translate physical data formats
                                        // with precision and scale
    UINT16          uSrcType,           // Physical type
    pVOID           pSrc,               // Physical buffer ptr
    UINT16          uSrcLen,            // Physical len
    UINT16          uDstType,           // Physical type
    pVOID           pDst,               // Physical buffer ptr
    UINT16          uDstLen,            // Physical len
    INT16           iUnits1,            // Precision
    INT16           iUnits2             // Scale
);

SQLX_RC SQLX_TranslateSPParam (         // Translate sp params
    SQLX_HDBC       hDBC,               // db connection handle
    pVOID           pParams,            // source param descs
    UINT16          uOffset,            // offset into pParams
    pUINT16         puOverload,         // returned. overload number
    pVOID           pDst                // destination buffer
);

SQLX_RC SQLX_DescribeObj (              // Describe attributes of a object table
    SQLX_HSTMT      hSTMT,              // statement handle
    pCHAR           pszName,            // Object table name
    pINT32          pNoAttr             // No of attributes
);

SQLX_RC SQLX_DescribeProc(              // Describe a stored proc
    SQLX_HSTMT      hSTMT,              // statement handle
    pCHAR           pszName,            // Procedure name
    pUINT32         pNoAttr             // No of arguments
);

SQLX_RC SQLX_DescribeType (             // Describe attributes of a object type
    SQLX_HSTMT      hSTMT,              // statement handle
    pCHAR           pszName,            // Object type name
    pINT32          pNoAttr             // No of attributes
);

SQLX_RC SQLX_NumObjCols (               // Get no.of object columns
    SQLX_HDBC       hDBC,               // db connection handle
    SQLX_HSTMT      *phSTMT,            // returned.  statement handle
    pINT16          piCols              // returned.  number of object columns
);

SQLX_RC SQLX_GetTypeName (              // Get type name for an object column
    SQLX_HSTMT      hSTMT,              // statement handle
    UINT16          uColumn,
    pCHAR           pszColName,
    pINT16          piColNameLen,
    pINT16          piPrecision,
    pINT16          piScale,
    pINT16          piPos
);

SQLX_RC SQLX_DescribeCol (              // describe a column
    SQLX_HSTMT      hSTMT,              // statement handle
    UINT16          uColumn,
    pCHAR           pszColName,
    INT16           iColNameLen,
    pINT16          piColNameLen,
    pUINT16         puPHYType,          // physical datatype (not CLI)
    pUINT16         puPHYSubType,       // for fldstAUTOINC
    pUINT16         puLength,
    pINT16          piPrecision,
    pINT16          piScale,
    pINT16          piNullable,
    pCHAR           pszTblName
);

SQLX_RC SQLX_DescribeSP (
    SQLX_HDBC       hDBC,               // db connection handle
    pCHAR           pszSPName,          // stored procedure name
    pUINT16         puCnt,              // returned. count of params
    ppBYTE          ppBuf               // returend. param buffer
);

SQLX_RC SQLX_DescribeProcParam (        // describe a procedure parameter
    SQLX_HSTMT      hSTMT,              // statement handle
    UINT16          uColumn,
    pVOID           pParam
);

SQLX_RC SQLX_BindCol (                  // bind a column
    SQLX_HSTMT      hSTMT,              // statement handle
    UINT16          uColumn,
    UINT16          uPHYType,           // physical datatype (not CLI)
    pVOID           pBuff,
    INT32           iLength,
    pINT32          piLength
);
SQLX_RC SQLX_SetColOffset (                  // bind a column
    SQLX_HSTMT      hSTMT,              // statement handle
    UINT16          uNumColumn,
    UINT16          uColumn,
    pVOID           pBuff,
    INT32           iLength,
    pINT32          piLength
);
SQLX_RC SQLX_Fetch (                    // fetch a row
    SQLX_HSTMT      hSTMT               // statement handle
);
SQLX_RC SQLX_FetchFromRef (             // fetch a row
    SQLX_HSTMT      hSTMT,              // statement handle
    pCHAR           refHEX,
    BOOL            bLock,              // Lock the reocrd
    BOOL            bPin_Latest         // Pin option
);
SQLX_RC SQLX_DeleteRecordByRef (        // Delete a record by REF
    SQLX_HSTMT      hSTMT,              // Statement handle
    pCHAR           RefHEX              // REF in HEX format
);
SQLX_RC SQLX_ModifyRecordByRef (        // Update a record by REF
    SQLX_HSTMT      hSTMT,              // Statement handle
    pCHAR           RefHEX,             // REF in HEX format
    pBYTE           pBuff               // Record Buff
);
SQLX_RC SQLX_InsertRecordByRef (        // Insert a record by REF
    SQLX_HSTMT      hSTMT,              // Statement handle
    pCHAR           RefTblName,         // REF table name
    pBYTE           pBuff               // Record Buff
);
SQLX_RC SQLX_GetNewRef (                   // Get Ref
    SQLX_HSTMT      hSTMT,              // Statement handle
    ppCHAR          ppRefHEX            // REF in HEX format
);
SQLX_RC SQLX_DatabaseFlushRef (         // flush ref changes in a database
    SQLX_HSTMT      hSTMT               // statement handle
);
SQLX_RC SQLX_RecordFlushRef (           // flush ref change in the current record
    SQLX_HSTMT      hSTMT               // statement handle
);
SQLX_RC SQLX_CommitTran (               // Commit a transaction
    SQLX_HSTMT      hSTMT               // statement handle
);
SQLX_RC SQLX_ExtendedFetch (            // fetch n rows
    SQLX_HSTMT      hSTMT,              // statement handle
    pUINT16         puRows              // in/out. number of rows to fetch
);

/////////////////
// Blob functions

SQLX_RC SQLX_GetBlobHandleLen (         // Get the length of a blob handle
    SQLX_HSTMT      hSTMT,              // Statement handle
    pUINT16         puLength            // Returned.  Blob handle length
);

SQLX_RC SQLX_GetBlobHandle (            // Get the column blob handle
    SQLX_HSTMT      hSTMT,              // Statement handle
    UINT16          uColumn,            // Blob column (1 based)
    SQLX_HBLOB      hBlob               // Returned.  Blob handle
);

SQLX_RC SQLX_GetBlob (                  // Read a portion of a blob
    SQLX_HSTMT      hSTMT,              // Statement handle
    SQLX_HBLOB      hBlob,              // I/O.  SQL-specific blob handle
    pCHAR           pszTblName,         // table name
    pCHAR           pszColName,         // column name
    UINT16          iFldType,           // Field type
    pVOID           pBuff,              // Blob data returned
    UINT32          ulOffset,           // Offset into blob to start
    INT32           lLength,            // Length of blob to read
    pINT32          plLength            // Returned. Actual length read
);

SQLX_RC SQLX_GetLong (                  // Read a portion of a blob
    SQLX_HSTMT      hSTMT,              // Statement handle
    SQLX_HBLOB      hBlob,              // I/O.  SQL-specific blob handle
    pCHAR           pszTblName,         // table name
    pCHAR           pszColName,         // column name
    UINT16          iFldType,           // Field type
    pVOID           pBuff,              // Blob data returned
    UINT32          ulOffset,           // Offset into blob to start
    INT32           lLength,            // Length of blob to read
    pINT32          plLength            // Returned. Actual length read
);


SQLX_RC SQLX_GetBlobLen (               // Get the length of a blob
    SQLX_HSTMT      hSTMT,              // Statement handle
    SQLX_HBLOB      hBlob,              // I/O.  SQL-specific blob handle
    pUINT32         pulLength           // Returned. Blob length
);

SQLX_RC SQLX_StartStoreBlob (           // Start the store of a blob
    SQLX_HSTMT      hSTMT,              // Statement handle
    SQLX_HBLOB      hBlob,              // I/O.  SQL-specific blob handle
    pCHAR           pszTblName,         // table name
    pCHAR           pszColName,         // column name
    UINT32          ulLength,           // Total length of blob
    BOOL            bLog                // Log the blob write operation
);

SQLX_RC SQLX_StoreBlob (                // Store a portion of a blob
    SQLX_HSTMT      hSTMT,              // Statement handle
    pVOID           pBuff,              // Blob data to write
    INT32           lLength             // Length of blob section to write
);

SQLX_RC SQLX_Complete (                 // statement status interogation
    SQLX_HSTMT      hSTMT               // Statement handle
);

SQLX_RC SQLX_GetNextCursor (            // Get the next cursor
    SQLX_HSTMT      hSTMT               // Statement handle
);

#endif

