/***************************************************************************
                                                                             
 Kylix and Delphi Cross-Platform Visual Component Library                    
                                                                             
 Copyright (c) 2000, 2001 Borland Software Corporation                       
                                                                             
 This file may be distributed and/or modified under the terms of the GNU     
 General Public License (GPL) version 2 as published by the Free Software    
 Software Foundation and appearing at http://www.borland.com/kylix/gpl.html. 
                                                                             
 Licensees holding a valid Borland No-Nonsense License for this Software may 
 use this file in accordance with such license, which appears in the file    
 license.txt that came with this Software.                                   
                                                                             
****************************************************************************/

#ifndef QCLXDRAGOBJECT_H
#define QCLXDRAGOBJECT_H

#include <qdragobject.h>
#include <qstring.h>
#include <qmap.h>

class Q_EXPORT QClxDragObject : public QDragObject {
  private:
    typedef QMap<QString, QByteArray> QClxMap;
    QClxMap formatMap;
    QStrList *supportedFormats;
  public:
    QClxDragObject( QWidget * dragSource = 0, const char * name = 0 ) : QDragObject(dragSource, name), supportedFormats(new QStrList()) { }; 
    ~QClxDragObject() {
      delete supportedFormats;
    };
    virtual const char * format(int n) const {
      if ((n >= 0) && (n < supportedFormats->count()))
        return const_cast<QStrList*>(supportedFormats)->at(n);
      else
        return NULL;
    };
    virtual QByteArray encodedData(const char* format) const {
      if (provides(format))
        return formatMap[format];
      else
        return QByteArray();
    };
    virtual bool provides(const char *mimeType) const {
      return const_cast<QStrList*>(supportedFormats)->find(mimeType) != -1; 
    };
    void addFormat(const char* format, const QByteArray& ByteArray) {
      int index = supportedFormats->find(format); 
      if (index != -1)
      {
        supportedFormats->take(index);
        supportedFormats->insert(index, format);
      }
      else
        supportedFormats->append(format);
        
      formatMap[format] = ByteArray;
    };
};

#endif /* QCLXDRAGOBJECT_H */
