/*************************************************************************** 
                                                                             
 Kylix and Delphi Cross-Platform Visual Component Library                    
                                                                             
 Copyright (c) 2000, 2001 Borland Software Corporation                       
                                                                             
 This file may be distributed and/or modified under the terms of the GNU     
 General Public License (GPL) version 2 as published by the Free Software    
 Software Foundation and appearing at http://www.borland.com/kylix/gpl.html. 
                                                                             
 Licensees holding a valid Borland No-Nonsense License for this Software may 
 use this file in accordance with such license, which appears in the file    
 license.txt that came with this Software.                                   
                                                                             
****************************************************************************/

#ifndef QCLXDRAWUTIL_H
#define QCLXDRAWUTIL_H

#include <qdrawutil.h>

class QClxDrawUtil {
public:

  static void DrawItem( QPainter *p, int x, int y, int w, int h,
		int flags, const QColorGroup &g, bool enabled,
		const QPixmap *pixmap, const QString& text,
		int len=-1, const QColor* penColor = 0 ) {
    if ( &g == NULL ) {
      QColorGroup g1 = QApplication::palette( NULL ).active();
      qDrawItem( p, qApp->style().guiStyle(), x, y, w, h, flags, g1, enabled, pixmap, text, len, penColor);
    }
    else
      qDrawItem( p, qApp->style().guiStyle(), x, y, w, h, flags, g, enabled, pixmap, text, len, penColor);
  } 

  static void DrawShadeLine( QPainter *p, int x1, int y1, int x2, int y2,
                     const QColorGroup &g, bool sunken = TRUE,
                     int lineWidth = 1, int midLineWidth = 0 ) {
    if ( &g == NULL ) {
      QColorGroup g1 = QApplication::palette( NULL ).active();
      qDrawShadeLine( p, x1, y1, x2, y2, g1, sunken, lineWidth, midLineWidth);
    }
    else
      qDrawShadeLine( p, x1, y1, x2, y2, g, sunken, lineWidth, midLineWidth);
  }

  static void DrawShadeLine( QPainter *p, const QPoint &p1, const QPoint &p2,
                     const QColorGroup &g, bool sunken = TRUE,
                     int lineWidth = 1, int midLineWidth = 0 ) {
    if ( &g == NULL ) {
      QColorGroup g1 = QApplication::palette( NULL ).active();
      qDrawShadeLine( p,  p1, p2, g1, sunken, lineWidth, midLineWidth);
    }
    else
      qDrawShadeLine( p,  p1, p2, g, sunken, lineWidth, midLineWidth);
  }

  static void DrawShadeRect( QPainter *p, int x, int y, int w, int h,
		     const QColorGroup &g, bool sunken=FALSE,
		     int lineWidth = 1, int midLineWidth = 0,
		     const QBrush *fill = 0 ) {
    if ( &g == NULL ) {
      QColorGroup g1 = QApplication::palette( NULL ).active();
      qDrawShadeRect( p, x, y, w, h, g1, sunken, lineWidth, midLineWidth, fill);
    }
    else
      qDrawShadeRect( p, x, y, w, h, g, sunken, lineWidth, midLineWidth, fill);
  }

  static void DrawShadeRect( QPainter *p, const QRect &r,
		     const QColorGroup &g, bool sunken=FALSE,
		     int lineWidth = 1, int midLineWidth = 0,
		     const QBrush *fill = 0 ) {
    if ( &g == NULL ) {
      QColorGroup g1 = QApplication::palette( NULL ).active();
      qDrawShadeRect( p, r, g1, sunken, lineWidth, midLineWidth, fill);
    }
    else
      qDrawShadeRect( p, r, g, sunken, lineWidth, midLineWidth, fill);
  }

  static void DrawShadePanel( QPainter *p, int x, int y, int w, int h,
		      const QColorGroup &g, bool sunken=FALSE,
		      int lineWidth = 1, const QBrush *fill = 0 ) {
    if ( &g == NULL ) {
      QColorGroup g1 = QApplication::palette( NULL ).active();
      qDrawShadePanel( p, x, y, w, h, g1, sunken, lineWidth, fill);
    }
    else
      qDrawShadePanel( p, x, y, w, h, g, sunken, lineWidth, fill);
  }

  static void DrawShadePanel( QPainter *p, const QRect &r,
		      const QColorGroup &g, bool sunken=FALSE,
		      int lineWidth = 1, const QBrush *fill = 0 ) {
    if ( &g == NULL ) {
      QColorGroup g1 = QApplication::palette( NULL ).active();
      qDrawShadePanel( p, r, g1, sunken, lineWidth, fill);
    }
    else
      qDrawShadePanel( p, r, g, sunken, lineWidth, fill);
  }

  static void DrawWinButton( QPainter *p, int x, int y, int w, int h,
		     const QColorGroup &g, bool sunken = FALSE,
		     const QBrush *fill = 0 ) {
    if ( &g == NULL ) {
      QColorGroup g1 = QApplication::palette( NULL ).active();
      qDrawWinButton( p, x, y, w, h, g1, sunken, fill);
    }
    else
      qDrawWinButton( p, x, y, w, h, g, sunken, fill);
  }

  static void DrawWinButton( QPainter *p, const QRect &r,
		     const QColorGroup &g, bool sunken = FALSE,
		     const QBrush *fill = 0 ){
    if ( &g == NULL ) {
      QColorGroup g1 = QApplication::palette( NULL ).active();
      qDrawWinButton( p, r, g1, sunken, fill);
    }
    else
      qDrawWinButton( p, r, g, sunken, fill);
  }

  static void DrawWinPanel( QPainter *p, int x, int y, int w, int h,
		    const QColorGroup &g, bool sunken=FALSE,
		    const QBrush *fill = 0 ) {
    if ( &g == NULL ) {
      QColorGroup g1 = QApplication::palette( NULL ).active();
      qDrawWinPanel( p, x, y, w, h, g1, sunken, fill);
    }
    else
      qDrawWinPanel( p, x, y, w, h, g, sunken, fill);
  }

  static void DrawWinPanel( QPainter *p, const QRect &r,
		    const QColorGroup &g, bool sunken=FALSE,
		    const QBrush *fill = 0 ) {
    if ( &g == NULL ) {
      QColorGroup g1 = QApplication::palette( NULL ).active();
      qDrawWinPanel( p, r, g1, sunken, fill);
    }
    else
      qDrawWinPanel( p, r, g, sunken, fill);
  }

  static void DrawPlainRect( QPainter *p, int x, int y, int w, int h, const QColor &c,
		     int lineWidth = 1, const QBrush *fill = 0 ) {
    qDrawPlainRect(p, x, y, w, h, c, lineWidth, fill);
  }

  static void DrawPlainRect( QPainter *p, const QRect &r, const QColor &c,
		     int lineWidth = 1, const QBrush *fill = 0 ) {
    qDrawPlainRect(p, r, c, lineWidth, fill);
  }

};


#endif /* QCLXDRAWUTIL_H */
