/*************************************************************************** 
                                                                             
 Kylix and Delphi Cross-Platform Visual Component Library                    
                                                                             
 Copyright (c) 2000, 2001 Borland Software Corporation                       
                                                                             
 This file may be distributed and/or modified under the terms of the GNU     
 General Public License (GPL) version 2 as published by the Free Software    
 Software Foundation and appearing at http://www.borland.com/kylix/gpl.html. 
                                                                             
 Licensees holding a valid Borland No-Nonsense License for this Software may 
 use this file in accordance with such license, which appears in the file    
 license.txt that came with this Software.                                   
                                                                             
****************************************************************************/

#ifndef QCLXEXTRAS_H
#define QCLXEXTRAS_H

#include <qcstring.h>
#include <qevent.h>
#include <qpaintdevice.h>
#include <qobject.h>
#include <qglist.h>
#include <qlist.h>
#include <qobjectlist.h>
#include <qwidget.h>
#include <qmime.h>
#include <qclxmimesource.h>
#include <qwidgetlist.h>
#include <qwindowdefs.h>
#include <qpixmap.h>
#include <qbitmap.h>

C_EXPORT QByteArrayH QByteArray_create(int Size)
{
  return (QByteArrayH)new QByteArray(Size);
}

C_EXPORT void QByteArray_destroy(QByteArrayH ab)
{
  delete (QByteArray *)ab;
}

#ifdef _WS_X11_
C_EXPORT Display *QtDisplay()
{
  return qt_xdisplay();
}
#endif

C_EXPORT void QMimeSource_destroy(QMimeSourceH mime)
{
  delete (QMimeSource *)mime;
}

C_EXPORT bool QMimeSource_is_QClxMimeSource(QMimeSourceH mime)
{
  return dynamic_cast<QClxMimeSource *>((QMimeSource *)mime);
}

C_EXPORT void bitBlt_1(QPaintDeviceH dst, const QPointH dp, QPaintDeviceH src, const QRectH sr, Qt::RasterOp rop, bool ignoreMask)
{
  bitBlt((QPaintDevice *)dst, *(const QPoint*)dp, (const QPaintDevice *)src, *(const QRect*)sr, rop, ignoreMask);
}

C_EXPORT void bitBlt_2(QPaintDeviceH dst, int dx, int dy, QPaintDeviceH src, int sx, int sy, int sw, int sh, Qt::RasterOp rop, bool ignoreMask)
{
  bitBlt((QPaintDevice *)dst, dx, dy, (QPaintDevice *)src, sx, sy, sw, sh, rop, ignoreMask);
}

C_EXPORT void bitBlt_3(QImage* dst, int dx, int dy, const QImage* src, int sx, int sy, int sw, int sh, int conversion_flags)
{
  bitBlt(dst, dx, dy, src, sx, sy, sw, sh, conversion_flags);
}

C_EXPORT unsigned long QClxWidget_MaxWidgetSize(void)
{
  return QWIDGETSIZE_MAX;
}

C_EXPORT void QWorkspace_next(QWorkspaceH workspace)
{
  QWidget *aWidget;
  QWidgetList aList;
  QWorkspace *wSpace;

  if (workspace)
  {
    wSpace = (QWorkspace *)workspace;
    aList = wSpace->windowList();
    if (aList.find(wSpace->activeWindow()) != -1) {
      if (!(aWidget = aList.next()))
        aWidget = aList.first();
    }
    else
      aWidget = aList.first();
    aWidget->setFocus();
  }
}

C_EXPORT void QWorkspace_previous(QWorkspaceH workspace)
{
  QWidget *aWidget;
  QWidgetList aList;
  QWorkspace *wSpace;

  if (workspace)
  {
    wSpace = (QWorkspace *)workspace;
    aList = wSpace->windowList();
    if (aList.find(wSpace->activeWindow()) != -1) {
      if (!(aWidget = aList.prev()))
        aWidget = aList.last();
    }
    else
      aWidget = aList.first();
    aWidget->setFocus();
  }
}

/* helper functions provided for type-checking of event classes */

C_EXPORT bool QEvent_isQCustomEvent(QEventH e)
{
  return dynamic_cast<QCustomEvent *>((QEvent*) e) != NULL;
}

C_EXPORT bool QEvent_isQTimerEvent(QEventH e)
{
  return dynamic_cast<QTimerEvent *>((QEvent*) e) != NULL;
}

C_EXPORT bool QEvent_isQMouseEvent(QEventH e)
{
  return dynamic_cast<QMouseEvent *>((QEvent*) e) != NULL;
}

C_EXPORT bool QEvent_isQWheelEvent(QEventH e)
{
  return dynamic_cast<QWheelEvent *>((QEvent*) e) != NULL;
}

C_EXPORT bool QEvent_isQKeyEvent(QEventH e)
{
  return dynamic_cast<QKeyEvent *>((QEvent*) e) != NULL;
}

C_EXPORT bool QEvent_isQFocusEvent(QEventH e)
{
  return dynamic_cast<QFocusEvent *>((QEvent*) e) != NULL;
}

C_EXPORT bool QEvent_isQPaintEvent(QEventH e)
{
  return dynamic_cast<QPaintEvent *>((QEvent*) e) != NULL;
}

C_EXPORT bool QEvent_isQMoveEvent(QEventH e)
{
  return dynamic_cast<QMoveEvent *>((QEvent*) e) != NULL;
}

C_EXPORT bool QEvent_isQResizeEvent(QEventH e)
{
  return dynamic_cast<QResizeEvent *>((QEvent*) e) != NULL;
}

C_EXPORT bool QEvent_isQCloseEvent(QEventH e)
{
  return dynamic_cast<QCloseEvent *>((QEvent*) e) != NULL;
}

C_EXPORT bool QEvent_isQShowEvent(QEventH e)
{
  return dynamic_cast<QShowEvent *>((QEvent*) e) != NULL;
}

C_EXPORT bool QEvent_isQHideEvent(QEventH e)
{
  return dynamic_cast<QHideEvent *>((QEvent*) e) != NULL;
}

C_EXPORT bool QEvent_isQDropEvent(QEventH e)
{
  return dynamic_cast<QDropEvent *>((QEvent*) e) != NULL;
}

C_EXPORT bool QEvent_isQDragMoveEvent(QEventH e)
{
  return dynamic_cast<QDragMoveEvent *>((QEvent*) e) != NULL;
}

C_EXPORT bool QEvent_isQDragEnterEvent(QEventH e)
{
  return dynamic_cast<QDragEnterEvent *>((QEvent*) e) != NULL;
}

C_EXPORT bool QEvent_isQDragResponseEvent(QEventH e)
{
  return dynamic_cast<QDragResponseEvent *>((QEvent*) e) != NULL;
}

C_EXPORT bool QEvent_isQDragLeaveEvent(QEventH e)
{
  return dynamic_cast<QDragLeaveEvent *>((QEvent*) e) != NULL;
}

C_EXPORT bool QEvent_isQChildEvent(QEventH e)
{
  return dynamic_cast<QChildEvent *>((QEvent*) e) != NULL;
}

/* wrapper for Qt QObjectList */

C_EXPORT QObjectListH QObjectList_create(void)
{
  return (QObjectListH)(new QObjectList());
}

C_EXPORT QObjectListH QObjectList_create2(const QObjectListH list)
{
  return (QObjectListH)(new QObjectList(*(QObjectList*)list));
}

C_EXPORT void QObjectList_destroy(QObjectListH obj)
{
  delete obj;
}

C_EXPORT uint QObjectList_count(QObjectListH obj)
{
  return ((QObjectList *)obj)->count();
}

C_EXPORT bool QObjectList_isEmpty(QObjectListH obj)
{
  return ((QObjectList *)obj)->isEmpty();
}

C_EXPORT bool QObjectList_insert(QObjectListH obj, uint i, const QObjectH d)
{
  return ((QObjectList *)obj)->insert(i,(QObject*)d);
}

C_EXPORT void QObjectList_inSort(QObjectListH obj, const QObjectH d)
{
  ((QObjectList *)obj)->inSort((QObject*)d);
}

C_EXPORT void QObjectList_prepend(QObjectListH obj, const QObjectH d)
{
  ((QObjectList *)obj)->prepend((QObject*)d);
}

C_EXPORT void QObjectList_append(QObjectListH obj, const QObjectH d)
{
  ((QObjectList *)obj)->append((QObject*)d);
}

C_EXPORT bool QObjectList_remove(QObjectListH obj, uint i)
{
  return ((QObjectList *)obj)->remove(i);
}

C_EXPORT bool QObjectList_remove2(QObjectListH obj)
{
  return ((QObjectList *)obj)->remove();
}

C_EXPORT bool QObjectList_remove3(QObjectListH obj, const QObjectH d)
{
  return ((QObjectList *)obj)->remove((QObject*)d);
}

C_EXPORT bool QObjectList_removeRef(QObjectListH obj, const QObjectH d)
{
  return ((QObjectList *)obj)->removeRef((QObject*)d);
}

C_EXPORT bool QObjectList_removeFirst(QObjectListH obj)
{
  return ((QObjectList *)obj)->removeFirst();
}

C_EXPORT bool QObjectList_removeLast(QObjectListH obj)
{
  return ((QObjectList *)obj)->removeLast();
}

C_EXPORT QObjectH QObjectList_take(QObjectListH obj, uint i)
{
  return (QObjectH)((QObjectList *)obj)->take(i);
}

C_EXPORT QObjectH QObjectList_take2(QObjectListH obj)
{
  return (QObjectH)((QObjectList *)obj)->take();
}

C_EXPORT void QObjectList_clear(QObjectListH obj)
{
  ((QObjectList *)obj)->clear();
}

C_EXPORT void QObjectList_sort(QObjectListH obj)
{
  ((QObjectList *)obj)->sort();
}

C_EXPORT int QObjectList_find(QObjectListH obj, const QObjectH d)
{
  return ((QObjectList *)obj)->find((QObject*)d);
}

C_EXPORT int QObjectList_findNext(QObjectListH obj, const QObjectH d)
{
  return ((QObjectList *)obj)->findNext((QObject*)d);
}

C_EXPORT int QObjectList_findRef(QObjectListH obj, const QObjectH d)
{
  return ((QObjectList *)obj)->findRef((QObject*)d);
}

C_EXPORT int QObjectList_findNextRef(QObjectListH obj, const QObjectH d)
{
  return ((QObjectList *)obj)->findNextRef((QObject*)d);
}

C_EXPORT uint QObjectList_contains(QObjectListH obj, const QObjectH d)
{
  return ((QObjectList *)obj)->contains((QObject*)d);
}

C_EXPORT uint QObjectList_containsRef(QObjectListH obj, const QObjectH d)
{
  return ((QObjectList *)obj)->containsRef((QObject*)d);
}

C_EXPORT QObjectH QObjectList_at(QObjectListH obj, uint i)
{
  return (QObjectH)((QObjectList *)obj)->at(i);
}

C_EXPORT int QObjectList_at2(QObjectListH obj)
{
  return ((QObjectList *)obj)->at();
}

C_EXPORT QObjectH QObjectList_current(QObjectListH obj)
{
  return (QObjectH)((QObjectList *)obj)->current();
}

C_EXPORT QObjectH QObjectList_getFirst(QObjectListH obj)
{
  return (QObjectH)((QObjectList *)obj)->getFirst();
}

C_EXPORT QObjectH QObjectList_getLast(QObjectListH obj)
{
  return (QObjectH)((QObjectList *)obj)->getLast();
}

C_EXPORT QObjectH QObjectList_first(QObjectListH obj)
{
  return (QObjectH)((QObjectList *)obj)->first();
}

C_EXPORT QObjectH QObjectList_last(QObjectListH obj)
{
  return (QObjectH)((QObjectList *)obj)->last();
}

C_EXPORT QObjectH QObjectList_next(QObjectListH obj)
{
  return (QObjectH)((QObjectList *)obj)->next();
}

C_EXPORT QObjectH QObjectList_prev(QObjectListH obj)
{
  return (QObjectH)((QObjectList *)obj)->prev();
}

C_EXPORT QBitmapH QBitmap_from_QPixmap(QBitmapH bm, QPixmapH pm)
{
  return (QBitmapH)&(*(QBitmap*)bm = *(QPixmap*)pm);
}

/* wrapper for Qt QWidgetList */

C_EXPORT QWidgetListH QWidgetList_create(void)
{
  return (QWidgetListH)(new QWidgetList());
}

C_EXPORT QWidgetListH QWidgetList_create2(const QWidgetListH list)
{
  return (QWidgetListH)(new QWidgetList(*(QWidgetList*)list));
}

C_EXPORT void QWidgetList_destroy(QWidgetListH obj)
{
  delete obj;
}

C_EXPORT uint QWidgetList_count(QWidgetListH obj)
{
  return ((QWidgetList *)obj)->count();
}

C_EXPORT bool QWidgetList_insert(QWidgetListH obj, uint i, const QWidgetH d)
{
  return ((QWidgetList *)obj)->insert(i,(QWidget*)d);
}

C_EXPORT bool QWidgetList_remove(QWidgetListH obj, uint i)
{
  return ((QWidgetList *)obj)->remove(i);
}

C_EXPORT bool QWidgetList_remove3(QWidgetListH obj, const QWidgetH d)
{
  return ((QWidgetList *)obj)->remove((QWidget*)d);
}

C_EXPORT void QWidgetList_clear(QWidgetListH obj)
{
  ((QWidgetList *)obj)->clear();
}

C_EXPORT int QWidgetList_find(QWidgetListH obj, const QWidgetH d)
{
  return ((QWidgetList *)obj)->find((QWidget*)d);
}

C_EXPORT int QWidgetList_findNext(QWidgetListH obj, const QWidgetH d)
{
  return ((QWidgetList *)obj)->findNext((QWidget*)d);
}

C_EXPORT uint QWidgetList_contains(QWidgetListH obj, const QWidgetH d)
{
  return ((QWidgetList *)obj)->contains((QWidget*)d);
}

C_EXPORT QWidgetH QWidgetList_at(QWidgetListH obj, uint i)
{
  return (QWidgetH)((QWidgetList *)obj)->at(i);
}

C_EXPORT QWidgetH QWidgetList_current(QWidgetListH obj)
{
  return (QWidgetH)((QWidgetList *)obj)->current();
}

C_EXPORT QWidgetH QWidgetList_first(QWidgetListH obj)
{
  return (QWidgetH)((QWidgetList *)obj)->first();
}

C_EXPORT QWidgetH QWidgetList_last(QWidgetListH obj)
{
  return (QWidgetH)((QWidgetList *)obj)->last();
}

C_EXPORT QWidgetH QWidgetList_next(QWidgetListH obj)
{
  return (QWidgetH)((QWidgetList *)obj)->next();
}

C_EXPORT QWidgetH QWidgetList_prev(QWidgetListH obj)
{
  return (QWidgetH)((QWidgetList *)obj)->prev();
}

#endif /* QCLXEXTRAS_H */
