/****************************************************************************
** $Id: qclxfiledialog.h,v 1.1 2001/05/29 21:45:11 mduncan Exp $
**
** Definition of QClxFileDialog class
**
** Created : 950428
**
** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
**
** This file is part of the dialogs module of the Qt GUI Toolkit.
**
** This file may be distributed under the terms of the Q Public License
** as defined by Troll Tech AS of Norway and appearing in the file
** LICENSE.QPL included in the packaging of this file.
**
** Licensees holding valid Qt Enterprise Edition or Qt Professional Edition
** licenses may use this file in accordance with the Qt Commercial License
** Agreement provided with the Software.  This file is part of the dialogs
** module and therefore may only be used if the dialogs module is specified
** as Licensed on the Licensee's License Certificate.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
** information about the Professional Edition licensing, or see
** http://www.trolltech.com/qpl/ for QPL licensing information.
**
*****************************************************************************/

#ifndef QCLXFILEDIALOG_H
#define QCLXFILEDIALOG_H

struct QClxFileDialogPrivate;
class QPushButton;
class QLabel;
class QWidget;
class QClxFileDialog;
class QTimer;
class QNetworkOperation;

#ifndef QT_H
#include "qdir.h"
#include "qdialog.h"
#include "qlistbox.h"
#include "qlineedit.h"
#include "qlistview.h"
#include "qurloperator.h"
#include "qurlinfo.h"
#include "qstrlist.h"
#include "../qtypes.h"
#endif // QT_H

#ifndef QT_NO_FILEDIALOG

class Q_EXPORT QClxFileIconProvider : public QObject
{
    Q_OBJECT
public:
    QClxFileIconProvider( QObject * parent = 0, const char* name = 0 );
    virtual const QPixmap * pixmap( const QFileInfo & );

private:	// Disabled copy constructor and operator=
#if defined(Q_DISABLE_COPY)
    QClxFileIconProvider( const QClxFileIconProvider & );
    QClxFileIconProvider& operator=( const QClxFileIconProvider & );
#endif
};

class Q_EXPORT QClxFilePreview
{
public:
    QClxFilePreview();
    virtual void previewUrl( const QUrl &url ) = 0;

};

class Q_EXPORT QClxFileDialog : public QDialog
{
    friend class QClxFileListBox;
    friend class QClxFileListView;

    Q_OBJECT
    Q_ENUMS( Mode ViewMode PreviewMode )
    // ##### Why are this read-only properties ?
    Q_PROPERTY( QString selectedFile READ selectedFile )
    Q_PROPERTY( QString selectedFilter READ selectedFilter )
    Q_PROPERTY( QStringList selectedFiles READ selectedFiles )
    // #### Should not we be able to set the path ?
    Q_PROPERTY( QString dirPath READ dirPath )
    Q_PROPERTY( bool showHiddenFiles READ showHiddenFiles WRITE setShowHiddenFiles )
    Q_PROPERTY( Mode mode READ mode WRITE setMode )
    Q_PROPERTY( ViewMode viewMode READ viewMode WRITE setViewMode )
    Q_PROPERTY( PreviewMode previewMode READ previewMode WRITE setPreviewMode )
    Q_PROPERTY( bool infoPreview READ isInfoPreviewEnabled WRITE setInfoPreviewEnabled )
    Q_PROPERTY( bool contentsPreview READ isContentsPreviewEnabled WRITE setContentsPreviewEnabled )

public:
    QClxFileDialog( const QString& dirName, const QString& filter = QString::null,
                 QWidget *parent=0, const char *name = 0, bool modal = FALSE );
    QClxFileDialog( QWidget *parent=0, const char *name = 0, bool modal = FALSE );
    QClxFileDialog( QWidget *parent, bool modal, WFlags f );
    ~QClxFileDialog();

    // recommended static functions

    static QString getOpenFileName( const QString &initially = QString::null,
				    const QString &filter = QString::null,
				    QWidget *parent = 0, const char* name = 0 ); // ## merge 3.0
    static QString getOpenFileName( const QString &initially,
				    const QString &filter,
				    QWidget *parent, const char* name, const QString& caption );
    static QString getSaveFileName( const QString &initially = QString::null,
				    const QString &filter = QString::null,
				    QWidget *parent = 0, const char* name = 0);// ## merge 3.0
    static QString getSaveFileName( const QString &initially,
				    const QString &filter,
				    QWidget *parent, const char* name,
				    const QString& caption);
    static QString getExistingDirectory( const QString &dir = QString::null,
					 QWidget *parent = 0,
					 const char* name = 0 );// ## merge 3.0
    static QString getExistingDirectory( const QString &dir,
					 QWidget *parent,
					 const char* name,
					 const QString& caption );
    static QString getExistingDirectory( const QString &dir,
					 QWidget *parent,
					 const char* name,
					 const QString& caption,
					 bool dirOnly );
    static QStringList getOpenFileNames( const QString &filter= QString::null,
					 const QString &dir = QString::null,
					 QWidget *parent = 0,
					 const char* name = 0);// ## merge 3.0
    static QStringList getOpenFileNames( const QString &filter,
					 const QString &dir,
					 QWidget *parent,
					 const char* name,
					 const QString& caption);


    // other static functions

    static void setIconProvider( QClxFileIconProvider * );
    static QClxFileIconProvider * iconProvider();

    // non-static function for special needs

    QString selectedFile() const;
    QString selectedFilter() const;
    void setSelection( const QString &);

    void selectAll( bool b );

    QStringList selectedFiles() const;

    QString dirPath() const;

    void setDir( const QDir & );
    const QDir *dir() const;

    void setShowHiddenFiles( bool s );
    bool showHiddenFiles() const;

    void rereadDir();
    void resortDir();

    enum Mode { AnyFile, ExistingFile, Directory, ExistingFiles, DirectoryOnly };
    void setMode( Mode );
    Mode mode() const;

    enum ViewMode { Detail, List };
    enum PreviewMode { NoPreview, Contents, Info };
    void setViewMode( ViewMode m );
    ViewMode viewMode() const;
    void setPreviewMode( PreviewMode m );
    PreviewMode previewMode() const;

    bool eventFilter( QObject *, QEvent * );

    bool isInfoPreviewEnabled() const;
    bool isContentsPreviewEnabled() const;
    void setInfoPreviewEnabled( bool );
    void setContentsPreviewEnabled( bool );

    void setInfoPreview( QWidget *w, QClxFilePreview *preview );
    void setContentsPreview( QWidget *w, QClxFilePreview *preview );

    QUrl url() const;

    void setEditText( QString s ) { nameEdit->setText( s ); }
    void setFilterIndex( int index );
    void hook_addFile(QHook &hook) { addFileCallback = hook; }
    QString selectedExt() { return fSelectedExt; }
    void setDefaultExt( QString ext ) { fDefaultExt = ext; }
    QString defaultExt() { return fDefaultExt; }
    bool isRegisteredExtension( QString ext );

public slots:
    void done( int );
    void setDir( const QString& );
    void setUrl( const QUrlOperator &url );
    void setFilter( const QString& );
    void setFilters( const QString& );
    void setFilters( const char ** );
    void setFilters( const QStringList& );
    void show();

protected:
    void resizeEvent( QResizeEvent * );
    void keyPressEvent( QKeyEvent * );

    void addWidgets( QLabel *, QWidget *, QPushButton * );
    void addToolButton( QButton *b, bool separator = FALSE );
    void addLeftWidget( QWidget *w );
    void addRightWidget( QWidget *w );
    void addFilter( const QString &filter );

signals:
    void fileHighlighted( const QString& );
    void fileSelected( const QString& );
    void dirEntered( const QString& );
    void closeRequest( bool &canClose );
    void filterChanged( int index );

private slots:
    void detailViewSelectionChanged();
    void listBoxSelectionChanged();
    void changeMode( int );
    void fileNameEditReturnPressed();
    void stopCopy();
    void removeProgressDia();
    void comboSetFilter( const QString& );

    void fileSelected( int );
    void fileHighlighted( int );
    void dirSelected( int );
    void pathSelected( int );

    void updateFileNameEdit( QListViewItem *);
    void selectDirectoryOrFile( QListViewItem * );
    void popupContextMenu( QListViewItem *, const QPoint &, int );
    void popupContextMenu( QListBoxItem *, const QPoint & );
    void updateFileNameEdit( QListBoxItem *);
    void selectDirectoryOrFile( QListBoxItem * );
    void fileNameEditDone();

    void okClicked();
    void filterClicked(); // not used
    void cancelClicked();

    void cdUpClicked();
    void newFolderClicked();

    void fixupNameEdit();

    void doMimeTypeLookup();

    void updateGeometries();
    void modeButtonsDestroyed();
    void urlStart( QNetworkOperation *op );
    void urlFinished( QNetworkOperation *op );
    void dataTransferProgress( int bytesDone, int bytesTotal, QNetworkOperation * );
    void insertEntry( const QValueList<QUrlInfo> &fi, QNetworkOperation *op );
    void removeEntry( QNetworkOperation * );
    void createdDirectory( const QUrlInfo &info, QNetworkOperation * );
    void itemChanged( QNetworkOperation * );
    void goBack();

private:
    enum PopupAction {
        PA_Open = 0,
        PA_Delete,
        PA_Rename,
        PA_SortName,
        PA_SortSize,
        PA_SortType,
        PA_SortDate,
        PA_SortUnsorted,
        PA_Cancel,
        PA_Reload,
        PA_Hidden
    };

    void init();
    bool trySetSelection( bool isDir, const QUrlOperator &, bool );
    void deleteFile( const QString &filename );
    void popupContextMenu( const QString &filename, bool withSort,
                           PopupAction &action, const QPoint &p );
    QString expandFileName( QString s );

    QDir reserved; // was cwd
    QString fileName;

    QClxFileDialogPrivate *d;
    QClxFileListView  *files;

    QLineEdit  *nameEdit; // also filter
    QPushButton *okB;
    QPushButton *cancelB;
    QString userFilter;
    QHook addFileCallback;
    QString fSelectedExt;
    QString fDefaultExt;
    QStrList *extensions;

private:	// Disabled copy constructor and operator=
#if defined(Q_DISABLE_COPY)
    QClxFileDialog( const QClxFileDialog & );
    QClxFileDialog &operator=( const QClxFileDialog & );
#endif
};

typedef QClxFileDialog::Mode QClxFileDialog_Mode;

#endif

#endif // QFILEDIALOG_H
