/*************************************************************************** 
                                                                             
 Kylix and Delphi Cross-Platform Visual Component Library                    
                                                                             
 Copyright (c) 2000, 2001 Borland Software Corporation                       
                                                                             
 This file may be distributed and/or modified under the terms of the GNU     
 General Public License (GPL) version 2 as published by the Free Software    
 Software Foundation and appearing at http://www.borland.com/kylix/gpl.html. 
                                                                             
 Licensees holding a valid Borland No-Nonsense License for this Software may 
 use this file in accordance with such license, which appears in the file    
 license.txt that came with this Software.                                   
                                                                             
****************************************************************************/

#ifndef QCLXICONVIEWITEM_H
#define QCLXICONVIEWITEM_H

#include <qiconview.h>

/* callbacks into a QComCtrls.TCustomIconView */
class QClxIconViewHooks {
  friend class QClxIconViewItem;
  private:
    QHook paintItem_hook;
    QHook paintFocus_hook;
    QHook setSelected_hook;
    QHook change_hook;
    QHook changing_hook;
    QHook destroyed_hook;
  public:                                                        
    QClxIconViewHooks() {
      paintItem_hook.func = NULL;
      paintFocus_hook.func = NULL;
      setSelected_hook.func = NULL;
      change_hook.func = NULL;
      changing_hook.func = NULL;
      destroyed_hook.func = NULL;
    }
    void hook_PaintItem( QHook h ) { paintItem_hook = h; }
    void hook_PaintFocus( QHook h ) { paintFocus_hook = h; }
    void hook_setSelected( QHook h) { setSelected_hook = h; };
    void hook_change( QHook h) { change_hook = h; };
    void hook_changing( QHook h) { changing_hook = h; };
    void hook_destroyed( QHook h) { destroyed_hook = h; };
};

class QClxIconViewItem : public QIconViewItem {
  private:
    typedef enum {ctText, ctImage, ctState} TItemChange;
    typedef void (*TChangingFunc)(void *data, QIconViewItemH Self, TItemChange type, bool &Allow);
    typedef void (*TChangeFunc)(void *data, QIconViewItemH Self, TItemChange type);
    typedef void (*TDestroyedHook)(void *data, QIconViewItemH Self);
    QClxIconViewHooks *hooks;
  protected:
    void setSelected( bool Value, bool cb)
    {
      typedef void (*func_type)(void *data, QIconViewItemH Self, bool Value);
      bool Allow = true;

      if ((isSelected() != Value) || (iconView()->currentItem() == this))
      {
        if ((isSelected() != Value) && (hooks->changing_hook.func))
          (*(TChangingFunc)hooks->changing_hook.func)(hooks->changing_hook.data,
            (QIconViewItemH)this, ctState, Allow);

        if (Allow)
        {
          QIconViewItem::setSelected(Value, cb);
          if (hooks->setSelected_hook.func)
          (*(func_type)hooks->setSelected_hook.func)(hooks->setSelected_hook.data,
            (QIconViewItemH)this, Value);
          if ((isSelected() != Value) && (hooks->change_hook.func))
            (*(TChangeFunc)hooks->change_hook.func)(hooks->change_hook.data,
              (QIconViewItemH)this, ctState);
        }
      }
      else
        QIconViewItem::setSelected(Value, cb);
    }

    void setText( const QString &Str)
    {
      bool Allow = true;

      if ( hooks->changing_hook.func )
        (*(TChangingFunc)hooks->changing_hook.func)(hooks->changing_hook.data,
          (QIconViewItemH)this, ctText, Allow);

      if (Allow)
        QIconViewItem::setText(Str);

      if ( hooks->change_hook.func )
        (*(TChangeFunc)hooks->change_hook.func)(hooks->change_hook.data,
          (QIconViewItemH)this, ctText);
    }

    void setPixmap(const QPixmap &icon)
    {
      bool Allow = true;

      if ( hooks->changing_hook.func )
        (*(TChangingFunc)hooks->changing_hook.func)(hooks->changing_hook.data,
          (QIconViewItemH)this, ctImage, Allow);

      if (Allow)
        QIconViewItem::setPixmap(icon);

      if ( hooks->change_hook.func )
        (*(TChangeFunc)hooks->change_hook.func)(hooks->change_hook.data,
          (QIconViewItemH)this, ctImage);
    }

    void paintItem( QPainter *p, const QColorGroup &cg )
    {
      typedef void (*func_type)(void *data, QPainterH p, QIconViewItemH item,
        QColorGroupH cg, DrawStage &stage);

      DrawStage stage = Pre;

      if ( hooks->paintItem_hook.func )
        (*(func_type)hooks->paintItem_hook.func)(hooks->paintItem_hook.data,
        (QPainterH)p, (QIconViewItemH)this, (QColorGroupH)&cg, stage );
      else
        stage = DefaultDraw;

      if (stage == DefaultDraw)
        QIconViewItem::paintItem( p, cg);

      if ( hooks->paintItem_hook.func )
      {
        stage = Post;
        (*(func_type)hooks->paintItem_hook.func)(hooks->paintItem_hook.data,
        (QPainterH)p, (QIconViewItemH)this, (QColorGroupH)&cg,stage );
      }
    }
	
    void paintFocus( QPainter *p, const QColorGroup &cg )
    {
      typedef void (*func_type)(void *data, QPainterH p, QIconViewItemH item,
        QColorGroupH cg, DrawStage &stage);

      DrawStage stage = Pre;
      if ( hooks->paintFocus_hook.func )
        (*(func_type)hooks->paintFocus_hook.func)( hooks->paintFocus_hook.data,
        (QPainterH)p, (QIconViewItemH)this, (QColorGroupH)&cg, stage );
      else
        stage = DefaultDraw;
	
      if ( stage == DefaultDraw )
        QIconViewItem::paintFocus( p, cg);
    }
  public:
    /* constructors */
    QClxIconViewItem( QIconView *parent, QClxIconViewHooks *h ) :
      QIconViewItem( parent ) { hooks = h; }
    QClxIconViewItem( QIconView *parent, QIconViewItem *after,
      QClxIconViewHooks *h ) : QIconViewItem( parent, after ) { hooks = h; }

    /* destructor */
    ~QClxIconViewItem(void)
    {
      if ( hooks->destroyed_hook.func )
        (*(TDestroyedHook)hooks->destroyed_hook.func)( hooks->destroyed_hook.data,
        QIconViewItemH(this));
    }
};

#endif /* QCLXICONVIEWITEM_H */
