/*************************************************************************** 
                                                                             
 Kylix and Delphi Cross-Platform Visual Component Library                   
                                                                             
 Copyright (c) 2000, 2001 Borland Software Corporation                       
                                                                             
 This file may be distributed and/or modified under the terms of the GNU     
 General Public License (GPL) version 2 as published by the Free Software    
 Software Foundation and appearing at http://www.borland.com/kylix/gpl.html. 
                                                                             
 Licensees holding a valid Borland No-Nonsense License for this Software may 
 use this file in accordance with such license, which appears in the file    
 license.txt that came with this Software.                                   
                                                                             
****************************************************************************/

#ifndef QCLXIODEVICE_H
#define QCLXIODEVICE_H

#include <qiodevice.h>

// These definitions must match exactly to the definitions in QTypes.pas

typedef int (*StreamReadProc)(void *rec, void *buffer, int size);
typedef int (*StreamWriteProc)(void *rec, void *buffer, int size);
typedef int (*StreamSeekProc)(void *rec, int location, int offset);
typedef void (*StreamFreeProc)(void *rec);

typedef struct {
  StreamReadProc read;
  StreamWriteProc write;
  StreamSeekProc seek;
  StreamFreeProc free;
} ClxStream;

const int csFromStart = 0;
const int csCurrent = 1;
const int csFromEnd = 2;

class QClxIODevice : public QIODevice {
  public:
    QClxIODevice(void *stream) { this->stream = (ClxStream *)stream; }
    virtual ~QClxIODevice() { stream->free(stream); }

  protected:
    virtual bool open( int mode ) { setType(IO_Combined); setMode(IO_Raw); }
    virtual void close() { reset(); }
    virtual void flush() { }
    virtual int at() const { return stream->seek(stream, 0, csCurrent); }
    virtual bool at(int pos) { return stream->seek(stream, pos, csFromStart); }
    virtual uint size() const { 
      int pos = stream->seek(stream, 0, csCurrent);
      stream->seek(stream, 0, csFromStart);         
      int s = stream->seek(stream, 0, csFromEnd); 
      stream->seek(stream, pos, csFromStart); 
      return s; 
    }
    virtual int	 readBlock( char *data, uint maxlen ) { return stream->read(stream, (void *)data, maxlen); }
    virtual int	 writeBlock( const char *data, uint len ) { return stream->write(stream, (void *)data, len); }
    virtual int	 getch() { char c; if ( readBlock(&c, 1) ) return c; else return -1; }
    virtual int	 putch( int ch ) { char c = ch; if ( writeBlock(&c, 1) ) return ch; else return -1; }
    virtual int	 ungetch( int ) { int pos = at(); if ( pos > 0 ) { at(pos - 1); return 0; } else return -1; }

  private:
    ClxStream *stream;
};

#endif /* QCLXIODEVICE_H */
