/*************************************************************************** 
                                                                             
 Kylix and Delphi Cross-Platform Visual Component Library                    
                                                                             
 Copyright (c) 2000, 2001 Borland Software Corporation                       
                                                                             
 This file may be distributed and/or modified under the terms of the GNU     
 General Public License (GPL) version 2 as published by the Free Software    
 Software Foundation and appearing at http://www.borland.com/kylix/gpl.html. 
                                                                             
 Licensees holding a valid Borland No-Nonsense License for this Software may 
 use this file in accordance with such license, which appears in the file    
 license.txt that came with this Software.                                   
                                                                             
****************************************************************************/

#ifndef QCLXLINEEDIT_H
#define QCLXLINEEDIT_H

#include <qlineedit.h>
#include <qevent.h>

class Q_EXPORT QClxLineEdit : public QLineEdit
{

private:
      bool fAutoSelect;
      bool fHideSelection;
      int fSelStart;
      int fSelEnd;
      int fSelLength;
      int fMark;
      int fOldPos;
      int fOldLen;
      bool needReset;
      bool dragSelect;
      bool autoSelectOnMouseUp;
      bool doubleClicked;
      bool QtSelect;
      bool dontDeselect;

protected:
      void focusInEvent( QFocusEvent *focusEvent ) {

            QtSelect = TRUE;
            QFocusEvent::setReason( QFocusEvent::ActiveWindow );
            QFocusEvent forceFocus( QEvent::FocusIn );
            QLineEdit::focusInEvent( &forceFocus );
            QFocusEvent::resetReason();

            if ( focusEvent->reason() != QFocusEvent::Popup )
            {
              if ( fAutoSelect )
              {
                selectAll();
                fSelStart = 0;
                fSelEnd = text().length();
                fMark = 0;
                autoSelectOnMouseUp = TRUE;
              }
              else
                setSelection( fSelStart, fSelEnd - fSelStart );
            }
      }

      void focusOutEvent( QFocusEvent *focusEvent ) {

            QFocusEvent::setReason( QFocusEvent::ActiveWindow );
            QFocusEvent forceFocus( QEvent::FocusOut );
            QLineEdit::focusOutEvent( &forceFocus );
            QFocusEvent::resetReason();

            if ( fHideSelection & (focusEvent->reason() != QFocusEvent::Popup) )
            {
              dontDeselect = TRUE;
              deselect();
              QtSelect = FALSE;
              dontDeselect = FALSE;
            }
      }

      void mouseMoveEvent( QMouseEvent *e ) {

            if ( (e->state() & LeftButton) & !dragSelect ) {
              fSelStart = cursorPosition();
              fMark = cursorPosition();
              dragSelect = TRUE;
              autoSelectOnMouseUp = FALSE;
            }

            QLineEdit::mouseMoveEvent( e );

            if ( (e->state() & LeftButton) & dragSelect ) {
              fSelStart = fMark;
              fSelEnd = cursorPosition();
              if ( fSelEnd < fSelStart ) {
                int tmp = fSelEnd;
                fSelEnd = fSelStart;
                fSelStart = tmp;
              }
            }

      }

      void mousePressEvent( QMouseEvent * e ) {

            doubleClicked = FALSE;
            dragSelect = FALSE;
            fMark = -1;

            QLineEdit::mousePressEvent( e );

            if ( (e->state() & LeftButton) &!dragSelect ) {
              deselect();
            }

            if ( cursorPosition() < fSelStart || cursorPosition() > fSelEnd )
            {
              fSelStart = cursorPosition();
              fSelEnd = fSelStart;
            }
      }

      void mouseReleaseEvent( QMouseEvent * e ) {

           if ( doubleClicked )
             return;
           QLineEdit::mouseReleaseEvent( e );
           if ( autoSelectOnMouseUp )
           {
             selectAll();
             fSelStart = 0;
             fSelEnd = text().length();
             fMark = 0;
           }
           else
           {
             if ( !dragSelect )
               fSelStart = cursorPosition();
           }
           autoSelectOnMouseUp = FALSE;
           dragSelect = FALSE;
      }

      void mouseDoubleClickEvent( QMouseEvent *e ) {

           doubleClicked = TRUE;
           QLineEdit::mouseDoubleClickEvent( e );
      }

      void keyPressEvent( QKeyEvent *e ) {

           int oldCursorPos = cursorPosition();
           int oldMarkLength = markedText().length();
           needReset = FALSE;
           QLineEdit::keyPressEvent( e );
           if ( ( e->state() == ShiftButton ) &&
             ( (e->key() == Key_Right ) || ( e->key() == Key_Left ) ) )
           {
             if ( fMark == -1 )
             fMark = cursorPosition();
           }
           else {
             if ( e->key() != Key_Shift )
               fMark = -1;
             if ( cursorPosition() != oldCursorPos || markedText().length() != oldMarkLength )
               needReset = TRUE;
           }
           autoSelectOnMouseUp = FALSE;

      }

      void keyReleaseEvent( QKeyEvent *e ) {

                  QLineEdit::keyReleaseEvent( e );
          if ( ( e->state() && ShiftButton != 0 ) &&
             ( (e->key() == Key_Right ) || ( e->key() == Key_Left ) ) )
          {
            if ( cursorPosition() < fSelStart )
              setSelection( cursorPosition(), fSelEnd - cursorPosition() );
            else
              if ( cursorPosition() < fMark )
                setSelection( cursorPosition(), fMark - cursorPosition() );
              else
                setSelection( fSelStart, cursorPosition() - fSelStart );
          } else {
              if ( needReset & !( e->key() == Key_Tab || e->key() == Key_Backtab ))
                resetSelection();
          }
      }


public:
      QClxLineEdit( QWidget *parent, const char *name=0 )
            : QLineEdit( parent, name ) {
            fAutoSelect = FALSE;
            fHideSelection = TRUE;
            fSelStart = 0;
            fSelLength = 0;
            dragSelect = FALSE;
            autoSelectOnMouseUp = FALSE;
            QtSelect = TRUE;
            dontDeselect = FALSE;
      }

      bool autoSelect() { return fAutoSelect; }

      void setAutoSelect( bool value ) { fAutoSelect = value; }

      bool hideSelection() { return fHideSelection; }

      void setHideSelection( bool value ) { fHideSelection = value; }

      void clearSelection() {
        QString tmp = text();
        int cursorPos = cursorPosition();
        tmp.remove( fSelStart, fSelEnd - fSelStart );
        setText( tmp );
        setSelection( cursorPos, 0 );
      }

      void resetSelection() {
        fSelStart = cursorPosition();
        fSelEnd = fSelStart;
      }

      int selStart() { return fSelStart; };

      void setSelection( int start, int len ) {

        if ( QtSelect )
          QLineEdit::setSelection( start, len );
        if ( !dontDeselect )
        {
          fSelStart = start;
          if ( start + len > text().length() )
            len = text().length() - start;
          fSelEnd = start + len;
          if ( !len )
            fMark = fSelEnd;
        }
      }

      void setSelStart( int pos ) {
        QLineEdit::setCursorPosition( pos );
        fSelStart = pos;
        fSelEnd = pos;
        fMark = -1;
      }

      virtual void doSetSelection( int start, int len ) {
        setSelection( start, len );
      }

      int selLen() { return fSelEnd - fSelStart; };

      void setMarkedText( QString s ) {
        int start = selStart();
        if ( selLen() > 0 ) {
          QString tmp = text();
          tmp.replace( start, selLen(), s );
          setText( tmp );
          setCursorPosition( start + s.length() );
          setSelection( start, s.length() );
        }
        else {
          insert( s );
          setSelection( start, s.length() );
        }
      }

      void updateSelection() {
        fSelStart = cursorPosition();
        fSelEnd = fSelStart + QLineEdit::markedText().length();
      }

      bool hasMarkedText() const {
        return !markedText().isEmpty() || fSelEnd != fSelStart;
      }

      QString markedText() const {
          return text().mid( fSelStart, fSelEnd - fSelStart );
      }

};

#endif /* QCLXLINEEDIT_H */
