/*************************************************************************** 
                                                                             
 Kylix and Delphi Cross-Platform Visual Component Library                    
                                                                             
 Copyright (c) 2000, 2001 Borland Software Corporation                       
                                                                             
 This file may be distributed and/or modified under the terms of the GNU     
 General Public License (GPL) version 2 as published by the Free Software    
 Software Foundation and appearing at http://www.borland.com/kylix/gpl.html. 
                                                                             
 Licensees holding a valid Borland No-Nonsense License for this Software may 
 use this file in accordance with such license, which appears in the file    
 license.txt that came with this Software.                                   
                                                                             
****************************************************************************/

#ifndef qclxlistboxitem_H
#define qclxlistboxitem_H

#include <qlistbox.h>

#define ClxListBoxItem_selected 0x01;
#define ClxListBoxItem_grayed 0x02;
#define ClxListBoxItem_focused 0x04;

class QClxListBoxHooks {
  friend class QClxListBoxItem;
  public:
    QClxListBoxHooks() {
      measure_hook.func = NULL;
      paint_hook.func = NULL;
    }
    void hook_paint( QHook paintHook ) { paint_hook = paintHook; }
    void hook_measure( QHook measureHook ) { measure_hook = measureHook; }
  private:
    QHook measure_hook;
    QHook paint_hook;
};

class QClxListBoxItem : public QListBoxText {
  public:
    QClxListBoxItem( const QString & text, QClxListBoxHooks *listBox_hooks, void *d ) : QListBoxText( text ) {
      hooks = listBox_hooks;
      data = d;
    }

    int width( const QListBox *lb ) const {
      int w = QListBoxText::width( lb );
      int dummy = -1;
      getHeightAndWidth( dummy, w );
      return w;
    }

    int height( const QListBox *lb ) const {
      int h = QListBoxText::height( lb );
      int dummy = -1;
      getHeightAndWidth( h, dummy );
      return h;
    }
    
    void* getData() { return data; };
    void setData( void *d ) { data = d; };
    void setCustomHighlighting( bool value ) { QListBoxItem::setCustomHighlighting( value ); }

  protected:
    void paint( QPainter *p ) {
      bool handled = FALSE;
      if ( hooks->paint_hook.func ) {
        QListBox *lb = listBox();
        int index = lb->index( this );
        QRect r = lb->itemRect( this );
        int flags = 0;
        if ( selected() ) flags += ClxListBoxItem_selected;
        if ( !(lb->isEnabled()) ) flags += ClxListBoxItem_grayed;
        if ( current() ) flags += ClxListBoxItem_focused;
        typedef void (*func_type)(void *data, QListBoxH lb, QPainterH p, int index, QRectH r, int flags, bool &handled);
        (*(func_type)hooks->paint_hook.func)(hooks->paint_hook.data, QListBoxH(lb), (QPainterH)p, index, QRectH(&r), flags, handled);
      }
        
	  QListBoxText::setCustomHighlighting( handled );
      if ( !handled ) 
      	QListBoxText::paint( p );
    }
		
    void getHeightAndWidth( int &h, int &w ) const {		

      if ( hooks->measure_hook.func ) {
        typedef void (*func_type)(void *data, const QClxListBoxItem *i, int &h, int &w);
        (*(func_type)hooks->measure_hook.func)(hooks->measure_hook.data, this, h, w);
      }
    }
    
  private:
    QClxListBoxHooks *hooks;
    void *data;

};

#endif