/*************************************************************************** 
                                                                             
 Kylix and Delphi Cross-Platform Visual Component Library                    
                                                                             
 Copyright (c) 2000, 2001 Borland Software Corporation                       
                                                                             
 This file may be distributed and/or modified under the terms of the GNU     
 General Public License (GPL) version 2 as published by the Free Software    
 Software Foundation and appearing at http://www.borland.com/kylix/gpl.html. 
                                                                             
 Licensees holding a valid Borland No-Nonsense License for this Software may 
 use this file in accordance with such license, which appears in the file    
 license.txt that came with this Software.                                   
                                                                             
****************************************************************************/

#ifndef QCLXLISTVIEWITEM_H
#define QCLXLISTVIEWITEM_H

#include <qlistview.h>

/* callbacks into a QComCtrls.TCustomViewControl */
class QClxListViewHooks {
  friend class QClxListViewItem;
  friend class QClxCheckListItem;
  private:
    QHook paintCell_hook;
    QHook paintBranch_hook;
    QHook setSelected_hook;
    QHook change_hook;
    QHook changing_hook;
    QHook expanding_hook;
    QHook expanded_hook;
    QHook checked_hook;
    QHook destroyed_hook;
  public:                                                        
    QClxListViewHooks() {
      paintCell_hook.func = NULL;
      paintBranch_hook.func = NULL;
      setSelected_hook.func = NULL;
      change_hook.func = NULL;
      changing_hook.func = NULL;
      expanding_hook.func = NULL;
      expanded_hook.func = NULL;
      checked_hook.func = NULL;
      destroyed_hook.func = NULL;
    }
    void hook_PaintCell( QHook h ) { paintCell_hook = h; }
    void hook_PaintBranches( QHook h ) { paintBranch_hook = h; }
    void hook_setSelected( QHook h) { setSelected_hook = h; };
    void hook_change( QHook h) { change_hook = h; };
    void hook_changing( QHook h) { changing_hook = h; };
    void hook_expanding( QHook h) { expanding_hook = h; };
    void hook_expanded( QHook h) { expanded_hook = h; };
    void hook_checked( QHook h) { checked_hook = h; };
    void hook_destroyed( QHook h) { destroyed_hook = h; };
};

class QClxListViewItem : public QListViewItem {
  private:
  /* must match QComCtrls.TItemChange */
  typedef enum {ctText, ctImage, ctState} TItemChange;
 
  /* common function ptr declarations */
  typedef void (*TChangingFunc)(void *data, QListViewItemH Self, TItemChange type, bool *Allow);
  typedef void (*TChangeFunc)(void *data, QListViewItemH Self, TItemChange type);
  typedef void (*TExpandingFunc) (void *data, QListViewItemH Self, bool Expand, bool *Allow);
  typedef void (*TExpandedFunc) (void *data, QListViewItemH Self, bool Expand);
  typedef void (*TDestroyedHook)(void *data, QListViewItemH Self);  
    QClxListViewHooks *hooks;
  protected:
    void setSelected( bool Value)
    {
      typedef void (*func_type)(void *data, QListViewItemH Self, bool Value);
      bool Allow = true;

      if ((isSelected() != Value) && (!listView()->signalsBlocked()))
      {
        if ( hooks->changing_hook.func )
          (*(TChangingFunc)hooks->changing_hook.func)(hooks->changing_hook.data,
            (QListViewItemH)this, ctState, &Allow);

        if (Allow)
        {
          QListViewItem::setSelected(Value);
          if (hooks->setSelected_hook.func)
            (*(func_type)hooks->setSelected_hook.func)(hooks->setSelected_hook.data,
              (QListViewItemH)this, Value);
          if ( hooks->change_hook.func )
            (*(TChangeFunc)hooks->change_hook.func)(hooks->change_hook.data,
              (QListViewItemH)this, ctState);
        }
      }
      else
        QListViewItem::setSelected(Value);
    }

    void setOpen(bool Expanding)
    {
      bool Allow = true;

      if ((isOpen() != Expanding) && (!listView()->signalsBlocked()))
      { 
        if ( hooks->expanding_hook.func )
          (*(TExpandingFunc)hooks->expanding_hook.func)(hooks->expanding_hook.data,
            (QListViewItemH)this, Expanding, &Allow);

        if (Allow)
        {
          QListViewItem::setOpen(Expanding);
          if ( hooks->expanded_hook.func )
            (*(TExpandedFunc)hooks->expanded_hook.func)(hooks->expanded_hook.data,
              (QListViewItemH)this, Expanding);
        }
      }
      else
        QListViewItem::setOpen(Expanding);
    }

    void setText( int Col, const QString &Str)
    {
      bool Allow = true;

      if ((text(Col) != Str) && (!listView()->signalsBlocked()))
      {
        if ( hooks->changing_hook.func )
          (*(TChangingFunc)hooks->changing_hook.func)(hooks->changing_hook.data,
            (QListViewItemH)this, ctText, &Allow);

        if (Allow)
        {
          QListViewItem::setText(Col, Str);

          if ( hooks->change_hook.func )
            (*(TChangeFunc)hooks->change_hook.func)(hooks->change_hook.data,
              (QListViewItemH)this, ctText);
        }
      }
      else
        QListViewItem::setText(Col, Str);
    }

    void setPixmap( int Col, const QPixmap &Pix)
    {
      bool Allow = true;

      if ((pixmap(Col) != &Pix) && (!listView()->signalsBlocked()))
      {
        if ( hooks->changing_hook.func )
          (*(TChangingFunc)hooks->changing_hook.func)(hooks->changing_hook.data,
            (QListViewItemH)this, ctImage, &Allow);

        if (Allow)
        {
          QListViewItem::setPixmap(Col, Pix);

          if ( hooks->change_hook.func )
            (*(TChangeFunc)hooks->change_hook.func)(hooks->change_hook.data,
              (QListViewItemH)this, ctImage);
        }
      }
      else
        QListViewItem::setPixmap(Col, Pix);
    }

    void paintCell( QPainter *p, const QColorGroup &cg,
      int column, int width, int alignment )
    {
      typedef void (*func_type)(void *data, QPainterH p, QListViewItemH item, int column, int width, int alignment, DrawStage &stage);

      DrawStage stage = Pre;

      if ( hooks->paintCell_hook.func )
        (*(func_type)hooks->paintCell_hook.func)(hooks->paintCell_hook.data, (QPainterH)p, (QListViewItemH)this,
          column, width, alignment, stage );
      else
        stage = DefaultDraw;

      if (stage == DefaultDraw)
        QListViewItem::paintCell( p, cg, column, width, alignment );

      if ( hooks->paintCell_hook.func )
      {
        stage = Post;
        (*(func_type)hooks->paintCell_hook.func)(hooks->paintCell_hook.data, (QPainterH)p, (QListViewItemH)this,
          column, width, alignment, stage );
      }
    }
	
    void paintBranches( QPainter *p, const QColorGroup &cg, int w, int y, int h, GUIStyle s )
    {
      typedef void (*func_type)(void *data, QPainterH p, QListViewItemH item, int w, int y, int h, GUIStyle s, DrawStage &stage );

      DrawStage stage = Pre;
      if ( hooks->paintBranch_hook.func )
        (*(func_type)hooks->paintBranch_hook.func)( hooks->paintBranch_hook.data, (QPainterH)p, (QListViewItemH)this,
          w, y, h, s, stage );
      else
        stage = DefaultDraw;
	
      if ( stage == DefaultDraw )
        QListViewItem::paintBranches( p, cg, w, y, h, s );
    }
  public:
    /* constructor */
    QClxListViewItem( QListView *parent, QListViewItem *after, QClxListViewHooks *h ):
      QListViewItem( parent, after ) { hooks = h; }

    /* constructor */
    QClxListViewItem( QListViewItem *parent, QListViewItem *after, QClxListViewHooks *h ):
      QListViewItem( parent, after ) { hooks = h; }

    /* destructor */
    ~QClxListViewItem(void)
    {
      if ( hooks->destroyed_hook.func )
        (*(TDestroyedHook)hooks->destroyed_hook.func)( hooks->destroyed_hook.data,
        QListViewItemH(this));
    }
};

class QClxCheckListItem : public QCheckListItem {
  private:
  /* must match QComCtrls.TItemChange */
  typedef enum {ctText, ctImage, ctState} TItemChange;
 
  /* common function ptr declarations */
  typedef void (*TChangingFunc)(void *data, QListViewItemH Self, TItemChange type, bool *Allow);
  typedef void (*TChangeFunc)(void *data, QListViewItemH Self, TItemChange type);
  typedef void (*TExpandingFunc) (void *data, QListViewItemH Self, bool Expand, bool *Allow);
  typedef void (*TExpandedFunc) (void *data, QListViewItemH Self, bool Expand);
  typedef void (*TDestroyedHook)(void *data, QListViewItemH Self);
    QClxListViewHooks *hooks;
  protected:
    void setSelected( bool Value)
    {
      typedef void (*func_type)(void *data, QListViewItemH Self, bool Value);
      bool Allow = true;

      if ((isSelected() != Value) && (!listView()->signalsBlocked()))
      {
        if ( hooks->changing_hook.func )
          (*(TChangingFunc)hooks->changing_hook.func)(hooks->changing_hook.data,
            (QListViewItemH)this, ctState, &Allow);

        if (Allow)
        {
          QCheckListItem::setSelected(Value);
          if (hooks->setSelected_hook.func)
            (*(func_type)hooks->setSelected_hook.func)(hooks->setSelected_hook.data,
              (QListViewItemH)this, Value);
          if ( hooks->change_hook.func )
            (*(TChangeFunc)hooks->change_hook.func)(hooks->change_hook.data,
              (QListViewItemH)this, ctState);
        }
      }
      else
        QCheckListItem::setSelected(Value);
    }

    void setOn(bool Value)
    {
      bool Allow = true;

      QCheckListItem::setOn(Value);

      if (hooks->checked_hook.func)
        (*(TExpandedFunc)hooks->checked_hook.func)(hooks->checked_hook.data,
          (QListViewItemH)this, Value);
    }

    void setOpen(bool Expanding)
    {
      bool Allow = true;

      if ((isOpen() != Expanding) && (!listView()->signalsBlocked()))
      {
        if ( hooks->expanding_hook.func )
          (*(TExpandingFunc)hooks->expanding_hook.func)(hooks->expanding_hook.data,
            (QListViewItemH)this, Expanding, &Allow);

        if (Allow)
        {
          QCheckListItem::setOpen(Expanding);
          if ( hooks->expanded_hook.func )
            (*(TExpandedFunc)hooks->expanded_hook.func)(hooks->expanded_hook.data,
              (QListViewItemH)this, Expanding);
        }
      }
      else
        QCheckListItem::setOpen(Expanding);
    }

    void setText( int Col, const QString &Str)
    {
      bool Allow = true;

      if ((text(Col) != Str) && (!listView()->signalsBlocked()))
      {
        if ( hooks->changing_hook.func )
          (*(TChangingFunc)hooks->changing_hook.func)(hooks->changing_hook.data,
            (QListViewItemH)this, ctText, &Allow);

        if (Allow)
        {
          QCheckListItem::setText(Col, Str);
          if ( hooks->change_hook.func )
            (*(TChangeFunc)hooks->change_hook.func)(hooks->change_hook.data,
              (QListViewItemH)this, ctText);
        }
      }
      else
        QCheckListItem::setText(Col, Str);
    }

    void setPixmap( int Col, const QPixmap &Pix)
    {
      bool Allow = true;

      if ((pixmap(Col) != &Pix) && (!listView()->signalsBlocked()))
      {
        if ( hooks->changing_hook.func )
          (*(TChangingFunc)hooks->changing_hook.func)(hooks->changing_hook.data,
            (QListViewItemH)this, ctImage, &Allow);

        if (Allow)
        {
          QCheckListItem::setPixmap(Col, Pix);
          if ( hooks->change_hook.func )
            (*(TChangeFunc)hooks->change_hook.func)(hooks->change_hook.data,
              (QListViewItemH)this, ctImage);
        } 
      }
      else
        setPixmap(Col, Pix);
    }

    void paintCell( QPainter *p, const QColorGroup &cg,
      int column, int width, int alignment )
    {
      typedef void (*func_type)(void *data, QPainterH p, QCheckListItemH item, int column, int width, int alignment, DrawStage &stage);
      DrawStage stage = Pre;

      if ( hooks->paintCell_hook.func )
        (*(func_type)hooks->paintCell_hook.func)(hooks->paintCell_hook.data, (QPainterH)p, (QCheckListItemH)this,
	  column, width, alignment, stage );
      else
        stage = DefaultDraw;

      if ( stage == DefaultDraw )
        QCheckListItem::paintCell( p, cg, column, width, alignment );

      if ( hooks->paintCell_hook.func )
      {
        stage = Post;
	(*(func_type)hooks->paintCell_hook.func)(hooks->paintCell_hook.data, (QPainterH)p, (QCheckListItemH)this,
	  column, width, alignment, stage );
      }
    }

    void paintBranches( QPainter *p, const QColorGroup &cg, int w, int y, int h, GUIStyle s )
    {
      typedef void (*func_type)(void *data, QPainterH p, QCheckListItemH item, int w, int y, int h, GUIStyle s, DrawStage &stage );
      DrawStage stage = Pre;

      if ( hooks->paintBranch_hook.func )
        (*(func_type)hooks->paintBranch_hook.func)( hooks->paintBranch_hook.data, (QPainterH)p, (QCheckListItemH)this,
	  w, y, h, s, stage );
      else
        stage = DefaultDraw;

      if ( stage == DefaultDraw )
      QCheckListItem::paintBranches( p, cg, w, y, h, s );
    }

  public:
    /* constructor */
    QClxCheckListItem( QListView *parent, const QString &text, Type t, QClxListViewHooks *h ) :
      QCheckListItem( parent, text, t ) { hooks = h; }

    /* constructor */
    QClxCheckListItem( QListViewItem *parent, const QString &text, Type t, QClxListViewHooks *h ) :
      QCheckListItem( parent, text, t ) { hooks = h; }

    /* destructor */
    ~QClxCheckListItem(void)
    {
      if ( hooks->destroyed_hook.func )
        (*(TDestroyedHook)hooks->destroyed_hook.func)( hooks->destroyed_hook.data,
        QListViewItemH(this));
    }
};

#endif
