/*************************************************************************** 
                                                                             
 Kylix and Delphi Cross-Platform Visual Component Library                    
                                                                             
 Copyright (c) 2000, 2001 Borland Software Corporation                       
                                                                             
 This file may be distributed and/or modified under the terms of the GNU     
 General Public License (GPL) version 2 as published by the Free Software    
 Software Foundation and appearing at http://www.borland.com/kylix/gpl.html. 
                                                                             
 Licensees holding a valid Borland No-Nonsense License for this Software may 
 use this file in accordance with such license, which appears in the file    
 license.txt that came with this Software.                                   
                                                                             
****************************************************************************/

#ifndef QCLXMIMESOURCE_H
#define QCLXMIMESOURCE_H

#include <qmime.h>
#include <qstring.h>
#include <qmap.h>

class Q_EXPORT QClxMimeSource : public QMimeSource {
  private:
    typedef QMap<QString, QByteArray> QClxMap;
    QClxMap formatMap;
    QStrList *supportedFormats;
  public:
    QClxMimeSource() : QMimeSource(), supportedFormats(new QStrList()) { };
    QClxMimeSource(QMimeSource *source) : QMimeSource(),
      supportedFormats(new QStrList())
    {
      int i;
      QString format;
      QByteArray Temp;

      if (source)
      {
        i = 0;

        while (true)
        {
          format = source->format(i++);
          if (!format)
            break;
          Temp = source->encodedData(format);
          addFormat(format, &Temp);
        }
      }
    };
    ~QClxMimeSource() {
      delete supportedFormats;
    };
    virtual const char * format(int n) const {
      if ((n >= 0) && (n < supportedFormats->count()))
        return const_cast<QStrList*>(supportedFormats)->at(n);
      else
        return NULL;
    };
    virtual QByteArray encodedData(const char* format) const {
      if (provides(format))
        return formatMap[format];
      else
        return QByteArray();
    };

    virtual bool provides(const char *mimeType) const {
      return const_cast<QStrList*>(supportedFormats)->find(mimeType) != -1;
    };

    void addFormat(const char* format, QByteArray *ByteArray) {
      int index = supportedFormats->find(format); 
      if (index != -1)
      {
        supportedFormats->take(index);
        supportedFormats->insert(index, format);
      }
      else
        supportedFormats->append(format);
        
      formatMap[format] = *ByteArray;
    };
};

class Q_EXPORT QClxMimeSourceFactory : public QMimeSourceFactory {
  private:
    typedef void (*TDataCallback)(void *data, PWideString AbsName,
      QMimeSourceH &OutMimeSource);
    QHook FDataHook;
  public:
    /* constructor */
    QClxMimeSourceFactory() : QMimeSourceFactory() { FDataHook.func = NULL; };
    /* assign the callback */
    void setDataCallBack(QHook &hook)
    {
      FDataHook = hook;
    }
    /*
      data:
        calls into the CLX namespace to query user code for resolving mime-type
        data.  If the users CLX code does not provide data for the specified name,
        the default QMimeSourceFactory::data is called.
    */
    virtual const QMimeSource* data(const QString& abs_name) const
    {
      QMimeSource *Result = NULL;
      QMimeSourceH *Temp = reinterpret_cast<QMimeSourceH *>(&Result);

      if (FDataHook.func)
      {
        PWideString t_text;
        initializePWideString(t_text);
        copyQStringToPWideString(abs_name, t_text);
        (*(TDataCallback)FDataHook.func)(FDataHook.data, t_text,
          *Temp);
        finalizePWideString(t_text);
      }
      if (!Result)
        Result = const_cast<QMimeSource *>(QMimeSourceFactory::data(abs_name));
      return Result;
    };
};

#endif /* QCLXMIMESOURCE_H */
