/*************************************************************************** 
                                                                             
 Kylix and Delphi Cross-Platform Visual Component Library                    
                                                                             
 Copyright (c) 2000, 2001 Borland Software Corporation                       
                                                                             
 This file may be distributed and/or modified under the terms of the GNU     
 General Public License (GPL) version 2 as published by the Free Software    
 Software Foundation and appearing at http://www.borland.com/kylix/gpl.html. 
                                                                             
 Licensees holding a valid Borland No-Nonsense License for this Software may 
 use this file in accordance with such license, which appears in the file    
 license.txt that came with this Software.                                   
                                                                             
****************************************************************************/

#ifndef QCLXMOVIE_HOOK_H
#define QCLXMOVIE_HOOK_H

#include <qmovie.h>
#include "../qt_hook.h"

QHook nullHook = { 0, 0 };

class QMovie_hook : public QObject {
  Q_OBJECT
  public:
    QMovie_hook(QMovie *movie) : QObject(NULL) {
      this->movie = movie;
      status_event.func = NULL;
      update_event.func = NULL;
      resize_event.func = NULL;
    }

    ~QMovie_hook() {
      setMovie( 0 );
    }

    void hook_status(QHook &hook) {
      if ( !hook.func && movie ) {
        movie->disconnectStatus( this, SLOT(status_hook(int)) );
      }
      status_event = hook;
      if ( status_event.func && movie )
         movie->connectStatus( this, SLOT(status_hook(int)) );
    }

    void hook_resize(QHook &hook) {
      if ( !hook.func && movie )
        movie->disconnectResize( this, SLOT(resize_hook(const QSize&)) );
      resize_event = hook;
      if ( resize_event.func && movie )
        movie->connectResize( this, SLOT(resize_hook(const QSize&)) );
    }

    void hook_update(QHook &hook) {
      if ( !hook.func && movie )
        movie->disconnectUpdate( this, SLOT(update_hook(const QRect&)) );
      update_event = hook;
      if ( update_event.func && movie )
        movie->connectUpdate( this, SLOT(update_hook(const QRect&)) );
    }


    void setMovie( QMovie *m ) {
      if ( movie ) {
        hook_status( nullHook );
        hook_resize( nullHook );
        hook_update( nullHook );
        delete movie;
      }
      movie = m;
      //rehook all events
      if ( movie ) {
        hook_status( status_event );
        hook_resize( resize_event );
        hook_update( update_event );
      }
    }

  private slots:
    void status_hook( int status ) {
      if ( status_event.func ) {
        typedef void (*func_type)(void *data, int status );
	(*(func_type)status_event.func)(status_event.data, status);
      }
    }

    void resize_hook( const QSize &size ) {
      if ( resize_event.func ) {
        typedef void (*func_type)(void *data, int cx, int cy);
        (*(func_type)resize_event.func)(resize_event.data, size.width(), size.height());
      }
    }

   void update_hook( const QRect &r ) {
      if ( update_event.func ) {
        typedef void (*func_type)(void *data, int x, int y, int w, int h);
        (*(func_type)update_event.func)(update_event.data, r.left(), r.top(),
          r.right(), r.bottom());
      }
    }

  private:
    QMovie *movie;
    QHook status_event;
    QHook resize_event;
    QHook update_event;
};

#include "qclxmovie_hook.moc"

#endif