/*************************************************************************** 
                                                                             
 Kylix and Delphi Cross-Platform Visual Component Library                    
                                                                             
 Copyright (c) 2000, 2001 Borland Software Corporation                       
                                                                             
 This file may be distributed and/or modified under the terms of the GNU     
 General Public License (GPL) version 2 as published by the Free Software    
 Software Foundation and appearing at http://www.borland.com/kylix/gpl.html. 
                                                                             
 Licensees holding a valid Borland No-Nonsense License for this Software may 
 use this file in accordance with such license, which appears in the file    
 license.txt that came with this Software.                                   
                                                                             
****************************************************************************/

#ifndef QOpenMultiLineEdit_H
#define QOpenMultiLineEdit_H

#include <qmultilineedit.h>

static int defTabStop = 8;

static int tabStopDist( const QFontMetrics &fm )
{
    return defTabStop*fm.width( QChar('x') );
}


static int textWidthWithTabs( const QFontMetrics &fm, const QString &s, uint start, uint nChars, int align )
{
    if ( s.isEmpty() )
	return 0;

    int dist = -fm.leftBearing( s[(int)start] );
    int i = start;
    int tabDist = -1; // lazy eval
    while ( (uint)i < s.length() && (uint)i < start+nChars ) {
	if ( s[i] == '\t' && align == Qt::AlignLeft ) {
	    if ( tabDist<0 )
		tabDist = tabStopDist(fm);
	    dist = ( (dist+tabDist+1)/tabDist ) * tabDist;
	    i++;
	} else {
	    int ii = i;
	    while ( (uint)i < s.length() && (uint)i < start + nChars && ( align != Qt::AlignLeft || s[i] != '\t' ) )
		i++;
	    dist += fm.width( s.mid(ii,i-ii) );
	}
    }
    return dist;
}

static int xPosToCursorPos( const QString &s, const QFontMetrics &fm,
			    int xPos, int width, int align )
{
    int i = 0;
    int	  dist;
    int tabDist;

    if ( s.isEmpty() )
	return 0;
    if ( xPos > width )
	xPos = width;
    if ( xPos <= 0 )
	return 0;

    dist    = -fm.leftBearing( s[0] );

    if ( align == Qt::AlignCenter || align == Qt::AlignHCenter )
	dist = ( width - textWidthWithTabs( fm, s, 0, s.length(), align ) ) / 2;
    else if ( align == Qt::AlignRight )
	dist = width - textWidthWithTabs( fm, s, 0, s.length(), align );

    int     distBeforeLastTab = dist;
    tabDist = tabStopDist(fm);
    while ( (uint)i < s.length() && dist < xPos ) {
	if ( s[i] == '\t' && align == Qt::AlignLeft ) {
	    distBeforeLastTab = dist;
	    dist = (dist/tabDist + 1) * tabDist;
	} else {
	    dist += fm.width( s[i] );
	}
	i++;
    }
    if ( dist > xPos ) {
	if ( dist > width ) {
	    i--;
	} else {
	    if ( s[i-1] == '\t' && align == Qt::AlignLeft ) { // dist equals a tab stop position
		if ( xPos - distBeforeLastTab < (dist - distBeforeLastTab)/2 )
		    i--;
	    } else {
		if ( fm.width(s[i-1])/2 < dist-xPos )
		    i--;
	    }
	}
    }
    return i;
}

class QOpenMultiLineEdit : public QMultiLineEdit {
  private:
    int extraMarg() const {
      return wrapPolicy() == Anywhere && wordWrap() != NoWrap ? 8 : 0;
    }
    
  public:
    QOpenMultiLineEdit( QWidget *parent=0, const char *name=0 )
      : QMultiLineEdit( parent, name ) { }

    bool hasMarkedText() const {
      return QMultiLineEdit::hasMarkedText();
    }

    QString markedText() const {
      return QMultiLineEdit::markedText();
    }

    int	textWidth( int i ) {
      return QMultiLineEdit::textWidth( i );
    }

    int	textWidth( const QString &text) {
      return QMultiLineEdit::textWidth( text );
    }

    QPoint cursorPoint() const {
      return QMultiLineEdit::cursorPoint();
    }

    virtual void insert( const QString &text, bool mark ) {
      QMultiLineEdit::insert( text, mark );
    }                                                           

    virtual void newLine() {
      QMultiLineEdit::newLine();
    }

    virtual void killLine() {
      QMultiLineEdit::killLine();
    }

    virtual void pageUp( bool mark=FALSE ) {
      QMultiLineEdit::pageUp( mark );
    }

    virtual void pageDown( bool mark=FALSE ){
      QMultiLineEdit::pageDown( mark );
    }

    virtual void cursorLeft( bool mark=FALSE, bool wrap = TRUE ){
      QMultiLineEdit::cursorLeft( mark, wrap );
    }

    virtual void cursorRight( bool mark=FALSE, bool wrap = TRUE ){
      QMultiLineEdit::cursorRight( mark, wrap );
    }

    virtual void cursorUp( bool mark=FALSE ){
      QMultiLineEdit::cursorUp( mark );
    }

    virtual void cursorDown( bool mark=FALSE ){
      QMultiLineEdit::cursorDown( mark );
    }

    virtual void backspace(){
      QMultiLineEdit::backspace();
    }

    virtual void del(){
      QMultiLineEdit::del();
    }

    virtual void home( bool mark=FALSE ){
      QMultiLineEdit::home( mark );
    }

    virtual void end( bool mark=FALSE ){
      QMultiLineEdit::end( mark );
    }

    bool getMarkedRegion( int *line1, int *col1,
                          int *line2, int *col2 ) const{
      return QMultiLineEdit::getMarkedRegion( line1, col1, line2, col2 );
    }

    int	lineLength( int row ) const{
      return QMultiLineEdit::lineLength( row );
    }

    QString *getString( int row ) const{
      return QMultiLineEdit::getString( row );
    }

    QString stringShown( int row ) const{
      return QMultiLineEdit::stringShown( row );
    }

    int textLength( void ) {
      return getText().length();
    }

    bool isEndOfParagraph( int row ) const {
      return QMultiLineEdit::isEndOfParagraph( row );
    }

    QString getText() const
    {
      QString tmp;
      for( int i = 0 ; i < numLines() ; i++ ) {
	tmp += textLine(i);
        if ( isEndOfParagraph( i ) )
          tmp += "\n";

      }
      return tmp;
    }

    int search( QString text, int line, int col, bool caseSens, bool highlight,
      bool wholeword, bool useRegExp ) {
       if ( text.isEmpty() ) return -2;
       QRegExp expr;
       if ( useRegExp ) {
          expr = QRegExp( text, caseSens, FALSE );
 	  if ( !expr.isValid() ) return -3;
       }
//       if ( forward ) {
          int i;
	  for ( i = line; i < numLines(); i++ ) {
	      QString s = textLine( i );
              // to enable the match to occur across possibly wrapped lines
	      int wlines = 0;
              while ( !isEndOfParagraph( i + wlines ) && i + wlines < numLines() ) {
	        wlines++;
		s += textLine( i + wlines );
	      }

	      int pos = -1;
	      if ( useRegExp ) {
	        pos = s.find( expr, i == line ? col : 0 );
	      }
	      else
	        pos = s.find( text, i == line ? col : 0, caseSens );

	      if ( pos != -1 ) {
                  // check offset into entire string which may be more than
                  // one logical line; fixup row and match position.
		  int j = 0;
		  if ( wlines > 0 ) {
	             while ( pos > j + lineLength( i ) ) {
		        j += lineLength( i );
		        i++;
		     }
		  }

                  int matchLen = 0;
                  if ( useRegExp ) {
                     expr.match( s, pos, &matchLen, FALSE );
                  }
                  else
                     matchLen = text.length();

                  //check for whole word boundary
                  if ( wholeword ) {
                     bool leftwb = ( pos == 0 ) | ( s[ pos ].isSpace() );
                     bool rightwb = ( pos + matchLen == s.length() ) 
		       | ( s[ pos  + matchLen ].isSpace() );
                     if ( !( leftwb & rightwb ) ) return -1;
                  }

                  setCursorPosition( i, pos - j, FALSE );
                  if ( highlight ) {
                     // for regular expressions, the match may wrap onto the
                     // next line.
                     if ( pos - j + matchLen > lineLength( i ) ) {
                        pos = ( pos - j + matchLen ) - lineLength( i );
                        i++;
                        setCursorPosition( i, pos, TRUE );
                     }
                     else
                       setCursorPosition( i, pos - j + matchLen, TRUE );
                  }
		  return 0;
	      }
	      else
	        i += wlines;

	  }
       //}
       return -1;
    }

    void pixelPosToCursorPos(QPoint p, int* x, int* y) const
    {
        *y = findRow( p.y() );
        if ( *y < 0 ) {
            if ( p.y() < lineWidth() ) {
                *y = topCell();
            } else {
                *y = lastRowVisible();
                p.setX(cellWidth());
            }
        }
//        *y = QMIN( (int)contents->count() - 1, *y );
        *y = QMIN( (int)numLines() - 1, *y );
        QFontMetrics fm( font() );
        *x = xPosToCursorPos( stringShown( *y ), fm,
                              p.x() - hMargin() + xOffset(),
                              cellWidth() - 2 * hMargin() -

                              extraMarg(),

//                              d->marg_extra,
                              alignment() );
//        QMultiLineEditRow* r = contents->at( *y );
//        if (r && !r->newline && ((unsigned int)*x == r->s.length()) && (*x > 0))
//            --*x;
    }


    uint	tableFlags() const { return QTableView::tableFlags(); }
    bool	testTableFlags( uint f ) const { return QTableView::testTableFlags( f ); }
    void 	setTableFlags( uint f ) { QTableView::setTableFlags( f ); }
    void	clearTableFlags( uint f = ~0 ) { QTableView::clearTableFlags( f ); }

    enum 	TableFlags {
    			vScrollBar = Tbl_vScrollBar,
				hScrollBar	= Tbl_hScrollBar,
				autoVScrollBar	= Tbl_autoVScrollBar,
				autoHScrollBar	= Tbl_autoHScrollBar,
				autoScrollBars	= Tbl_autoScrollBars,

			    clipCellPainting = Tbl_clipCellPainting,
				cutCellsV	= Tbl_cutCellsV,
				cutCellsH	= Tbl_cutCellsH,
                cutCells	= Tbl_cutCells,

				scrollLastHCell	= Tbl_scrollLastHCell,
				scrollLastVCell	= Tbl_scrollLastVCell,
				scrollLastCell	= Tbl_scrollLastCell,

				smoothHScrolling = Tbl_smoothHScrolling,
				smoothVScrolling = Tbl_smoothVScrolling,
				smoothScrolling	= Tbl_smoothScrolling,
				snapToHGrid	= Tbl_snapToHGrid,
				snapToVGrid	= Tbl_snapToVGrid,
				snapToGrid	= Tbl_snapToGrid
			};
};

#endif
