#ifndef QOPENSCROLLVIEW_H
#define QOPENSCROLLVIEW_H

#include <qscrollview.h>
#include <qpalette.h>
#include <qregion.h>

class Q_EXPORT QOpenScrollView : public QScrollView {
  private:
    QHook sbg_hook;
    int oldW, oldH;
    bool updatingValues;
    QRegion ViewportClipRegion;
  public:
    QOpenScrollView( QHook scrollBarGeometry_hook, QWidget *parent=0, const char *name=0, WFlags f=0 )
      : QScrollView(parent, name, f)
    {
    	sbg_hook = scrollBarGeometry_hook;
        oldW = 0;
        oldH = 0;
        updatingValues = FALSE;
    };
	/*
	void drawContentsOffset( QPainter *p, int ox, int oy, int cx, int cy, int cw, int ch )
	{
		insert callback code
	}
	void drawContents( QPainter *p, int cx, int cy, int cw, int ch )
	{
		insert callback code
	}
	*/

        void resizeContents( int w, int h )
        {
           if ( updatingValues ) return;
           int vAdj = oldH - h;
           int hAdj = oldW - w;
           QScrollView::resizeContents( w, h );
           updatingValues = TRUE;
           verticalScrollBar()->setValue( verticalScrollBar()->value() + vAdj );
           horizontalScrollBar()->setValue( horizontalScrollBar()->value() + vAdj );
           updatingValues = FALSE;
           oldH = h;
           oldW = w;
        }
	
	void setVBarGeometry( QScrollBar &vbar, int x, int y, int w, int h )
	{
        y += topMargin();
        h -= topMargin();
		getScrollBarGeometry( vbar, x, y, w, h );
		QScrollView::setVBarGeometry( vbar, x, y, w, h );
	}
	
	void setHBarGeometry( QScrollBar &hbar, int x, int y, int w, int h )
	{
		getScrollBarGeometry( hbar, x, y, w, h );
		QScrollView::setHBarGeometry( hbar, x, y, w, h );
	}
	
	void getScrollBarGeometry( QScrollBar &sb, int &x, int &y, int &w, int &h ) 
	{
		if ( sbg_hook.func )
		{
			typedef void (*func_type)( void *data, QScrollBarH sb, int &x, int &y, int &w, int &h );
			(*(func_type)sbg_hook.func)( sbg_hook.data, (QScrollBarH)&sb, x, y, w, h );
		}
	}
	
	void setMargins( int left, int top, int right, int bottom )
	{
		QScrollView::setMargins( left, top, right, bottom );
	}
	int leftMargin() const
	{
		return QScrollView::leftMargin();
	}
	int topMargin() const
	{
		return QScrollView::topMargin();
	}
	int rightMargin() const
	{
		return QScrollView::rightMargin();
	}
	int bottomMargin() const
	{
		return QScrollView::bottomMargin();
	}
    protected:
        void viewportPaintEvent( QPaintEvent* pe )
        {
          ViewportClipRegion = pe->region();
          QScrollView::viewportPaintEvent(pe);
        }
        void drawContents(QPainter* p, int cx, int cy, int cw, int ch)
        {
           QPixmap *bg;

           p->setClipRegion(ViewportClipRegion);
           bg = viewport()->palette().brush(QPalette::Active,
                QColorGroup::Background).pixmap();
           if ( bg && !bg->isNull() ) {
             int rowheight=bg->height();
             int toprow=cy/rowheight;
             int bottomrow=(cy+ch+rowheight-1)/rowheight;
             int colwidth=bg->width();
             int leftcol=cx/colwidth;
             int rightcol=(cx+cw+colwidth-1)/colwidth;
             for (int r=toprow; r<=bottomrow; r++) {
                 int py=r*rowheight;
                 for (int c=leftcol; c<=rightcol; c++) {
                     int px=c*colwidth;
                     p->drawPixmap(px, py, *bg);
                 }
             }
           }
           QScrollView::drawContents( p, cx, cy, cw, ch );
        }
};

#endif
